c     program eotbias-cor4-doc.f
c
c$  MAIN PROGRAM DOCUMENTATION BLOCK
c                .      .    .                                       .
c MAIN PROGRAM: eotbias-cor4-doc.f  COMPUTES bias error and bias correction
c   PRGMMR: R.W. REYNOLDS   DATE: 2009-10-20
c
c ABSTRACT: eotbias-cor4-doc reads the weights and zonal adjustments computed in 
c eotbias-wt4-doc.f from 1 or more days. The weights from several days
c are combined by multiplying by smoothing factors (dfw1, dfw1-dfw3, or
c dfw1-dfw5) where the number of smoothing factors is nuse (max value = 9). 
c Note that the sum of the dwf's used must equal 1.
c If any EOT weights are missing the program will 
c adjust the sum of the dwf's available to equal 1. If this is done a 
c message is printed. Smoothed satellite and in situ anomalies are then
c computed. The zonal corrects to the satellite anomalies are smoothed by the
c same smoothing weights and the satellite anomalies are readjusted.
c The difference between the satellite and in situ anomalies is now computed 
c as the bias and stored in  variables bias and biasp. Biasp is the same as bias 
c except that biasp includes missing values and is extended in longtiude
c for interpolation. The bias error is computed based on modes used
c and written out. In the final step the satellite data are
c read in, adjusted by biasp and written out.
c
c   Details of the bias correction procedures including the bias error
c  can be found in:
c   Reynolds, R. W., T. M. Smith, C. Liu, D. B. Chelton, K. S. Casey, 
c     and M. G. Schlax, 2007: Daily high-resolution blended analyses 
c     for sea surface temperature. J. Climate, 20, 5473-5496.
c
C   NOTE: 
C         OI    =  OPTIMUM INTERPOLATION.
C         SST   =  SEA SURFACE TEMPERATURE
C         EOT   =  empirical orthogonal teleconnections
c     in situ   =  ship and buoy
c
C
C PROGRAM HISTORY LOG:
c   2009-10-20  Richard W. Reynolds
c
c   Input Data on unit 5 needed by program (input from script)
c
c   The 1st eotcor4 input record has 2 parameters: nmodes, nuse 
c number of modes used; can't be larger than 130: e.g.,  nmodes=130	        
c number of factors to smooth weights from different days: nuse should be 1,3 or 5
c   The 2nd eotcor4 input record has 1-9 parameters depending on nuse
c dfw1-dfw9. The sum of these dfw factors must equal 1. 
c The binomial weight for nuse=1 would be 1.0
c Binomial weights for nuse=3 would be 0.25, 0.5, 0.25
c Binomial weights for nuse=5 would be 0.0625, 0.25, 0.375, 0.25, 0.0625
c  The next input record is the date: computed by script 8 digits
c    in the form: e.g.,20031231 for 31 December 2003			 
c
c  All remaining records are input and output file names which must be in order
c  The 1st file name is the two-degree land/sea mask
c  e.g., MASK2D=/raid/rreynold/sstwkly/clm/lstags.twodeg.dat
c  The 2nd file name is the input EOT modes (filled over land with 0)
c  e.g., FMODE=/raid2/sstday/eot/eot6.damp-zero.ev130.ano.dat 
c  The 3rd file name is the variance of each of the 130 modes
c  e.g., VARMD=/raid/rreynold/sstday/stats/var-mode
c  Nuse now determines the number of files read. They will be weighted by dwf
c  e.g., WTSDaym2=/raid2/sstday/$YEARm2/eotwt-finv2/path-err/dsat/pwtday.$TMm2
c        WTSDaym1=/raid2/sstday/$YEARm1/eotwt-finv2/path-err/dsat/pwtday.$TMm1
c        WTSDay=/raid2/sstday/$YEAR/eotwt-finv2/path-err/dsat/pwtday.$TM
c        WTSDayp1=/raid2/sstday/$YEARp1/eotwt-finv2/path-err/dsat/pwtday.$TMp1
c        WTSDayp2=/raid2/sstday/$YEARp2/eotwt-finv2/path-err/dsat/pwtday.$TMp2
c        where $TMm2=$Tm - 2, $TMm1=$Tm - 1,$TMp2=$Tp + 2, $TMp1=$Tm + 1
c        The weighting smoothing factors are applied in order: dwf1, ...
c  The next file name is the input uncorrected satellite data
c  e.g., SATDay=/raid2/sstday/$YEAR/sobs/path/dsat/pday.$TM
c  The next file name is the output bias error estimate 
c  e.g., ANALDay=/raid2/sstday/$YEAR/eotcor-finv2/path-err/dsat/pday.$TM     
c  The final file name is the output bias corrected satellite data
c  e.g., SATCORDay=/raid2/sstday/$YEAR/cobs/eotcor-finv2/path-err/dsat/pday.$TM
c
c  Fortran files: only units 6 and 51, 52 are output files
c   Files with TM ($TM) in name vary with time, the others are static
c     variable unit    name
c              5       input data parameters and file names
c              6       print out
c
c     iu       2       2-degree mask, MASK2D
c     iu       4       130 modes, FMODE
c     iu       9       Expected variance of the 130 modes
c     iu       15      Input EOT weights and zonal bias
c     iusat    13      uncorrected satellite data, SATDay 
c
c     iub      51      bias, situ and satellite anomalies and bias error
c                      ANALDay - output                  
c     iun      53      bias corrected satellite data, SATCORDay - output
c
C     PARAMETERS in Main:
c      parameter (imx=180,imy=90,max=130,ntyp=2,ndfw=9,
c     1 iadd=1,imxp=imx+2*iadd,imin=iadd+1,imax=iadd+imx)
c              - IMX IS LONGITUDE DIMENSION OF analysis (1 to 359E)
C              - IMX = 180  (2-degree grid)                            
C              - IMY IS LATITUDE DIMENSION OF analysis  (-89 to 89)
C                IMY =  90   (2-degree grid)            (89S to 89N)
c     Larger grid, for interpolation in longitude (182,92)
c              - iadd =1, Number of extra longitude points on each side of grid
c              - IMxp LONGITUDE DIMENSION OF larger fields   (-1 to 361 )
C              - IMxp = 182  for iadd =1                     (1W TO 361E or 1E)
c              - imin = 2,  start locations in larger field of original
c              - imax = 181, end  locations in larger field of original
c              - so first and last points of larger field wrap around
c              - e.g., dimension ls(imx,imy),lsp(imxp,imy)
c              - e.g., lsp(2,j) = ls(1,j); lsp(181,j) = ls(180,j)
c              - e.g., lsp(1,j) = lsp(181,j); lsp(182,j) = lsp(2,j)
c              - ntype=2 is the number of anomalies (in situ + satellite)
c              - max = 130,the number of EOT modes
c              - ndfw = 9, the maximum number of smoothing factors
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - getwt, gengrd, intrp1, ptbias, verr, getmsk, getmod, 
c                getvar, maxmin, mxmn3, biascr, fdbias, gtbias
C
C   EXIT STATES: There are 3 main categories
c   call exit (1) : Normal End, code color 'green'
c
c   call exit (10): Normal End with WARNING, code color 'yellow'
c                   The problem should be corrected and the code rerun.
c   The following warning 'yellow' errors may occur 
c       eotbias-cor4-main: Missing Weights
c       eotbias-cor4-main: NO SATELLITE DATA CORRECTED
c
c   call exit (100): Fatal error, Premature STOP, code color 'red'
c                    The problem MUST be corrected and the code rerun.
c   The following fatal 'red' errors may occur 
c       eotbias-cor4-biascr: premature end of satellite input data
c       eotbias-cor4-main: INPUT/OUTPUT DATA MISSING
c       eotbias-cor4-main: no EOT weights
c       eotbias-cor4-main: NO LAND SEA MASK 
c       eotbias-cor4-main: Spatial modes missing
c       eotbias-cor4-varmod: Mode variance missing
c       eotbias-cor4-getwt: Weights Bad
c       eotbias-cor4-getmod: error in getmod
c       eotbias-cor4-getvar: error in getvar
c     
c remarks:  
c       the analysis grid is 2-deg 
c  Unless specifically defined in this program and subroutines,
c      Scalars and arrays starting with letters A-H, and O-Z are real*4 
c      Scalars and arrays starting with letters I-N are integer*4 
c
c   note the zonal correction is removed from the bias and sat anomaly
c   following the call to fdbias in the main program
c
c common block : jrec in subroutine biascr: variables are stored obs
c                data has been written in blocks of up to 10000
c     rsum(10000) - number of obs
c     rsst(10000) - sst 
c     rlon(10000) - longitude 
c     rlat(10000) - latitude            
c                                                                   
c     - VARIABLES FOLLOW:
c
c  Input scalar variables from input lines (see above)
c
c  nmodes  - number of modes used; can't be larger than 130
c  nuse    - number of factors to smooth weights from different days, 
c            maximum value is 9
c
c Other IMPORTANT scalar variables
c     iyr      - year of analysis (4 digits)
c     imo      - month of analysis (2 digits)
c     ida      - day of analysis (2 digits)
c     ndays    - number of days used in computing anomalies
c     deg      - grid interval in degrees = 2
c     undef    - indicates variable is undefined = -9999.
c     ndfw     - the maximum number of smoothing factors = 9
c
c  Important character variables
c     fdate*8     - date computed by script 8 digits
c     fls*80      - file name for land/sea table
c     fmode*80    - file name for 130 EOT modes
c     fscor*80    - file name for bias corrected satellite data -output
c     fsat*80     - file name for uncorrected satellite data
c     fvarmd*80   - file name for variance of 130 EOT modes
c     fanal*80    - file name analysis: bias, situ & sat anom, bias error
c                      - output
c      fwt*80     - file name for weights 
c                 - File names with full path, may not be larger than 80
c      
c     Important numerical arrays: 
c      zon(imy,ndwf)       - smoothed zonal bias 
c      wk(imx,imy)    - work array
c      ls(imx,imy)    - land sea mask (0 ocean, 1 land) from 1-deg mask
c      lsp(imxp,imy)  - longitude expanded land sea mask
c      wtmk(imx,imy)  - bias weights to eliminate any sat bias computed 
c                       in high latitudes, plus Caspian Sea and Hudson's Bay
c      smode(imx,imy,max)
c                     - EOT modes, max = 130 the number of modes
c      xt(imx)        - longitude (E) points: 1, 3, 5, .... 359
c      yt(imy)        - latitude points: -89, -87, ...., 89
c      ccos(imy)      - cosine of latitudes for yt and other (lon,lat) arrays
c      xtp(imxp)      - longitude expanded: -1, 1, 3, 5, .... 359, 361
c      bias(imx,imy)  - bias correction: (satellite - in situ)
c      biasp(imxp,imy)- bias correction, longitude extended, 
c                       interpolated over land
c      xsat(imx,imy)  - average satellite anomalies for ndays
c      situ(imx,imy)  - average weighted ship and buoy anomalies
c      dfw(ndfw)      - weights of smoothing factors
c      wt(max,ntyp,ndfw)
c                     - EOT weights (=0 if not used), 2-D
c      wf(max)        - EOT weights,- only modes accepted, later moved to wt, 1-D
c                       linear solution code required only 1-dimension
c      ft(imx,imy,ntyp) 
c                     - final anomaly field 
c      varbs(imx,imy) - variance bias error - output
c      varmod(max)    - variance for each EOT mode
c      iuse(max,ndfw) - modes supported by satellite and/or in situ data
c                       determined by checking data using crit in eotbias-wt4
c                 iuse is 0 if NOT supported by in situ or satellite
c                 iuse is 1 if supported by in situ but NOT by satellite
c                 iuse is 2 if NOT supported by in situ but supported by satellite
c                 iuse is 3 if supported by in situ and by satellite
c                 iuse must be = 3 to be used in fit
c
c ATTRIBUTES:
c   LANGUAGE: FORTRAN 77 
c             
c   MACHINE:  Linux: warmcore/rainband
c
c$
c
c         max ...........number of EOT modes,
c         ntyp...........here 2 types: satellite and in situ.
c
c
c smooth the weights using up to 9 sets of weights
c
      parameter (imx=180,imy=90,max=130,ntyp=2,ndfw=9,
     1 iadd=1,imxp=imx+2*iadd,imin=iadd+1,imax=iadd+imx)
      dimension zon(imy,ndfw),gsum(imy)
      dimension wk(imx,imy),wtmk(imx,imy)
      dimension smode(imx,imy,max)
      dimension ls(imx,imy),lsp(imxp,imy)
      dimension xt(imx),yt(imy),ccos(imy),xtp(imxp)
      dimension biasp(imxp,imy),dfw(ndfw)
      dimension bias(imx,imy),varbs(imx,imy)
      dimension xsat(imx,imy)
      dimension situ(imx,imy)
      dimension ft(imx,imy,ntyp)
c     bias has land/sea mask applied
c     biasp is defined everywhere with extra lon points for interpolation
      dimension wt(max,ntyp,ndfw)
      dimension iuse(max,ndfw),varmod(max)
c
      character*80 fls,fmode,fscor,fwt,fsat,fvarmd,fanal
      character*8  fdate 
c      unit 2 is 2-degree mask
c      unit 4 is 130 modes
c      unit 9 is variance of 130 modes
c      unit 13 is input satellite data
c      unit 15 is weights
c      unit 51 is bias (satellite - in situ)
c      unit 52 is corrected satellite data
c
      logical lexist
c
c     input data: order is critical
c
c23456
c
c
c
      call timer (itime1)
c
c
      undef = -9999.
c
      read *,nmodes,nuse 
      print 118,nmodes,nuse
  118 format ('max modes used =',i4,3x,'nuse =',i3)
c      if (nuse.ne.1.and.nuse.ne.3.and.nuse.ne.5) then
c        PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: bad value of nuse  '
c        call exit (100)
c      endif
      read *,(dfw(ll),ll=1,nuse)
      print 112,(dfw(ll),ll=1,nuse)
  112 format ('dfw =',5f10.4/5x,4f10.4)
      sum = 0.
      do ll=1,nuse
        sum = sum + dfw(ll)
      enddo
      print 113,sum
  113 format ('dfw sum =',f10.4)
c      if (abs(sum-1.).gt.0.001) then
c        PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: sum weights not 1 '
c        call exit (100)
c      endif
      read 12,fdate
   12 format (a8)
      read (fdate,11) iyr,imo,ida 
   11 format (i4,i2,i2)
      print 17,iyr,imo,ida
   17 format ('analaysis date: ',i5,i3,i3)
      read 13,fls
      print *,'unit 2  '
      print *,fls
      read 13,fmode
      print *,'unit 4   '
      print *,fmode
      read 13,fvarmd
      print *,'unit 9   '
      print *,fvarmd
      read 13,fsat
      print *,'unit 13  '
      print *,fsat
      iu = 15 
      iwt = 0
      do ll=1,nuse
        read 13,fwt
        print *,'unit 15  '
        print *,fwt
        inquire(file=fwt,exist=lexist)
c       print *,lexist
        if(.not.lexist) then
           PRINT *,'YELLOW - WARNING - EOTBIAS-cor4-MAIN:',
     1 ' Missing Weights'
          iwt = 1
	  dfw(ll) = 0.
	  do j=1,imy
	    zon(j,ll) = 0.
	  enddo
	  do i=1,max
	    iuse(i,ll) = 0
	    do j=1,ntyp
	      wt(i,j,ll) = 0.
	    enddo
	  enddo
        else
          open (15,err=900,file=fwt,form='unformatted')
c
          call getwt (iu,iuse ,wt ,ll, ndfw ,max,ntyp,iwerr, zon,imy)
c	  do m=1,10
c	    print *,ll,iuse(m,ll),wt(m,1,ll),wt(m,2,ll)
c          enddo
          call mxmn3 (wt ,max,ntyp,ndfw,ll,undef,'fit wt   ')
	  if (iwerr.ne.0) go to 1022
        endif
      enddo
      sum = 0.
      do ll=1,nuse
        sum = sum + dfw(ll)
      enddo
      print 113,sum
      if (abs(sum).lt.0.001) then
        PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: no EOT weights'
        call exit (100)
      endif
      if (abs(sum-1.).gt.0.001) then
        PRINT *,' sum of weights adjusted to 1 '
c        print 112,(dfw(ll),ll=1,nuse)
        do ll=1,nuse
          dfw(ll) = dfw(ll)/sum
        enddo
c        print 112,(dfw(ll),ll=1,nuse)
        sum = 0.
	do ll=1,nuse
          sum = sum + dfw(ll)
        enddo
        print 113,sum
      endif
      read 13,fanal
      print *,'unit 51  '
      print *,fanal
      read 13,fscor
      print *,'unit 52  '
      print *,fscor
   13 format (a80)
c
c      print *, 'open  ',fls
      inquire(file=fls,exist=lexist)
c     print *,lexist
      if(.not.lexist) then
        print *,'no land sea'
	print *,' '
        go to 1016 
      endif  
c
      print *,'unit 2 land/sea  '
      open (2,err=900,file=fls,form='unformatted')
      print *,'unit 4 eot modes  '
      open (4,err=900,file=fmode,form='unformatted')
      print *,'unit 9 eot varianaces '
      open (9,err=900,file=fvarmd,form='unformatted')
      print *,'unit 13 input satellite data'
      open (13,err=900,file=fsat,form='unformatted')
c
      print *,'unit 51 analysis bias'
      open (51,err=900,file=fanal,form='unformatted')
      print *,'unit 52 corrected satellite data'
      open (52,err=900,file=fscor,form='unformatted')
c
      deg = 2.
      call gengrd (imx,imxp,imy,iadd,xt,yt,xtp,ccos,deg)
      print 111,xt(1),xt(imx),xtp(1),xtp(imxp),yt(1),yt(imy),
     1 ccos(1),ccos(imy)
  111 format ('min xt (lon) =',f8.1,3x,'max xt (lon) =',f8.1/
     1 'min xtp (lon-p) =',f8.1,3x,'max xtp (lon-p) =',f8.1/
     2 'min yt (lat) =',f8.1,3x,'max yt (lat) =',f8.1/
     2 'min cos (lat) =',f8.3,3x,'max cos (lat) =',f8.3)
c
      iu = 2
      call getmsk (wk,ls,lsp,wtmk,imx,imxp,imy,iu,
     1 iadd,imin,imax)
c
      iu = 4
      ierr = 0
      call getmod(imx,imy,max,wk,smode,iu,undef,ierr)
      if (ierr.ne.0) go to 1018
c
      iu = 9     
      ierr = 0
      call getvar (varmod,max,iu,ierr)
      if (ierr.ne.0) go to 1020
      call maxmin (varmod,max,1,undef,'var mode')
c
c
      call gtbias (wt,max,ntyp,ndfw,dfw,nuse,
     1 ft,smode,imx,imy)
c
      call fdbias (biasp,bias,situ,xsat,ft,ntyp,imx,imxp,imy,
     1 undef,ls,wtmk,iadd,imin,imax)
c     
c  Note the zonal bias subtracted in eotbias-wt is restored to bias, 
c  biasp and xsat
      do j=1,imy
        gsum(j) = 0.
        do ll=1,nuse
          gsum(j) = gsum(j) + dfw(ll)*zon(j,ll)
        enddo
      enddo
      print 107,(zon(1,ll),ll=1,nuse)
  107 format ('Orig SH pole biases =',9f8.3)
      print 108,(zon(imy,ll),ll=1,nuse)
  108 format ('Orig NH pole biases =',9f8.3)
      print 109,gsum(1),gsum(imy),nuse
  109 format ('final pole biases =',2f8.3,3x,'num weights =',i3)
c
      print *,'add zonal bias to EOT derived bias, biasp and ',
     1 'satellite arrays'
      do j=1,imy
        do i=1,imx
          if (ls(i,j).eq.1) then
            bias(i,j) = bias(i,j) + gsum(j)
            xsat(i,j) = xsat(i,j) + gsum(j)
	  endif
        enddo
      enddo
c
c   biasp is is defined everywhere
c
      do i=1,imxp
        do j=1,imy
          biasp(i,j) = biasp(i,j) + gsum(j)
        enddo
      enddo
c
      print *,'bias and satellite field adjusted by zonal bias'
c
c     compute bias variance error: varbs
c
c  iuse is 0 if NOT supported by in situ or satellite
c  iuse is 1 if supported by in situ but NOT by satellite
c  iuse is 2 if NOT supported by in situ but supported by satellite
c  iuse is 3 if supported by in situ and by satellite
c
c   To generate bias iuse = 2 
c      These are satellite modes that would have 
c      been corrected if the in situ data were better
c
      call verr (varbs,varmod,smode,iuse,nuse,
     1 imx,imy,max,dfw,ndfw)
c
      call maxmin (varbs,imx,imy,undef,'var bias')
c
c
c
c - write out eot analysis 
c
      iub = 51
      call ptbias (iub,bias,situ,xsat,varbs,
     1 imx,imy,iyr,imo,ida)
      iw = 52
      iu = 13
c
c  set land/sea mask to ocean everywhere
c  biasp has filled field to match
c  bias has undefined values over land for plotting
c
c
      do j=1,imy
        do i=1,imxp
          lsp(i,j) = 1
        enddo
      enddo
      ierr = 0
      call biascr (biasp,iu,iw,lsp,xtp,yt,imxp,imy,ierr)
c ierr = 1 stop
      if (ierr.eq.1) go to 1011                
      if (iwt.ne.0) then
        PRINT *,'YELLOW - WARNING - EOTBIAS-cor4-MAIN:',
     1 ' missing weights'
        call exit (10)
      endif                
c ierr = 2 warning
      if (ierr.eq.2) go to 1012
c
      call timer (itime2)
      tsec = float(itime2-itime1)*.01
      print *,tsec,' seconds to compute bias adjustment'
      print *,'GREEN - OK - EOTBIAS-cor4-MAIN: completed'
      call exit (1)
c
  900 PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: OPEN ERROR ',
     1 'INPUT/OUTPUT DATA MISSING'
      call exit (100)
 1011 PRINT *,'RED - STOP -  EOTBIAS-cor4-MAIN:',
     1 ' PREMATURE END OF SATELLITE DATA'
      call exit (100)
 1012 PRINT *,'YELLOW - WARNING - EOTBIAS-cor4-MAIN:',
     1 ' NO SATELLITE DATA CORRECTED'
      call exit (10)
 1016 PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: NO LAND SEA MASK '
      call exit (100)
 1018 PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: Spatial modes missing'
      call exit (100)
 1020 PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: Mode variance missing'
      call exit (100)
 1022 PRINT *,'RED - STOP - EOTBIAS-cor4-MAIN: Weights Bad'
      call exit (100)
      END
      subroutine getwt (iu,iuse,wt, ll, ndfw,max,ntyp,iwerr,zon,imy)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getwt       Gets EOT weights and zonal bias correction
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: Reads the weights, iuse and zonal corrections. This routine
c           is called for each dfw smoothing factor, i.e., called nuse times
c   nuse    - number of factors to smooth weights from different days
c
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     IU       - input unit number for weights
c     max      - the maximum number of EOT modes, 130
C              - (FIRST INDEX OF ANALYSIS GRID)
c     ndfw     - the maximum number of smoothing factors = 9
c     ll       - index of ndwd 1 to nuse
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C                IMY =  90   (2-degree grid)   (89S to 89N)
c     ntype    - =2 is the number of anomalies (in situ + satellite)
c                index of 1 is in situ, 2 satellite
C   OUTPUT ARGUMENT LIST:
c      zon     - smoothed zonal bias 
c      iuse    - modes supported by satellite and/or in situ data
c                       determined by checking data using crit in eotbias-wt4
c                 iuse is 0 if NOT supported by in situ or satellite
c                 iuse is 1 if supported by in situ but NOT by satellite
c                 iuse is 2 if NOT supported by in situ but supported by satellite
c                 iuse is 3 if supported by in situ and by satellite
c                 iuse must be = 3 to be used in fit
c      wt       - EOT weights (=0 if not used), 3-D (mode,nty,ndfw)
C      iwerr    - iwerr = 0 if subroutine runs OK; iwerr =1 if there is an error
c                 iwerr = 1 will cause program to stop
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
      dimension wt(max,ntyp,ndfw),iuse(max,ndfw)
      dimension zon(imy,ndfw)
c      print *,iu,ntyp,max,dfw
      iwerr = 0
      read (iu,end=200,err=210) jyr,jmo,jda,
     1 ((wt(i,j,ll),i=1,max),j=1,ntyp),(iuse(i,ll),i=1,max),
     2 (zon(j,ll),j=1,imy)
      print 101,jyr,jmo,jda,zon(1,ll),zon(imy,ll)
  101 format ('wt dates = ',i5,2i3,3x,'zon pole bias (sat-situ) =',
     1 2f10.3)
      return
  200 print *,'RED - STOP - EOTBIAS-getwt: ',
     1  'END weights bad for unit =',iu
      iwerr = 1
      return
  210 print *,'RED - STOP - EOTBIAS-getwt: ',
     1  'ERR weights bad for unit =',iu
      iwerr = 1
      return
      end
      subroutine gengrd (imx,imxp,imy,iadd,xt,yt,xtp,ccos,deg)
C                .      .    .                                       .
C SUBPROGRAM:    GENGRD      GENERATES LAT AND LONG OF bias GRID
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-10-20
C
C ABSTRACT: GENERATES THE LATITUDE AND LONGITUDES
C   FOR THE GRID CENTER OF THE 2-deg bias correction GRID.  
c   IT ALSO GENERATES
C   THE COSINE OF THE LATITUDE POSITION
C
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
C     XT1      - First x (lon) grid point = 1
C     yT1      - First y (lat) grid point = -89.
c     deg      - x and y grid interval = 2
c     iadd     - =1, double to get number of extra points
c   
C   OUTPUT ARGUMENT LIST:
C     XT       - LONGITUDES ON ANALYSIS GRID
C     YT       - LATITUDES ON ANALYSIS GRID
C     ccos     - cosine of LATITUDES ON ANALYSIS GRID
c     xtp      - LONGITUDES ON expanded ANALYSIS GRID
C
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension xt(imx),yt(imy),xtp(imxp),ccos(imy)
      rad = 3.14159/180.
      xt1 = 1.
      xt1p = 1. -deg*float(iadd)
      yt1 = -89.
c - do the longitudes
      do i=1,imx
        xt(i) = deg*float(i-1) + xt1
      enddo
      do i=1,imxp
        xtp(i) = deg*float(i-1) + xt1p
      enddo
c - do the latitudes
      do j=1,imy
        yt(j) = deg*float(j-1)+ yt1
        ccos(j) = cos(rad*yt(j))
      enddo
      return
      end
      subroutine intrp1 (cn,imc,ic,jc,cx,cy,ls,fx,fy,fn,ierr)
c$  subprogram documentation block
c                .      .    .                                       .
c subprogram:    intrp1      finds interpolated value
c   prgmmr: r. w. reynolds   date: 2009-10-20
c
c abstract: finds a linearly interpolated value fn at location
c   fx (longitude) and fy (latitude) relative to
c   an analyzed field cn that is
c   defined by cx (longitude) and cy (latitude).
c   the interpolation includes an adjustment for the
c   convergence of longitude lines as a function of latitude.
c   land values (ls=0) are not used in the interpolation.
c   note: the location (fx,fy) must lie within the grid
c   defined by cx and cy.
c
c program history log:
c   1009-10-20  richard w. reynolds
c   modified from similar code by ron pacanowski: gfdl/noaa  date: 1987 ??     
c
c usage:    call intrp1 (cn,imc,ic,jc,cx,cy,ls,fx,fy,fn,ierr)
c
c   input argument list:
c     cn       - gridded input field
c     imc      - actual 1st dimension of cn in calling routine
c     ic       - first dimension of cn
c     jc       - second dimension of cn
c     cx       - distribution of longitude grid points in cn
c     cy       - distribution of latitude grid points in cn
c     ls       - land sea tag: (=1 over water and =0 over land)
c     fx       - longitude of fn relative to cx coordinates
c     fy       - latitude of fn relative to cy coordinates
c
c   output argument list:
c      fn       - linear interpolated value from cn array
c               - at location fx and fy
c      ierr     - ierr = 0: interpolated value ok
c               - ierr = 1, fn = 0:    no interpolation
c               -   because all grid points are over land,
c               -   or cx(1) is less than or equal to fx,
c               -   or cx(ic) is greater than or equal to fx,
c               -   or cx(1) is less than or equal to fx,
c               -   or cy(jc) is greater than or equal to fy,
c               -   or cx or cy does not monotonically increase
c
c   output files:
c     unit 6 - printout
c
c remarks:  if ierr=1, cx and cy arrays and fx and fy are printed
c           unless all grid points are over land
c
c attributes:
c   language: fortran 77
c   machine:  Linux: warmcore/rainband           
c
c$
      dimension cn(imc,jc),cx(ic),cy(jc)
      dimension ls(imc,jc)
      if (fx.lt.cx(1).or.fx.gt.cx(ic)) go to 10
      if (fy.lt.cy(1).or.fy.gt.cy(jc)) go to 10
      do 110 l=1,jc
      if (cy(l).lt.fy) go to 110
      jn = l
      js = l-1
      if (js.eq.0) jn = 2
      if (js.eq.0) js = 1
      go to 111
  110 continue
      jn = jc
      js = jc - 1
  111 continue
      do 120 l=1,ic
      if (cx(l).lt.fx) go to 120
      ie = l
      iw = l-1
      if (iw.eq.0) ie = 2
      if (iw.eq.0) iw = 1
      go to 121
  120 continue
      ie = ic
      iw = ic - 1
  121 continue
      a = (fx - cx(iw))
      b = (cx(ie) - fx)
      c = fy - cy(js)
      d = cy(jn) - fy
      ac = a*c * float( ls(ie,jn) )
      bc = b*c * float( ls(iw,jn) )
      ad = a*d * float( ls(ie,js) )
      bd = b*d * float( ls(iw,js) )
      top = ac*cn(ie,jn) + bc*cn(iw,jn) + ad*cn(ie,js) + bd*cn(iw,js)
      bot = ac + bc + ad + bd
c      print *,js,jn,iw,ie
c      print *,fx,fy
c      print *,cn(ie,jn),cn(iw,jn),cn(ie,js),cn(iw,js)
c      print *,ls(ie,jn),ls(iw,jn),ls(ie,js),ls(iw,js)
      ierr = 0
      if (bot.eq.0.) go to 200
      fn = top/bot
      return
c
c     no interpolated value
c
   10 print 15
   15 format (' problem in intrp1: fx or fy out of range;',
     1 '   no interpolated value')
      print 20,cx(1),cx(ic),fx
   20 format (' cx(1) =',f10.3,3x,'cx(ic) =',f10.3,3x,'fx =',f10.3)
      print 25,cy(1),cy(jc),fy
   25 format (' cy(1) =',f10.3,3x,'cy(jc) =',f10.3,3x,'fy =',f10.3)
  200 fn = 0.
      ierr = 1
c      stop 99
      return
      end
      subroutine ptbias (iub,bias,situ,xsat,varbs,
     1 imx,imy,iyr,imo,ida)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    ptbias   WRITES OUT DATES, bias, anomalies and bias error
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-10-20
C
C ABSTRACT: WRITES OUT THE DATES, bias, in situ anomalies, satellite 
c           anomalies and the variance of the bias error. Note this
c           is not the same as the ptbias subroutine in eotbias-wt4
C
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     iyr      - year of analysis (4 digits)
c     imo      - month of analysis (2 digits)
c     ida      - day of analysis (2 digits)
C     iub      - FORTRAN OUTPUT UNIT NUMBER
c     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     bias     - bias correction: (satellite - in situ)
c     xsat     - average satellite anomalies for ndays
c     situ     - average weighted ship and buoy anomalies
c     varbs    - variance bias error - output
C
C   OUTPUT FILE:  DETERMINE BY IUN; SEE MAIN PROGRAM
c                output: iyr,imo,ida,bias,situ,xsat,varbs
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
      dimension bias(imx,imy)
      dimension xsat(imx,imy)
      dimension situ(imx,imy),varbs(imx,imy)
      print 7,iyr,imo,ida,iub
    7 format ('dates: ',i5,2i3,3x
     1 'unit =',i5)
      write (iub) iyr,imo,ida,bias
      write (iub) iyr,imo,ida,situ
      write (iub) iyr,imo,ida,xsat
      write (iub) iyr,imo,ida,varbs
c
      return
      end
      subroutine verr (varbs,varmod,smode,iuse,nuse,
     1 imx,imy,max,dfw,ndfw)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    vererr       computes the bias error
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: The bias error is computed using the variance of each mode.
c          The variance is the average of modes not used for iuse =2 
c          (NOT supported by in situ but supported by satellite). These
c          are the satellite modes that can not be corrected. This variance
c          is weighted by the smoothing factors: dwf        
c
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     nuse     - number of factors to smooth weights from different days
c     max      - the maximum number of EOT modes, 130
C              - (FIRST INDEX OF ANALYSIS GRID)
c     ndfw     - the maximum number of smoothing factors = 9
c     dfw      - weights of smoothing factors
c     ntype    - =2 is the number of anomalies (in situ + satellite)
c                index of 1 is in situ, 2 satellite
c     smode    - EOT modes, max = 130 the number of modes
c     varmod   - variance for each EOT mode
c     iuse     - modes supported by satellite and/or in situ data
c                       determined by checking data using crit in eotbias-wt4
c                 iuse is 0 if NOT supported by in situ or satellite
c                 iuse is 1 if supported by in situ but NOT by satellite
c                 iuse is 2 if NOT supported by in situ but supported by satellite
c                 iuse is 3 if supported by in situ and by satellite
c                 iuse must be = 3 to be used in fit
c
C   OUTPUT ARGUMENT LIST:
c      varbs    - variance bias error - output
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
      dimension varbs(imx,imy),varmod(max)
      dimension smode(imx,imy,max),iuse(max,ndfw),dfw(ndfw)
      do i=1,imx
        do j=1,imy
	  varbs(i,j) = 0.
	enddo
      enddo
c
c     weight modes from current analysis when iuse = 2
c
      do ll=1,nuse
        nmode = 0
        do m=1,max
          if (iuse(m,ll).eq.2) then
	    nmode = nmode + 1
	    do i=1,imx
	      do j=1,imy
	        tval = dfw(ll)*smode(i,j,m)**2*varmod(m)
	        varbs(i,j) = varbs(i,j) + tval
	      enddo
	    enddo
           endif
        enddo
        print 101,ll,nmode
  101   format (i3,3x,'modes used for bias error estimation ='
     1   i5)
      enddo
      return
      end
      subroutine getmsk (wk,ls,lsp,wtmk,imx,imxp,imy,iu,
     1 iadd,imin,imax)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getmsk       GETS land/sea mask
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: Reads the 2-degree land/sea mask and bias weights from UNIT IU.  
c The code puts the mask in the integer ls and lsp arrays and the bias weights
c in wtmk
c
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     IU       - input unit number for mask
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     deg      - x and y grid interval = 2
c     iadd     - =1, double to get number of extra points
c     imin     - = 2,  start locations in larger field of original
c     imax     - = 181 end   locations in larger field of original
c   
C   OUTPUT ARGUMENT LIST:
c      wk      - work array
c      ls      - land sea mask (0 ocean, 1 land) from 1-deg mask
c      lsp     - longitude expanded land sea mask
c      wtmk    - bias weights to eliminate any sat bias computed 
c                in high latitudes, plus Caspian Sea and Hudson's Bay
c
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - maxmin
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension wk(imx,imy),ls(imx,imy),wtmk(imx,imy),
     1 lsp(imxp,imy)
      read (iu) wk
c     second field is mask with extended edges: skip
      read(iu)
c     third field is bias weights to eliminate biases in high latitudes, 
c      plus Caspian Sea and Hudsons Bay
      read(iu) wtmk 
      print *,'got land/sea mask: 2-deg'
      call maxmin (wk,imx,imy,0.,'mask 2-d')
      print *,'got bias weights: 2-deg'
      call maxmin (wtmk,imx,imy,0.,'wt mask ')
      do i=1,imx
        do j=1,imy
	  ls(i,j) = nint(wk(i,j))
	enddo
      enddo
      do i=1,imx
        ip = i + iadd
        do j=1,imy
	  lsp(ip,j) = ls(i,j)
        enddo
      enddo
      do j=1,imy
	lsp(1,j) = lsp(imax,j)
	lsp(imxp,j) = lsp(imin,j)
      enddo
      return
      end
      subroutine getmod (imx,imy,max,wk,smode,iu,undef,ierr)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getmod       GETS up to 130 eot modes
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: Reads the 2-degree eot modes. These modes have been filled
c      with zero over land. This will reduce biases near the coast.
c
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     IU       - input unit number for modes
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c      ls      - land sea mask (0 ocean, 1 land) from 1-deg mask
c      xt      - longitude (E) points: 1, 3, 5, .... 359
c      yt      - latitude points: -89, -87, ...., 89
c      ccos    - cosine of latitudes for yt and other (lon, lat) arrays
c      max     -  = 130,the number of EOT modes
c      nsp     - number of total ocean points
c      undef   - = -9999. undefined values
c   
C   OUTPUT ARGUMENT LIST:
c      wk       - work array
c      smode    - EOT modes, max = 130 the number of modes
c      gx       - smodes array stored only over the ocean, nsp=11506
c      cj       - cosine of latitudes for gx and other ocean-only arrays
C      ierr     - ierr = 0 if subroutine runs OK; ierr =1 if there is an error
c                 ierr = 1 will cause program to stop
c
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - maxmin
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension smode(imx,imy,max),wk(imx,imy)
c* Read in the modes.
      ierr = 0
      do nm=1,max
        read (iu,end=100) wk
c	call maxmin (wk,imx,imy,undef,'EOT Mode')
        do j=1,imy
          do i=1,imx
            smode(i,j,nm) = wk(i,j)
          enddo
        enddo
        if (nm.eq.1.or.nm.eq.max) then
	  print *,'mode =',nm
	  call maxmin (wk,imx,imy,undef,'EOT Mode')
	endif
      enddo
      return
  100 print *,'RED - STOP - EOTBIAS-GETMOD:'
      print *,'  error in getmod'
      ierr = 1 
      return
      end
      subroutine getvar (varmod,max,iu,ierr)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getvar       GETS variance of 130 eot modes
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: Reads the expected variance of each eot modes. 
c
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c      IU      - input unit number for modes
c      max     -  = 130,the number of EOT modes
c      nsp     - number of toatl ocean points
c   
C   OUTPUT ARGUMENT LIST:
c      varmod  - expected variance of each of the 130 EOT modes
C      ierr    - ierr = 0 if subroutine runs OK; ierr =1 if there is an error
c                ierr = 1 will cause program to stop
c
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension varmod(max)
c* Read in the mode variances.
      ierr = 0
      read (iu,end=100) varmod
      return
  100 print *,'RED - STOP - EOTBIAS-GETVAR:'
      print *, '  error in getvar'
      ierr = 1 
      return
      end
      subroutine maxmin (fld,imx,imy,undef,name)
c
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    maxmin       WRITES OUT statistics for the array fld
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-20
C
C ABSTRACT: WRITES OUT THE average, minimum, maximum and number of
c           values of array fld. Array values equal to undef are ignored
C
C PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS
C
C
C   INPUT ARGUMENT LIST:
C     fld      - ARRAY to be analyzed
C     IM       - FIRST DIMENSION OF fld array
C     jm       - SECOND DIMENSION OF fld array
c     undef    - indicates that a value in fld is undefined = -9999.
c     name     - a 8 character label of the fld array
C
C   OUTPUT FILES: PRINTOUT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C REMARKS:
C   The number of values in fld is printed out as 'inum ='
C$
c
c   compute maximum, minimum and average of fld and print
c   also set values lt undef to undef
c
      character *8 name
      dimension fld(imx,imy)
      fldmin = 1.e30
      fldmax = -fldmin
      inum = 0
      ave = 0.
      do 90 i=1,imx
      do 90 j=1,imy
      if (fld(i,j).ne.undef) then
        inum = inum + 1
        ave = ave + fld(i,j)
        if (fldmin.gt.fld(i,j))  fldmin = fld(i,j)
        if (fldmax.lt.fld(i,j))  fldmax = fld(i,j) 
      endif
   90 continue
      if (inum.gt.0) then
        r = 1./float(inum)
        ave = ave*r
      else
        fldmin = 0.
        fldmax = 0.
      endif
      print 88,ave,fldmin,fldmax,inum,name
   88 format ('ave =',f8.3,1x,'min =',f8.3,1x,
     1 'max =',f8.3,1x,'num =',i7,1x,a8)
      return 
      end
      subroutine mxmn3 (fld,imx,imy,imz,k,undef,name)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    mxmn2       WRITES OUT statistics for the array fld
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-14
C
C ABSTRACT: WRITES OUT THE average, minimum, maximum and number of
c           values of array fld. Array values equal to undef are ignored
C
C PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS
C
C
C   INPUT ARGUMENT LIST:
C     fld       - ARRAY to be analyzed
C     IMx       - FIRST DIMENSION OF fld array
C     imy       - SECOND DIMENSION OF fld array
c     imz       - third dimension of fld array
c     k         - value of third dimension
c     undef     - indicates that a value in fld is undefined = -9999.
c     name      - an 8 character label of the fld array
C
C   OUTPUT FILES: PRINTOUT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C REMARKS:
C   The number of values in fld is printed out as 'inum ='
C$
c   compute maximum, minimum and average of fld and print
c   also set values lt undef to undef
c
      character *8 name
      dimension fld(imx,imy,imz)
      fldmin = 1.e30
      fldmax = -fldmin
      inum = 0
      ave = 0.
      do 90 i=1,imx
      do 90 j=1,imy
c      do 90 k=1,imz
      if (fld(i,j,k).ne.undef) then
        inum = inum + 1
        ave = ave + fld(i,j,k)
        if (fldmin.gt.fld(i,j,k))  fldmin = fld(i,j,k)
        if (fldmax.lt.fld(i,j,k))  fldmax = fld(i,j,k) 
      endif
   90 continue
      if (inum.gt.0) then
        r = 1./float(inum)
        ave = ave*r
      else
        fldmin = 0.
        fldmax = 0.
      endif
      print 88,k,ave,fldmin,fldmax,inum,name
   88 format (i3,3x,'ave =',f8.3,1x,'min =',f8.3,1x,
     1 'max =',f8.3,1x,'num =',i7,1x,a8)
      return 
      end
      subroutine biascr (biasp,iu,iw,lsp,xtp,yt,imxp,imy,ierr)               
c
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    biascr     applies bias to satellite data
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: The subroutine reads in the original satellite data, interpolates
c           the bias correction to the data location and applies the correction
c           The corrected satellite data is then written out. 
c           biasp is defined as: satellite - in situ
c           The biasp correction is subtracted from the 
c           original satellite data to make the correction.
c
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - intrp1
c
C   INPUT ARGUMENT LIST:
c     biasp     - bias correction, longitude extended, 
c                 bias correction: (satellite - in situ)
c     iu        - 13, input  unit for uncorrected satellite data
c     iw        - 51, output unit for bias corrected satellite data
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMY       - LENGTH OF THE LATITUDE ARRAY
C                 (SECOND INDEX OF ANALYSIS GRID)
c     lsp       - longitude expanded land sea mask
c                 land sea mask (0 ocean, 1 land) from 1-deg mask
c     xtp       - LONGITUDES ON expanded ANALYSIS GRID
C     YT        - LATITUDES ON ANALYSIS GRID
c   
C   OUTPUT ARGUMENT LIST:
C     ierr      - ierr = 0 if subroutine runs OK; ierr =1 if there is an error
c                 ierr = 1 will cause program to stop
c
c Remarks:
c   common block : jrec in subroutine biascr: variables are stored obs
c                data has been written in blocks of up to 10000
c                satellite data is read/written using these blocks
c     rsum(10000) - number of obs
c     rsst(10000) - sst 
c     rlon(10000) - longitude 
c     rlat(10000) - latitude            
c                                                                   
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
c    bias is assumed to be defined as: satellite - in situ
c
      common /jrec/ rsum(10000),rsst(10000),rlon(10000),rlat(10000)             
      dimension yt(imy),xtp(imxp),biasp(imxp,imy),lsp(imxp,imy)
      character*8 type
      rewind iu                                                         
      print 510,iu,iw                                                   
  510 format (/'input sat data unit =',i3,                              
     1 3x,'output sat data unit =',i3)                                  
      read (iu,end=1000) iyr,imo,ida,type       
c
      print 260,iyr,imo,ida,type         
  260 format ('date = ',i5,2i3,3x,'type = ',a8)   
      write (iw) iyr,imo,ida,type           
      amin = +1.e30                                                     
      amax = -amin                                                      
      cmin = +1.e30                                                     
      cmax = -amin
      ncor = 0
      ntot = 0                                                      
   50 read (iu,end=550) ict                                       
      write (iw) ict
      if (ict.eq.0) go to 550                                               
c     print 40,type,ict                                                 
c  40 format (/'read',a10,3x,'numb super obs =',i10)                    
      read (iu,end=1000) (rsum(ii),ii=1,ict)                            
      read (iu,end=1000) (rsst(ii),ii=1,ict)                            
      read (iu,end=1000) (rlat(ii),ii=1,ict)                            
      read (iu,end=1000) (rlon(ii),ii=1,ict)                            
      do 30 l=1,ict                                                     
      sst = rsst(l)                                                     
      xlat = rlat(l)                                                    
      xlon = rlon(l)                                                    
      ntot = ntot + 1
      if (xlat.gt.yt(imy)) xlat = yt(imy)
      if (xlat.lt.yt(1)  ) xlat = yt(1)
      call intrp1 (biasp,imxp,imxp,imy,xtp,yt,lsp,xlon,xlat,corr,jerr) 
c
c jerr not equal to 0: no interpolated value possible                   
c
      if (jerr.eq.0) then                                               
        ncor = ncor + 1
	if (cmax.lt.corr) cmax = corr                                     
        if (cmin.gt.corr) cmin = corr                                     
        rsst(l) = sst - corr                                              
        if (rsst(l).lt.-1.8) rsst(l) = -1.8                                
      endif                                                             
      if (amax.lt.rsst(l)) amax = rsst(l)                                    
      if (amin.gt.rsst(l)) amin = rsst(l)                                     
   30 continue                                                          
      write (iw) (rsum(ii),ii=1,ict)                                    
      write (iw) (rsst(ii),ii=1,ict)                                    
      write (iw) (rlat(ii),ii=1,ict)                                    
      write (iw) (rlon(ii),ii=1,ict)                                    
      go to 50                                                          
  550 if (ncor.gt.0) then
        print 730,cmax,cmin,ncor                                               
  730   format ('sat bias corrections:  max =',f8.2,3x,'min =',f8.2,3x,
     1 'number =',i7)     
        print 735,amax,amin,ntot                                               
  735   format ('sat final ssts      :  max =',f8.2,3x,'min =',f8.2,3x,
     1 'number =',i7)     
      else
        print *,'YELLOW - WARNING - EOTBIAS-BIASCR: ',
     1   'no satellite corrections made'
	ierr = 2
      endif
      return                                                            
 1000 print *,'RED - STOP - EOTBIAS-BIASCR:'  
      print *,'  premature end of satellite input data'         
      ierr = 1
      end                                                               
      subroutine fdbias (biasp,bias,situ,xsat,ft,ntyp,imx,imxp,imy,
     1 undef,ls,wtmk,iadd,imin,imax)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    fdbias     computes final satellite bias correction
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: First the code reduces the in situ and satellite anomalies
c          by wtmk. This eliminates any anomaly in high latitudes, 
c          plus Caspian Sea and Hudson's Bay. The bias is then set to the
c          difference between the reduced satellite and reduced in situ anomaly.
c          The bias array is set to missing over land. Biasp is not set to missing.
c          However, it will be small over land because the eot in smode 
c          were set to zero there. Note the zonal correction to the bias and
c          the satellite anomaly is adjusted in the main code following 
c          the return from this subroutine.
c      
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     nuse     - number of factors to smooth weights from different days
c     max      - the maximum number of EOT modes, 130
C              - (FIRST INDEX OF ANALYSIS GRID)
c     ndfw     - the maximum number of smoothing factors = 9
c     dfw      - weights of smoothing factors
c     ft       - final anomaly field 
c                 last argument of array is 1 for in situ and 2 for satellite
c     ntype    - =2 is the number of anomalies (in situ + satellite)
c                index of 1 is in situ, 2 satellite
c     smode    - EOT modes, max = 130 the number of modes
c     varmod   - variance for each EOT mode
c     wt       - EOT weights (=0 if not used), 3-D (mode,nty,ndfw)
c     iuse     - modes supported by satellite and/or in situ data
c                       determined by checking data using crit in eotbias-wt4
c                 iuse is 0 if NOT supported by in situ or satellite
c                 iuse is 1 if supported by in situ but NOT by satellite
c                 iuse is 2 if NOT supported by in situ but supported by satellite
c                 iuse is 3 if supported by in situ and by satellite
c                 iuse must be = 3 to be used in fit
c      wtmk    - bias weights to eliminate any sat bias computed 
c                       in high latitudes, plus Caspian Sea and Hudson's Bay
c
C   OUTPUT ARGUMENT LIST:
c      xsat    - average satellite anomalies: reduced by wtmk
c      situ    - average weighted ship and buoy anomalies: reduced by wtmk
c      bias    - bias correction: (satellite - in situ)
c      biasp   - bias correction, longitude extended, 
c                       interpolated over land
c
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - maxmin
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c	  
c  sign of bias is: satellite - in situ
c
      dimension ls(imx,imy),wtmk(imx,imy)
      dimension biasp(imxp,imy)
      dimension bias(imx,imy)
      dimension ft(imx,imy,ntyp)
      dimension xsat(imx,imy)
      dimension situ(imx,imy)
c      ict = 0
c      print *,iadd,imin,imax,imx,imxp,imy
      do i=1,imx
        ip = i + iadd
        do j=1,imy
	  tsitu = ft(i,j,1)*wtmk(i,j)
	  txsat = ft(i,j,2)*wtmk(i,j)
	  tbias = txsat - tsitu
          biasp(ip,j) = tbias
          if (ls(i,j).eq.1) then
            bias(i,j) = tbias
            situ(i,j) = tsitu
            xsat(i,j) = txsat
	  else
	    bias(i,j) = undef
	    situ(i,j) = undef
	    xsat(i,j) = undef
          endif
        enddo
c      print *,ip,i,biasp(ip,15),bias(i,15)
      enddo
      do j=1,imy
	biasp(1,j) = biasp(imax,j)
	biasp(imxp,j) = biasp(imin,j)
      enddo
      call maxmin (situ,imx,imy,undef,'situ    ')
      call maxmin (xsat,imx,imy,undef,'sat     ')
      call maxmin (bias,imx,imy,undef,'sat-situ')
      call maxmin (biasp,imxp,imy,undef,'sat-sitP')
      return
c  100 print *,'error in getmod: ierr = 1 '
c      ierr = 1 
c      return
      end
      subroutine gtbias (wt,max,ntyp,ndfw,dfw,nuse,
     1 ft,smode,imx,imy)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    gtbias       GETS satellite and in situ anomalies
C   PRGMMR: R. W. REYNOLDS   2009-10-20
C
C ABSTRACT: Reads the 2-degree eot weights. Uses nuse and dfw to 
c  reconstruction the smoothed satellite and in situ anomalies. 
c
C PROGRAM HISTORY LOG:
C   2009-10-20  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     nuse     - number of factors to smooth weights from different days
c     max      - the maximum number of EOT modes, 130
C              - (FIRST INDEX OF ANALYSIS GRID)
c     ndfw     - the maximum number of smoothing factors = 9
c     dfw      - weights of smoothing factors
c     ntype    - =2 is the number of anomalies (in situ + satellite)
c                index of 1 is in situ, 2 satellite
c     smode    - EOT modes, max = 130 the number of modes
c     varmod   - variance for each EOT mode
c     wt       - EOT weights (=0 if not used), 3-D (mode,nty,ndfw)
c     iuse     - modes supported by satellite and/or in situ data
c                       determined by checking data using crit in eotbias-wt4
c                 iuse is 0 if NOT supported by in situ or satellite
c                 iuse is 1 if supported by in situ but NOT by satellite
c                 iuse is 2 if NOT supported by in situ but supported by satellite
c                 iuse is 3 if supported by in situ and by satellite
c                 iuse must be = 3 to be used in fit
c
C   OUTPUT ARGUMENT LIST:
c      ft       - final anomaly field 
c                 last argument of array is 1 for in situ and 2 for satellite
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
c  iuse is 0 if NOT supported by in situ or satellite
c  iuse is 1 if supported by in situ but NOT by satellite
c  iuse is 2 if NOT supported by in situ but supported by satellite
c  iuse is 3 if supported by in situ and by satellite
c
c   nt = 1 for in situ, nt = 2 for satellite
c
c   in this version iuse must be = 3 to be used in fit
c
      dimension ft(imx,imy,ntyp)
      dimension smode(imx,imy,max)
      dimension wt(max,ntyp,ndfw),dfw(ndfw)
      do ityp=1,ntyp
        do i=1,imx
          do j=1,imy
            sum = 0.0
            do m=1,max
c	      print *,i,j,m,sum,smode(i,j,m),wt(m,ityp)
	      do ll=1,nuse
	        sum = sum + smode(i,j,m)*wt(m,ityp,ll)*dfw(ll)
              enddo
            enddo
            ft(i,j,ityp) = sum
          enddo
        enddo
      enddo
      return
      end
