c      PROGRAM eotbias-wt4-doc.f
c   
c$  MAIN PROGRAM DOCUMENTATION BLOCK
c                .      .    .                                       .
c MAIN PROGRAM: eotbias-wt4-doc.f  COMPUTES EOT weights. 
c   PRGMMR: R.W. REYNOLDS   DATE: 2009-10-20
c
c ABSTRACT: reads super observations from buoys, ships, and satellite data.
c Each type of satellite data (e.g., day AMSR, night AVHRR) is computed 
c separately. The code combines buoy and ship data with stronger weighting 
c for the buoy data based on nby and nsp which are defined below. The code 
c anomaly fields for the in situ (ship + buoy) and satellite on a two degree
c grid using ndays (defined below) of in situ and satellite data then produces 
c anomaly super observations (means). The code then produces zonal averages
c of the in situ and satellite data and subtracts the zonal
c difference (sat-situ) from satellite the satellite anomalies. 
c The code will then fit spatial empirical orthogonal teleconnections (EOTs) 
c to the two anomaly fields. EOTs will only be used if there
c are sufficient data in both the in situ and satellite anomalies fields.
c The weights for each of the EOTs for both fields are saved as well as
c zonal corrections. In the following code eotbias-cor4-doc.f, the weights
c from several days are smoothed, the anomaly fields are computed from the 
c smoothed EOT weights. The zonal bias corrections are restored
c and the difference between the in situ and satellite anomalies becomes the
c bias which is removed from the satellite data. 
c
c   EOTs are defined and applied to SST data in:
c   Smith, T.M., and R.W. Reynolds, 2003: Extended Reconstruction of Global 
c   Sea Surface Temperatures Based on COADS Data (1854 1997). J. Climate, 16,
c   1495-1510.
c
c   Criteria for sufficient data are also defined there.
c   Details of the bias correction procedures can be found in:
c   Reynolds, R. W., T. M. Smith, C. Liu, D. B. Chelton, K. S. Casey, 
c     and M. G. Schlax, 2007: Daily high-resolution blended analyses 
c     for sea surface temperature. J. Climate, 20, 5473-5496.
c
C   NOTE: 
C         OI    =  OPTIMUM INTERPOLATION.
C         SST   =  SEA SURFACE TEMPERATURE
C         EOT   =  empirical orthogonal teleconnections
c     in situ   =  ship and buoy
c
C
C PROGRAM HISTORY LOG:
c   2009-10-20  Richard W. Reynolds
c
c   Input Data on unit 5 needed by program (input from script)
c
c   The 1st eotwt4 input record has 5 parameters: minz,njj,minob,ioff,joff
c minimum number of smoothed zonal points: e.g., minz=750       		
c number of latitude points for zonal smoothing: e.g., njj=10			
c minimum number of obs used in zonal average: e.g., minob=1	        	
c number of longitude boxes smoothed in subroutine smooth
c number of latitude boxes smoothed in subroutine smooth
c e.g., ioff=1, joff =1, means use 1 extra box before and after the
c the current box. subroutine smooth produces a smoothed result 
c used only for the zonal bias computation.  		
c   The 2nd eotwt4 input record has 6 parameters: nby,nsh,nst,ndays,nmodes
c number of ship obs: e.g., nby=7			 
c number of buoy obs: e.g., nsh=1	
c The number of ship and buoy obs weights the combined in situ values 		
c number of satellite obs: e.g., nst=1
c The number of satellite obs is not important as long as it is positive.		
c number of modes used; can't be larger than 130: e.g.,  nmodes=130	        
c number of days used in computing anomalies: e.g., ndays=15                
c   The 3rd record has 3 parameters: yrbuoy yrship yrsat
c character count of buoy first number of year e.g.,  yrbuoy=15		
c character count of ship first number of year e.g.,  yrship=15		
c character count of sat first number of year e.g., yrsat=15		
c    The 4th record has 3 parameters: daybuoy dayship daysat
c character count of buoy first number of date: e.g., daybuoy=40		
c character count of ship first number of date: e.g., dayship=40		
c character count of sat first number of date: e.g., daysat=41
c The 3rd and 4th records are needed to find the data using the 
c   5th-7th file names given below.		
c   The 5th record has 3 parameters: drej crit
c data REJECT LIMIT maximum ABSOLUTE anomaly SST used: e.g., drej=5.			 
c critical data/total area ratio (see EOT reference): e.g.,  crit=0.25
c
c  The 6th input record is the date: computed by script 8 digits
c    in the form: e.g.,20031231 for 31 December 2003
c  All remaining records are input and output file names which must be in order
c  The 1st file name is the two-degree land/sea mask
c  e.g., MASK2D=/raid/rreynold/sstwkly/clm/lstags.twodeg.dat
c  The 2nd file name is the monthly climate (1-deg)
c  e.g., CLIM2D=/raid/rreynold/sstwkly/clm/clim.71.00.gdat 
c  The 3rd file name is the input EOT modes (filled over land with 0)
c  e.g., FMODE=/raid2/sstday/eot/eot6.damp-zero.ev130.ano.dat 
c  The 4th file name is the analyzed weights (output) 
c  e.g., WTSDay=/raid2/sstday/$YEAR/eotwt-finv2/path-err/dsat/pwtdmed.$TM
c  The 5th file name is the input buoy data 
c  e.g., BUOY=/raid2/sstday/yyyy/sobs/coad/buoy/buoy.dddddddd
c  The 6th file name is the input ship data 
c  e.g., SHIP=/raid2/sstday/yyyy/cobs/coad/ship/ship.dddddddd
c  The 7th file name is the input satellite data
c  e.g., SATDay=/raid2/sstday/yyyy/sobs/path/dsat/pdmed.dddddddd
c  Multiple dates are read in and the complete file names
c    for these last 3 files dates are added as computed in the fortran code
c e.g., /raid2/sstday/yyyy/sobs/coad/buoy/buoy.dddddddd
c       12345678901234567890123456789012345678901234567890
c                1         2         3         4
c       In this example, yrbuoy=15 and daybuoy=40
c   The code finds the closest ndays of data. It will look over the
c   record for 2*ndays + 1. If the code only finds ndays/2 or less of
c   data the codes stops. If ndays of data are not found a warning is issued.
c
c  Fortran files: only units 6 and 53 are output files
c   Files with TM ($TM) in name vary with time, the others are static
c     variable unit    name
c              5       input data parameters and file names
c              6       print out
c
c     iu       2       2-degree mask, MASK2D
c     iu       3       climatology, CLIM2D
c     iu       4       130 modes, FMODE
c     iuby     11      buoy data, BUOY
c     iusp     12      ship data,  SHIP
c     iusat    13      satellite data, SATDay or SATNte
c
c     iun      53      EOT weights, WTSDay or WTSNte
c
C     PARAMETERS in Main:
c      parameter (imx=180,imy=90,max=130,nsp=11506,ntyp=2,mxady=61,
c     1 iadd=1,imxp=imx+2*iadd,imin=iadd+1,imax=iadd+imx)
c              - IMX IS LONGITUDE DIMENSION OF analysis (1 to 359E)
C              - IMX = 180  (2-degree grid)                            
C              - IMY IS LATITUDE DIMENSION OF analysis  (-89 to 89)
C                IMY =  90   (2-degree grid)            (89S to 89N)
c     Larger grid, for interpolation in longitude (182,92)
c              - iadd =1, Number of extra longitude points on each side of grid
c              - IMxp LONGITUDE DIMENSION OF larger fields   (-1 to 361 )
C              - IMxp = 182  for iadd =1                     (1W TO 361E or 1E)
c              - imin = 2,  start locations in larger field of original
c              - imax = 181, end  locations in larger field of original
c              - so first and last points of larger field wrap around
c              - e.g., dimension ls(imx,imy),lsp(imxp,imy)
c              - e.g., lsp(2,j) = ls(1,j); lsp(181,j) = ls(180,j)
c              - e.g., lsp(1,j) = lsp(181,j); lsp(182,j) = lsp(2,j)
c              - nsp  = 11506 the number of ocean points in the (180,90) grid
c              - ntype=2 is the number of anomalies (in situ + satellite)
c              - maxdy=61 is the maximum number of time points stored
c              - maxdy must be greater than or equal to ndays*2 + 1
c              - max = 130,the number of EOT modes
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - gengrd, intrp1, ptbias, getmsk, getmod, getvar, getclm, 
c                idaycl, maxmin, mxmn2, getsit, getsat, zero, ave, sum,
c                scrall, scrfta, lsfit, ludcmp, lubksb, zonal, smooth,
c                w3fs26
c      Note:     given julian day w3fs26: returns iyr,imo,ida, 
c                  day of the week and day of the year
C     UNIQUE:  - integer function iw3jdn (iyr,imo,ida) returns julian day
c                  from year (4 digits), month, and day
c                JAN. 1,1960 is defined as JULIAN DAY NUMBER 2436935
C
C   EXIT STATES: There are 3 main categories
c   call exit (1) : Normal End, code color 'green'
c
c   call exit (10): Normal End with WARNING, code color 'yellow'
c                   The problem should be corrected and the code rerun.
c   The following warning 'yellow' errors may occur 
c       eotbias-wt4-main: Number of days too low
c       eotbias-wt4-main: Number of Modes selected low
c
c   call exit (100): Fatal error, Premature STOP, code color 'red'
c                    The problem MUST be corrected and the code rerun.
c   The following fatal 'red' errors may occur 
c       eotbias-wt4-main: OPEN ERROR, INPUT/OUTPUT DATA MISSING
c       eotbias-wt4-main: NO CLIMATE SST AVAILABLE
c       eotbias-wt4-main: NO LAND SEA MASK 
c       eotbias-wt4-main: Spatial modes missing
c       eotbias-wt4-main: Mode variance missing
c       eotbias-wt4-main: CLIMATE Date Problem: month, day =
c       eotbias-wt4-main: In Situ Data Problem from subroutine sum
c       eotbias-wt4-main: Satellite Data Problem from subroutine sum
c       eotbias-wt4-main: Number of days of data too low: num = 
c     
c remarks:  
c       the analysis grid is 2-deg 
c  Unless specifically defined in this program and subroutines,
c      Scalars and arrays starting with letters A-H, and O-Z are real*4 
c      Scalars and arrays starting with letters I-N are integer*4 
c
c common block : jrec in subroutine sum: variables are stored obs
c                data has been written in blocks of up to 10000
c     rsum(10000) - number of obs
c     rsst(10000) - sst 
c     rlon(10000) - longitude 
c     rlat(10000) - latitude            
c
c     - VARIABLES FOLLOW:
c
c  Input scalar variables from input lines (see above)
c
c  minz    - minimum number of smoothed zonal points       		
c  njj     - 2*njj+1 = number of latitude points for zonal smoothing
c  minob   - minimum number of obs used in zonal average	        	
c  ioff    - 2*ioff + 1 =number of longitude boxes smoothed in subr. smooth
c  joff    - 2*joff + 1 = number of latitude boxes smoothed in subr. smooth
c  nby     - weighted number of ship obs		 
c  nsh     - weighted number of buoy obs
c  nst     - weighted number of satellite obs
c  nmodes  - number of modes used; can't be larger than 130
c  ndays   - number of days used in computing anomalies
c  yrbuoy  - character count of buoy first number of year 		
c  yrship  - character count of ship first number of year 	
c  yrsat   -  character count of sat first number of year	
c  daybuoy - character count of buoy first number of date:	
c  dayship - character count of ship first number of date	
c  daysat  - character count of sat first number of date
c  drej    - data REJECT LIMIT maximum ABSOLUTE anomaly SST used
c  crit    - critical data/total area ratio (see EOT reference)
c  fdate   - character*8 date computed by script 8 digits
c             = e.g.,20031231 for 31 December 2003
c
c Other IMPORTANT scalar variables
c     iyr      - year of analysis (4 digits)
c     imo      - month of analysis (2 digits)
c     ida      - day of analysis (2 digits)
c     ijanal   - Julian day of iyr,imo,ida from iw3jdn 
c     deg      - grid interval in degrees = 2
c     undef    - indicates variable is undefined = -9999.
c
c  Important character variables
c     fdate*8     - date computed by script 8 digits
c     fls*80      - file name for land/sea table
c     fclm*80     - file name for 1-degree climatology
c     fmode*80    - file name for 130 EOT modes
c     fship*80    - file name for ship data - dates modified as needed
c     fbuoy*80    - file name for buoy data - dates modified as needed
c     fsat*80     - file name for satellite data - dates modified as needed
c      fwt*80     - file name for output weights 
c                 - File names with full path, may not be larger than 80
c      
c     Important numerical arrays: 
c      csst(imx,imy,12) - climatology, the last index is month (1-12;jan-dec)
c      dclimp(imxp,imy) - climatology interpolated to day, expanded longitude
c
c      xsat(imx,imy)  - average satellite super obs for ndays
c                        super obs are the mean of the obs at a grid point
c      situ(imx,imy)  - average weighted ship and buoy superobs for ndays
c      nsitu(imx,imy) - number of weighted ship and buoy obs
c      nxsat(imx,imy) - number of weighted satellite obs
c      xsats(imx,imy) - smoothed average satellite super obs
c      situs(imx,imy) - smoothed average weighted ship and buoy superobs
c      nsitus(imx,imy)- smoothed number of weighted ship and buoy obs
c      nxsats(imx,imy)- smoothed number of weighted satellite obs
c      zon(imy)       - smoothed zonal bias 
c      nzon(imy)      - smoothed zonal number of points
c      norg(imy)      - unsmoothed zonal number of points
c      zorg(imy)      - unsmoothed zonal bias from smoothed data
c                       xsats, situs, nxsats and nsitus
c      wk(imx,imy)    - work array
c      ls(imx,imy)    - land sea mask (0 ocean, 1 land) from 1-deg mask
c      lsp(imxp,imy)  - longitude expanded land sea mask
c      wtmk(imx,imy)  - bias weights to eliminate any sat bias computed 
c                       in high latitudes, plus Caspian Sea and Hudson's Bay
c      smode(imx,imy,max)
c                     - EOT modes, max = 130 the number of modes
c      ndy(mxady)     - days offset from analysis day: order is 
c                       Days offset:    0  -1   1  -2   2  -3   3 ....
c                       up to ndays of data are used
c      xt(imx)        - longitude (E) points: 1, 3, 5, .... 359
c      yt(imy)        - latitude points: -89, -87, ...., 89
c      ccos(imy)      - cosine of latitudes for yt and other (lon,lat) arrays
c      xtp(imxp)      - longitude expanded: -1, 1, 3, 5, .... 359, 361
c      gx(nsp,max)    - smodes stored only over the ocean, nsp=11506
c      cj(nsp)        - cosine of latitudes for gx and other ocean-only arrays
c      gxf(nsp,max)   - smodes stored only over the ocean; 
c                       the second index varies only over modes which are used
c                       and is usually less than max
c      dat(nsp,ntyp)  - in situ and satellite super obs from situ and xsat
c                       nytp = 1 for in situ, ntyp = 2 for satellite
c
c      dat1(nsp)      - reduced data from dat
c                       linear solution code required only 1-dimension
c      wt(max,ntyp)   - EOT weights (=0 if not used), 2-D
c      wf(max)        - EOT weights,- only modes accepted, later moved to wt, 1-D
c                       linear solution code required only 1-dimension
c      ft(imx,imy,ntyp) 
c                     - final anomaly field - legacy - recomputed in eotbias-cor4
c      iuse(max)      - modes supported by satellite and/or in situ data
c                       determined by checking data using crit 
c                 iuse is 0 if NOT supported by in situ or satellite
c                 iuse is 1 if supported by in situ but NOT by satellite
c                 iuse is 2 if NOT supported by in situ but supported by satellite
c                 iuse is 3 if supported by in situ and by satellite
c                 iuse must be = 3 to be used in fit
c
c ATTRIBUTES:
c   LANGUAGE: FORTRAN 77 
c             
c   MACHINE:  Linux: warmcore/rainband
c
c$
c
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c     7                                                               72
c
c Note smoothed super obs are only used in zonal bias correction not EOTs
c
      parameter (imx=180,imy=90,max=130,nsp=11506,ntyp=2,mxady=61,
     1 iadd=1,imxp=imx+2*iadd,imin=iadd+1,imax=iadd+imx)
      dimension zon(imy),norg(imy),zorg(imy),nzon(imy)
      dimension wk(imx,imy),wtmk(imx,imy)
      dimension smode(imx,imy,max),ndy(mxady)
      dimension ls(imx,imy),lsp(imxp,imy)
      dimension xt(imx),yt(imy),ccos(imy),xtp(imxp)
      dimension gx(nsp,max),gxf(nsp,max),cj(nsp)
      dimension dat(nsp,ntyp),dat1(nsp)
      dimension wt(max,ntyp),wf(max),ft(imx,imy,ntyp),iuse(max)
      dimension xsat(imx,imy), nxsat(imx,imy)
      dimension situ(imx,imy), nsitu(imx,imy)
      dimension xsats(imx,imy), nxsats(imx,imy)
      dimension situs(imx,imy), nsitus(imx,imy)
      dimension csst(imx,imy,12)
      dimension dclimp(imxp,imy)
      dimension lcyrs(3),lcyre(3),lcdays(3),lcdaye(3)
      character*4 name(3)
      character*8 fdate,type
c
      character*80 fls,fclm,fmode,fship,fbuoy,fsat,fwt
      data name / 'buoy','ship','sat '/
c      unit 2 is 2-degree mask
c      unit 3 is climatology
c      unit 4 is 130 modes
c      unit 11 is buoy data
c      unit 12 is ship data
c      unit 13 is satellite data
c      unit 53 is EOT weights
c
      logical lexist
c     for preliminary version, the fortran files will be included in the code
c
c
c     input data: order is critical
c
c23456
c
c
      call timer (itime1)
c      minz = 500
c      njj = 10
c      minob = 1
c      ioff = 1 
c      joff = 1
      read *,minz,njj,minob,ioff,joff
      print *,'minz = ',minz,'  njj =  ',njj,'  minob = ',minob
      print *,'ioff = ',ioff,'  joff =  ',joff
      do i=1,mxady
        is = 2*mod(i,2) - 1
        ndy(i) = is*(i/2)
      enddo
c      print 201,ndy
c  201 format ('Days offset: ',16i4)
c
      read *,nby,nsh,nst,ndays,nmodes 
      print 118,nby,nsh,nst,ndays,nmodes
  118 format ('nby =',i3,3x,
     1 'nsh =',i3,3x,'nst =',i3,3x,'ndays smoothed =',i3,
     2 3x,'max modes used =',i4)
      mxdy = ndays*2 + 1
      if (mxdy.gt.mxady) mxdy = mxady
      mndy = ndays/2
      print 202,mxdy,mndy,ndays
  202 format ('max days considered =',i5/
     1 'stop if numb of days <',i5,3x,'warning if numb days <',i5 )
      read *,lcyrs
      read *,lcdays
      do ic=1,3
      lcyre(ic) = lcyrs(ic) + 3
      lcdaye(ic) = lcdays(ic) + 7
      print 216,name(ic),lcyrs(ic),lcyre(ic),lcdays(ic),lcdaye(ic)     
  216 format ('char for ',a4,3x,'year(',i2.2,':',i2.2,
     1 ')   date(',i2.2,':',i2.2,')') 
      enddo
      read *,drej,crit
      print 112,drej,crit
  112 format ('drej =',f7.2,3x,'crit =',f7.2)
      read 12,fdate
   12 format (a8)
      read (fdate,11) iyr,imo,ida 
   11 format (i4,i2,i2)
      iyranl = iyr
      imoanl = imo
      idaanl = ida
      ijanal = iw3jdn(iyranl,imoanl,idaanl)
      print 17,iyr,imo,ida
   17 format ('analaysis date: ',i5,i3,i3)
      read 13,fls
      print *,'unit 2  '
      print *,fls
      read 13,fclm
      print *,'unit 3   '
      print *,fclm
      read 13,fmode
      print *,'unit 4   '
c
      read 13,fwt
      print *,'unit 53  '
      print *,fwt
   13 format (a80)
c
c      print *, 'open  ',fls
      inquire(file=fls,exist=lexist)
c     print *,lexist
      if(.not.lexist) then
        print *,'no land sea'
	print *,' '
        go to 1016 
      endif  
c      print *, 'open  ',fclm
      inquire(file=fclm,exist=lexist)
c     print *,lexist
      if(.not.lexist) then
        print *,'no clim'
	print *,' '
        go to 1015 
      endif  
c
      open (2,err=900,file=fls,form='unformatted')
      print *,'unit 2  '
      open (3,err=900,file=fclm,form='unformatted')
      print *,'unit 3  '
      open (4,err=900,file=fmode,form='unformatted')
      print *,'unit 4  '
c
      open (53,err=900,file=fwt,form='unformatted')
      print *,'unit 53  '
c
      undef = -9999.
c
      deg = 2.
      call gengrd (imx,imxp,imy,iadd,xt,yt,xtp,ccos,deg)
      print 111,xt(1),xt(imx),xtp(1),xtp(imxp),yt(1),yt(imy),
     1 ccos(1),ccos(imy)
  111 format ('min xt (lon) =',f8.1,3x,'max xt (lon) =',f8.1/
     1 'min xtp (lon-p) =',f8.1,3x,'max xtp (lon-p) =',f8.1/
     2 'min yt (lat) =',f8.1,3x,'max yt (lat) =',f8.1/
     2 'min cos (lat) =',f8.3,3x,'max cos (lat) =',f8.3)
c
      iu = 2
      call getmsk (wk,ls,lsp,wtmk,imx,imxp,imy,iu,
     1 iadd,imin,imax)
c
      iu = 3
      call getclm (csst,wk,imx,imy,iu,undef)
      iu = 4
      ierr = 0
      call getmod (gx,cj,ccos,wk,ls,imx,imy,nsp,max,
     1 smode,iu,undef,ierr)
      if (ierr.ne.0) go to 1018
      call maxmin (gx,nsp,max,undef,'gx modes')
c     
c     zero out situ and xsat arrays
c
      call zero (situ,nsitu,imx,imy)
      call zero (xsat,nxsat,imx,imy)
c
c     data loop starts here for days
c
      read 13,fbuoy
      print *,'unit 11 ',fbuoy
      read 13,fship
      print *,'unit 12 ',fship
      read 13,fsat
      print *,'unit 13  ',fsat
c
      ictdy = 0
      do 100 jday = 1,mxdy
      close (11)
      close (12)
      close (13)
      ijday = ijanal - ndy(jday)
      call w3fs26(ijday,iyrday,imoday,idaday,idwk,idyr)
      print 119,iyrday,imoday,idaday
  119 format (/'input obs: year =',i5,3x,'month =',i3,3x,
     1 'day =',i3)
      jmo = imoday
      jda = idaday
      ierr = 0
      call idaycl (csst,jmo,jda,dclimp,imx,imxp,imy,
     1 iadd,imin,imax,ierr)
      if (ierr.ne.0) go to 1030
      call maxmin (dclimp,imxp,imy,undef,'clim    ')
      write (fbuoy(lcyrs(1):lcyre(1)),'(i4.4)') iyrday
      write (fship(lcyrs(2):lcyre(2)),'(i4.4)') iyrday
      write (fsat (lcyrs(3):lcyre(3)),'(i4.4)') iyrday
      write (fbuoy(lcdays(1):lcdaye(1)),'(i4.4,i2.2,i2.2)') 
     1 iyrday,imoday,idaday
      write (fship(lcdays(2):lcdaye(2)),'(i4.4,i2.2,i2.2)')
     1 iyrday,imoday,idaday
      write (fsat (lcdays(3):lcdaye(3)),'(i4.4,i2.2,i2.2)') 
     1 iyrday,imoday,idaday
c
c      print *, 'open  ',fbuoy
      inquire(file=fbuoy,exist=lexist)
c     print *,lexist
      if(.not.lexist) then
        print *,'no buoy data'
        print *,fbuoy
	print *,' '
	go to 100
      endif  
c      print *, 'open  ',fship
      inquire(file=fship,exist=lexist)
c     print *,lexist
      if(.not.lexist) then
        print *,'no ship data'
        print *,fship
	print *,' '
	go to 100
      endif  
c      print *, 'open  ',fsat
      inquire(file=fsat,exist=lexist)
c     print *,lexist
      if(.not.lexist) then
        print *,'no satellite data'
        print *,fsat
	print *,' '
	go to 100
      endif 
c
      ictdy = ictdy + 1
      open (11,err=900,file=fbuoy,form='unformatted')
      open (12,err=900,file=fship,form='unformatted')
      open (13,err=900,file=fsat,form='unformatted')
      do iu=11,13 
	read (iu) jyr,jmo,jda,type
	
	print*,'********',iu,jyr,jmo,jda,type
	
	jjday = iw3jdn(jyr,jmo,jda)
	if (jjday.ne.ijday) then
	  print 109,iu,iyrday,imoday,idaday,jyr,jmo,jda,type
  109     format ('dates do not match: iu =',i5,3x,
     1     'data =',i5,2i3,3x,'loop =',i5,2i3,3x,a8)
          go to 1010
        endif
      enddo 
      iuby = 11
      iusp = 12
      iusat = 13
c      print *,deg
      ierr = 0
      call getsit (situ,nsitu,iuby,iusp,xt,xtp,yt,
     1 imx,imxp,imy,nby,nsh,dclimp,lsp,drej,deg,ierr)
      if (ierr.gt.0) go to 1031
c      call maxmin (situ,imx,imy,undef,'situ         ')
c      print *,deg
      ierr = 0
      call getsat (xsat,nxsat,iusat,xt,xtp,yt,
     1 imx,imxp,imy,nst,dclimp,lsp,drej,deg,ierr)
      if (ierr.gt.0) go to 1032
c      call maxmin (xsat,imx,imy,undef,'sat            ')
      if (ictdy.eq.ndays) go to 110
  100 continue
c     
c     average situ and xsat arrays
c
  110 print *,'number of data days used =',ictdy
      kdyf = 0
      if (ictdy.lt.ndays) kdyf = 1
      if (ictdy.lt.mndy.or.ictdy.gt.ndays) then
        kdyf = 2
	go to 1040
      endif
      call smooth (situ,nsitu,situs,nsitus,ioff,joff,imx,imy)
      call smooth (xsat,nxsat,xsats,nxsats,ioff,joff,imx,imy)
c
      call ave (situ,nsitu,undef,imx,imy)
      call maxmin (situ,imx,imy,undef,'situ         ')
      call ave (xsat,nxsat,undef,imx,imy)
      call maxmin (xsat,imx,imy,undef,'sat            ')

c
      call ave (situs,nsitus,undef,imx,imy)
      call maxmin (situs,imx,imy,undef,'situ smooth  ')
      call ave (xsats,nxsats,undef,imx,imy)
      call maxmin (xsats,imx,imy,undef,'sat smmoth     ')
c
c    compute zonal bias (sat-situ)
c
      call zonal (zon,zorg,norg,nzon,situs,nsitus,
     1 xsat,nxsat,imx,imy,minob,minz,njj,yt,ccos)
c
c
c   subtract zonal bias (sat-situ) from satellite data
c
      print *,'subtract zonal bias (sat-situ) from satellite data'
c
      do j=1,imy
        do i=1,imx
          if (nxsat(i,j).gt.0) then
            xsat(i,j) = xsat(i,j) - zon(j)
	  endif
        enddo
      enddo
c
      ict = 0
      do j=1,imy
        do i=1,imx
          if (ls(i,j).eq.1)then
             ict = ict + 1
             if (wtmk(i,j).lt.0.1) then
	       dat(ict,1) = undef
	       dat(ict,2) = undef
	     else
	       dat(ict,1) = situ(i,j)
	       dat(ict,2) = xsat(i,j)
            endif
          endif
        enddo
      enddo
      print *,'ict =',ict,'  nsp =',nsp
      if (ict.ne.nsp) go to 1020
      call maxmin (dat,nsp,ntyp,undef,'data anm')
c
      call scrall(gx,dat,ntyp,nsp,max,cj,crit,
     1 nfit,iuse,undef)
      print *,'number of modes used =',nfit
      modlim = nmodes/2
c      if (nfit.lt.3) go to 1019
      if (nfit.gt.modlim) then
        iprob = 0
      else
        iprob = 1
      endif
c     
c    set higher modes to zero
c
      nmp = nmodes + 1
      if (nmp.le.max) then
         do l=nmp,max
	   iuse(l) = 0
	 enddo
      endif 
      nfit = 0
      print 212
  212 format ('iuse=0 if NOT supported by in situ or sat'/
     1 'iuse=1 if supported by in situ but NOT by sat'/
     2 'iuse=2 if NOT supported by in situ but supported by sat'/
     3 'iuse=3 if supported by in situ and by sat')
      do l=1,max
	nfit = nfit + iuse(l)/3
	if (iuse(l).ne.3) then
	  print 211,l,iuse(l)
  211     format ('  mode =',i4,3x,'iuse =',i4)	   
        endif
      enddo
      print *,'reduced number of modes used =',nfit
c
      do ityp=1,ntyp
        call scrfta (gx,dat,dat1,iuse,nsp,max,cj,nfit,ft,
     1   smode,imx,imy,wt,wf,gxf,ityp,ntyp,undef)
      enddo
      call maxmin (wt,max,ntyp,undef,'fit wt  ')
c
c
c - write out eot analysis 
c
      iun = 53
      call ptbias (iun,wt,max,ntyp,zon,imy,
     1 iyr,imo,ida,ndays,iuse)
c
      call timer (itime2)
      tsec = float(itime2-itime1)*.01
      print *,tsec,' seconds to compute bias adjustment weights'
      if (kdyf.ne.0) then
        PRINT *,'YELLOW-WARNING! -  EOTBIAS-MAIN: ',
     1 ' Number of days too low =',ictdy
        call exit (10)
      endif
      if (iprob.ne.0) then
        PRINT *,'YELLOW-WARNING! -  EOTBIAS-MAIN: ',
     1 ' Number of Modes selected low =',nfit
        call exit (10)
      endif
      print *,'GREEN-SUCCESSFUL! - EOTBIAS-MAIN:eot bias code completed'
      call exit (1)
c
  900 PRINT *,'RED-STOP! - EOTBIAS-MAIN: OPEN ERROR',
     1 '  INPUT/OUTPUT DATA MISSING'
      call exit (100)
 1010 PRINT *,'RED-STOP! - EOTBIAS-MAIN: INPUT DATA MISSING  '
      call exit (100)
 1015 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 '  NO CLIMATE SST AVAILABLE'
      call exit (100)
 1016 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 'NO LAND SEA MASK '
      call exit (100)
 1018 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 '  Spatial modes missing'
      call exit (100)
 1019 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 '  Number of Spatial modes used less than 3'
      call exit (100)
 1020 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 '  Mode variance missing'
      call exit (100)
 1030 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1'  CLIMATE Date Problem: month, day ',jmo,jda
      call exit (100)
 1031 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 '  In Situ Data Problem from subroutine sum'
      call exit (100)
 1032 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 '  Satellite Data Problem from subroutine sum'
      call exit (100)
 1040 PRINT *,'RED-STOP! -  EOTBIAS-MAIN:',
     1 '  Number of days of data too low: num = ',ictdy
      call exit (100)
      END
      
      subroutine gengrd (imx,imxp,imy,iadd,xt,yt,xtp,ccos,deg)
C                .      .    .                                       .
C SUBPROGRAM:    GENGRD      GENERATES LAT AND LONG OF bias GRID
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-10-15
C
C ABSTRACT: GENERATES THE LATITUDE AND LONGITUDES
C   FOR THE GRID CENTER OF THE 2-deg bias correction GRID.  
c   IT ALSO GENERATES
C   THE COSINE OF THE LATITUDE POSITION
C
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
C     XT1      - First x (lon) grid point = 1
C     yT1      - First y (lat) grid point = -89.
c     deg      - x and y grid interval = 2
c     iadd     - =1, double to get number of extra points
c   
C   OUTPUT ARGUMENT LIST:
C     XT       - LONGITUDES ON ANALYSIS GRID
C     YT       - LATITUDES ON ANALYSIS GRID
C     ccos     - cosine of LATITUDES ON ANALYSIS GRID
c     xtp      - LONGITUDES ON expanded ANALYSIS GRID
C
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension xt(imx),yt(imy),xtp(imxp),ccos(imy)
      rad = 3.14159/180.
      xt1 = 1.
      xt1p = 1. -deg*float(iadd)
      yt1 = -89.
c - do the longitudes
      do i=1,imx
        xt(i) = deg*float(i-1) + xt1
      enddo
      do i=1,imxp
        xtp(i) = deg*float(i-1) + xt1p
      enddo
c - do the latitudes
      do j=1,imy
        yt(j) = deg*float(j-1)+ yt1
        ccos(j) = cos(rad*yt(j))
      enddo
      return
      end
      subroutine intrp1 (cn,imc,ic,jc,cx,cy,ls,fx,fy,fn,ierr)
c$  subprogram documentation block
c                .      .    .                                       .
c subprogram:    intrp1      finds interpolated value
c   prgmmr: r. w. reynolds   date: 2009-10-15
c
c abstract: finds a linearly interpolated value fn at location
c   fx (longitude) and fy (latitude) relative to
c   an analyzed field cn that is
c   defined by cx (longitude) and cy (latitude).
c   the interpolation includes an adjustment for the
c   convergence of longitude lines as a function of latitude.
c   land values (ls=0) are not used in the interpolation.
c   note: the location (fx,fy) must lie within the grid
c   defined by cx and cy.
c
c program history log:
c   1009-10-15  richard w. reynolds
c   modified from similar code by ron pacanowski: gfdl/noaa  date: 1987 ??     
c
c usage:    call intrp1 (cn,imc,ic,jc,cx,cy,ls,fx,fy,fn,ierr)
c
c   input argument list:
c     cn       - gridded input field
c     imc      - actual 1st dimension of cn in calling routine
c     ic       - first dimension of cn
c     jc       - second dimension of cn
c     cx       - distribution of longitude grid points in cn
c     cy       - distribution of latitude grid points in cn
c     ls       - land sea tag: (=1 over water and =0 over land)
c     fx       - longitude of fn relative to cx coordinates
c     fy       - latitude of fn relative to cy coordinates
c
c   output argument list:
c      fn       - linear interpolated value from cn array
c               - at location fx and fy
c      ierr     - ierr = 0: interpolated value ok
c               - ierr = 1, fn = 0:    no interpolation
c               -   because all grid points are over land,
c               -   or cx(1) is less than or equal to fx,
c               -   or cx(ic) is greater than or equal to fx,
c               -   or cx(1) is less than or equal to fx,
c               -   or cy(jc) is greater than or equal to fy,
c               -   or cx or cy does not monotonically increase
c
c   output files:
c     unit 6 - printout
c
c remarks:  if ierr=1, cx and cy arrays and fx and fy are printed
c           unless all grid points are over land
c
c attributes:
c   language: fortran 77
c   machine:  Linux: warmcore/rainband           
c
c$
      dimension cn(imc,jc),cx(ic),cy(jc)
      dimension ls(imc,jc)
      if (fx.lt.cx(1).or.fx.gt.cx(ic)) go to 10
      if (fy.lt.cy(1).or.fy.gt.cy(jc)) go to 10
      do 110 l=1,jc
      if (cy(l).lt.fy) go to 110
      jn = l
      js = l-1
      if (js.eq.0) jn = 2
      if (js.eq.0) js = 1
      go to 111
  110 continue
      jn = jc
      js = jc - 1
  111 continue
      do 120 l=1,ic
      if (cx(l).lt.fx) go to 120
      ie = l
      iw = l-1
      if (iw.eq.0) ie = 2
      if (iw.eq.0) iw = 1
      go to 121
  120 continue
      ie = ic
      iw = ic - 1
  121 continue
      a = (fx - cx(iw))
      b = (cx(ie) - fx)
      c = fy - cy(js)
      d = cy(jn) - fy
      ac = a*c * float( ls(ie,jn) )
      bc = b*c * float( ls(iw,jn) )
      ad = a*d * float( ls(ie,js) )
      bd = b*d * float( ls(iw,js) )
      top = ac*cn(ie,jn) + bc*cn(iw,jn) + ad*cn(ie,js) + bd*cn(iw,js)
      bot = ac + bc + ad + bd
c      print *,js,jn,iw,ie
c      print *,fx,fy
c      print *,cn(ie,jn),cn(iw,jn),cn(ie,js),cn(iw,js)
c      print *,ls(ie,jn),ls(iw,jn),ls(ie,js),ls(iw,js)
      ierr = 0
      if (bot.eq.0.) go to 200
      fn = top/bot
      return
c
c     no interpolated value
c
   10 print 15
   15 format (' problem in intrp1: fx or fy out of range;',
     1 '   no interpolated value')
      print 20,cx(1),cx(ic),fx
   20 format (' cx(1) =',f10.3,3x,'cx(ic) =',f10.3,3x,'fx =',f10.3)
      print 25,cy(1),cy(jc),fy
   25 format (' cy(1) =',f10.3,3x,'cy(jc) =',f10.3,3x,'fy =',f10.3)
  200 fn = 0.
      ierr = 1
c      stop 99
      return
      end
      subroutine ptbias (iuw,wt,max,ntyp,zon,imy,
     1 iyr,imo,ida,ndays,iuse)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    ptbias   WRITES OUT DATES, eot Weights and zonal corrections
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-10-15
C
C ABSTRACT: WRITES OUT THE DATES, EOT Weights and EOT data and flags and
c         zonal corrections which have already been applied to the satellite data
C
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     iuw      - FORTRAN OUTPUT UNIT NUMBER
c     iuse     - modes supported by satellite and/or in situ data
c                       determined by checking data using crit 
C     IMY      - LATITUDE DIMENSION OF analysis  (-89 to 89)
C                IMY =  90   (2-degree grid)     (89S to 89N)
c     max      - 130, the number of modes
c     ntype    - =2 is the number of anomalies (in situ + satellite)
c                index of 1 is in situ, 2 satellite
c     wt       - EOT weights (= 0 if not used).
c     zon      - smoothed zonal bias 
c     ndays    - number of days used
C
C   OUTPUT FILE:  DETERMINE BY IUN; SEE MAIN PROGRAM
C                  output: yr,imo,ida,wt,iuse,zon        
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension wt(max,ntyp)
      dimension iuse(max)
      dimension zon(imy)      
      print 7,iyr,imo,ida,ndays,iuw
    7 format ('dates: ',i5,2i3,3x
     1 'ndays = ',i3,3x,'unit =',i5)
c
      write (iuw) iyr,imo,ida,wt,iuse,zon
c
      return
      end
      subroutine getmsk (wk,ls,lsp,wtmk,imx,imxp,imy,iu,
     1 iadd,imin,imax)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getmsk       GETS land/sea mask
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: Reads the 2-degree land/sea mask and bias weights from UNIT IU.  
c The code puts the mask in the integer ls and lsp arrays and the bias weights
c in wtmk
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     IU       - input unit number for mask
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     deg      - x and y grid interval = 2
c     iadd     - =1, double to get number of extra points
c     imin     - = 2,  start locations in larger field of original
c     imax     - = 181 end   locations in larger field of original
c   
C   OUTPUT ARGUMENT LIST:
c      wk      - work array
c      ls      - land sea mask (0 ocean, 1 land) from 1-deg mask
c      lsp     - longitude expanded land sea mask
c      wtmk    - bias weights to eliminate any sat bias computed 
c                in high latitudes, plus Caspian Sea and Hudson's Bay
c
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - maxmin
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension wk(imx,imy),ls(imx,imy),wtmk(imx,imy),
     1 lsp(imxp,imy)
c     first field is the land sea mask
      read (iu) wk
c     second field is mask with extended edges: skip
      read(iu)
c     third field is bias weights to eliminate biases in high latitudes, 
c      plus Caspian Sea and Hudsons Bay
      read(iu) wtmk 
      print *,'got land/sea mask: 2-deg'
      call maxmin (wk,imx,imy,0.,'mask 2-d')
      print *,'got bias weights: 2-deg'
      call maxmin (wtmk,imx,imy,0.,'wt mask ')
      do i=1,imx
        do j=1,imy
	  ls(i,j) = nint(wk(i,j))
	enddo
      enddo
      do i=1,imx
        ip = i + iadd
        do j=1,imy
	  lsp(ip,j) = ls(i,j)
        enddo
      enddo
      do j=1,imy
	lsp(1,j) = lsp(imax,j)
	lsp(imxp,j) = lsp(imin,j)
      enddo
      return
      end
      subroutine getmod (gx,cj,ccos,wk,ls,imx,imy,nsp,max,
     1 smode,iu,undef,ierr)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getmod       GETS up to 130 eot modes
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: Reads the 2-degree eot modes. These modes have been filled
c      with zero over land. This will reduce biases near the coast.
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     IU       - input unit number for modes
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c      ls      - land sea mask (0 ocean, 1 land) from 1-deg mask
c      xt      - longitude (E) points: 1, 3, 5, .... 359
c      yt      - latitude points: -89, -87, ...., 89
c      ccos    - cosine of latitudes for yt and other (lon, lat) arrays
c      max     -  = 130,the number of EOT modes
c      nsp     - number of total ocean points
c      undef   - = -9999. undefined values
c   
C   OUTPUT ARGUMENT LIST:
c      wk       - work array
c      smode    - EOT modes, max = 130 the number of modes
c      gx       - smodes array stored only over the ocean, nsp=11506
c      cj       - cosine of latitudes for gx and other ocean-only arrays
C      ierr     - ierr = 0 if subroutine runs OK; ierr =1 if there is an error
c                 ierr = 1 will cause program to stop
C
c   SUBPROGRAMS CALLED:
C     UNIQUE:  - maxmin
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension wk(imx,imy),ls(imx,imy)
      dimension gx(nsp,max),cj(nsp),ccos(imy)
      dimension smode(imx,imy,max)
c* Read in the modes.
      ierr = 0
      do nm=1,max
        read (iu,end=100) wk
c	call maxmin (wk,imx,imy,undef,'EOT Mode')
        ict = 0
        do j=1,imy
          do i=1,imx
            smode(i,j,nm) = wk(i,j)
	    if (ls(i,j).eq.1) then
               ict = ict + 1
	       if (ict.gt.nsp) go to 100
               gx(ict,nm) = wk(i,j)
c	       if (wk(i,j).gt.0.) then
c	         print *,i,j,ict,nm,smode(i,j,nm),wk(i,j),gx(ict,nm)
c	       endif
            endif
          enddo
        enddo
        if (nm.eq.1.or.nm.eq.max) then
	  print *,'mode =',nm,'   ict =',ict
	  call maxmin (wk,imx,imy,undef,'EOT Mode')
	endif
      enddo
      ict = 0
      do j=1,imy
        do i=1,imx
          if(ls(i,j).eq.1)then
             ict = ict + 1
	     if (ict.gt.nsp) go to 100
             cj(ict) = ccos(j)
          endif
        enddo
      enddo
      print *,'cosine weights   ict =',ict
      call maxmin (cj,nsp,1,undef,'cos wt  ')
      return
  100 print *,'RED-STOP! - EOTBIAS-GETMOD:'
      print *,'  error in getmod'
      ierr = 1 
      return
      end
      subroutine getvar (varmod,max,iu,ierr)
      dimension varmod(max)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getvar       GETS expected variance for 130 eot modes
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: gets the expected variance for each of eot modes. 
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c      IU      - input unit number for modes
c      max     -  = 130,the number of EOT modes
c   
C   OUTPUT ARGUMENT LIST:
c      varmode  - expected variance of the 130 EOT modes
C      ierr     - ierr = 0 if subroutine runs OK; ierr =1 if there is an error
c                 ierr = 1 will cause program to stop
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c* Read in the mode variances.
      ierr = 0
      read (iu,end=100) varmod
      return
  100 print *,'RED-STOP! - EOTBIAS-GETVAR:'
      print *, '  error in getvar'
      ierr = 1 
      return
      end
      subroutine getclm (csst,wk,imx,imy,iu,undef)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getclm       GETS the 1-degree 12 month climate
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: Reads the 1-deg climatology. Converts it to 2-degree and saves it
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     IU       - input unit number for modes
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     undef    - = -9999. undefined values
c   
C   OUTPUT ARGUMENT LIST:
c      wk       - work array
c      csst     - climatology, the last index is month (1-12;jan-dec)
c
c     Important numerical arrays: 
c      csst1    - 1- degree climatology, (360,180)
c                 has 44219 valid points
c
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension csst(imx,imy,12),wk(imx,imy)
      dimension sst1(360,180)
      do 100 imo=1,12
        read (iu) iyr,jmon,sst1
c
c  convert to 2-degree
c      	
        do i=1,imx
          ip = i*2 
          im = ip - 1
          do j=1,imy
            jp = j*2
            jm = jp - 1
  	    top = 0.
	    bot = 0.
	    do i1=im,ip
	      do j1=jm,jp
	        if (sst1(i1,j1).gt.undef) then
	          top = top + sst1(i1,j1)
		  bot = bot + 1.
	        endif
	      enddo
            enddo
	    if (bot.lt.0.5) then
	      wk(i,j) = undef
	    else 
	      wk(i,j) = top/bot
            endif
          enddo
        enddo
        do i=1,imx
          do j=1,imy
            csst(i,j,imo) = wk(i,j)
          enddo
        enddo
  100 continue  
      print *,'base period =',iyr,'   month =',jmon
      return
      end
      subroutine idaycl (csst,imo,ida,dclimp,imx,imxp,imy,
     1 iadd,imin,imax,ierr)
c$    subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    dayclm1d      finds climatology of given day and month   
c   prgmmr: d. c. stokes     date: 1996-8-9             
c                                                                       
c abstract: temporally interpolates the climatology field of the day    
c           (defined by input month and day of the month) from the      
c           12 monthly climatology fields.  all fields are defined on   
c           the 1/4-degree OI analysis grid                            
c                                                                       
c program history log:                                                  
c   1996-08-09  diane c. stokes
c   2009-10-14  R. W. Reynolds
c                                                                       
c   input argument list:                                                
c    csst      - field of the 12 months of climatology                  
c              - dimension (imx,imy,12)                                  
c              - the third index is the month,                          
c              - from january (= 1) to december (=12)                   
c              - the first index is longitude
c              - the second index is latitude          
c      dclimp- climatology interpolated to day, expanded longitude
c
c     imo      - month of desired interpolated climatology              
c     ida      - day of desired interpolated climatology                
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
c     iadd     - =1, double to get number of extra points
c     imin     - = 2,  start locations in larger field of original
c     imax     - = 181 end   locations in larger field of original
c                                                                       
c   output argument list:                                               
c     dclim    - field of the temporally interpolated                   
c                climatology for imo and ida                            
c                expanded longitude
c     ierr     - equal to 0 if interpolated field found                
c              - equal to 1 if no interpolated field because of        
c              - bad imo or ida                
c                                                                       
c   output files:                                                       
c     unit 6 - printout                                               
c                                                                       
c remarks:  Note if the month is 2 and the day is 29, the day is 
c           changed to 28. This will change the day in the main program.
c           Thus before calling the subroutine, dummy month and
c           day files are created:
c           kmo = imo;  kda = ida; call idaycl (sstc,kmo,kda,...
c                                                                    
c attributes:                                                           
c   language: fortran 77                                                
c   machine:  Linux: warmcore/rainband                                                    
c                                                                       
c$                                                                    
c
      dimension ndays(12)
      dimension csst(imx,imy,12)
      dimension dclimp(imxp,imy)
c
      data ndays/31,28,31,30,31,30,31,31,30,31,30,31/
c  change february 29 to february 28: effect of leap year ignored       
      ierr = 0
c
      jda = ida
      if (imo.eq.2.and.jda.eq.29) jda = 28
c                                                                       
c  check for bad date                                                   
      if (imo.lt.1.or.imo.gt.12.or.jda.lt.1.or.jda.gt.ndays(imo)) then
      Print *,' RED - STOP -  EOTBIAS-DCLIM:'
      print 10, imo,jda
   10 format('Interpolated climate for month = ',i3,3x,'day =',i3)
      print 222
  222 format ('error in dclim')
      ierr = 1
      return
      endif
      halfm1 = real(ndays(imo))/2.
      if (jda.le.int(halfm1)) then
        if (imo.eq.1) then
          jmo = 12
        else
          jmo = imo-1
        endif
        d = halfm1 - real(jda) + .5
      else
        if (imo.eq.12) then
          jmo = 1
        else
          jmo = imo+1
        endif
        d = real(jda) - halfm1 - .5
      endif
      halfm2 = real(ndays(jmo))/2.
      span = halfm1+halfm2
c      print *,'idaycl: imo,jmo,im,jm,span',imo,jmo,im,jm,span
      do i=1,imx
        ip = i + iadd
        do j=1,imy
          t1 = csst(i,j,imo)
          t2 = csst(i,j,jmo)
          dclim = ((t2-t1)/span)*d + t1
          dclimp(ip,j) = dclim
        enddo
      enddo
      do j=1,imy
	dclimp(1,j)    = dclimp(imax,j)
	dclimp(imxp,j) = dclimp(imin,j)
      enddo
      return
      end
      subroutine maxmin (fld,imx,imy,undef,name)
c
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    maxmin       WRITES OUT statistics for the array fld
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-14
C
C ABSTRACT: WRITES OUT THE average, minimum, maximum and number of
c           values of array fld. Array values equal to undef are ignored
C
C PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS
C
C
C   INPUT ARGUMENT LIST:
C     fld      - ARRAY to be analyzed
C     IM       - FIRST DIMENSION OF fld array
C     jm       - SECOND DIMENSION OF fld array
c     undef    - indicates that a value in fld is undefined = -9999.
c     name     - a 8 character label of the fld array
C
C   OUTPUT FILES: PRINTOUT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C REMARKS:
C   The number of values in fld is printed out as 'inum ='
C$

c   compute maximum, minimum and average of fld and print
c   also set values lt undef to undef
c
      character *8 name
      dimension fld(imx,imy)
      fldmin = 1.e30
      fldmax = -fldmin
      inum = 0
      ave = 0.
      do 90 i=1,imx
      do 90 j=1,imy
      if (fld(i,j).gt.undef) then
        inum = inum + 1
        ave = ave + fld(i,j)
        if (fldmin.gt.fld(i,j))  fldmin = fld(i,j)
        if (fldmax.lt.fld(i,j))  fldmax = fld(i,j) 
      endif
   90 continue
      if (inum.gt.0) then
        r = 1./float(inum)
        ave = ave*r
      else
        fldmin = 0.
        fldmax = 0.
      endif
      print 88,ave,fldmin,fldmax,inum,name
   88 format ('ave =',f8.3,1x,'min =',f8.3,1x,
     1 'max =',f8.3,1x,'num =',i7,1x,a8)
      return 
      end
      subroutine mxmn2 (fld,imx,imy,ntyp,ityp,undef,name)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    mxmn2       WRITES OUT statistics for the array fld
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-14
C
C ABSTRACT: WRITES OUT THE average, minimum, maximum and number of
c           values of array fld. Array values equal to undef are ignored
C
C PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS
C
C
C   INPUT ARGUMENT LIST:
C     fld      - ARRAY to be analyzed
C     IM       - FIRST DIMENSION OF fld array
C     jm       - SECOND DIMENSION OF fld array
c     ntype    - third dimension of fld array
c     iytpe    - value of third dimension
c     undef    - indicates that a value in fld is undefined = -9999.
c     name     - an 8 character label of the fld array
C
C   OUTPUT FILES: PRINTOUT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C REMARKS:
C   The number of values in fld is printed out as 'inum ='
C$
      character *8 name
      dimension fld(imx,imy,ntyp)
      fldmin = 1.e30
      fldmax = -fldmin
      inum = 0
      ave = 0.
      k = ityp
      do 90 i=1,imx
      do 90 j=1,imy
      if (fld(i,j,k).gt.undef) then
        inum = inum + 1
        ave = ave + fld(i,j,k)
        if (fldmin.gt.fld(i,j,k))  fldmin = fld(i,j,k)
        if (fldmax.lt.fld(i,j,k))  fldmax = fld(i,j,k) 
      endif
   90 continue
      if (inum.gt.0) then
        r = 1./float(inum)
        ave = ave*r
      else
        fldmin = 0.
        fldmax = 0.
      endif
      print 88,ityp,ave,fldmin,fldmax,inum,name
   88 format (i2,1x,'ave =',f8.3,1x,'min =',f8.3,1x,
     1 'max =',f8.3,1x,'num =',i7,1x,a8)
      return 
      end
      subroutine getsit (situ,nsitu,iuby,iusp,xt,xtp,yt,
     1 imx,imxp,imy,nby,nsh,dclimp,lsp,drej,deg,ierr)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getsit       sums in situ data using ship and buoy data
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: The subroutine finds the sum of in situ SST obs and number 
c       of SST obs on a 2-degree grid and stores them in situ and nsitu.
c       It is called for each day of data added. It calls the subroutine
c       sum to add the ship and to add the buoy data. The weighting of ships
c       and buoys is determined by nsp and nby. Before calling this routine
c       zero must be called to zero out the arrays. After all the data has
c       been summed, a call to ave is needed to change the sum to the average.
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - sum
c
C   INPUT ARGUMENT LIST:
c     iuby      - 11 input unit for buoy data
c     iusp      - 12 input unit for ship data
C     IMX       - LENGTH OF THE LONGITUDE ARRAY
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMY       - LENGTH OF THE LATITUDE ARRAY
C                 (SECOND INDEX OF ANALYSIS GRID)
c     nby       - weighted number of ship obs, = 7		 
c     nsh       - weighted number of buoy obs, = 1
c     xt        - longitude (E) points: 1, 3, 5, .... 359
c     yt        - latitude points: -89, -87, ...., 89
c     xtp       - longitude expanded: -1, 1, 3, 5, .... 359, 361
c     lsp       - longitude expanded land sea mask
c                 land sea mask (0 ocean, 1 land) from 1-deg mask
c     dclimp    - climatology interpolated to day, expanded longitude
c     drej      - data REJECT LIMIT maximum ABSOLUTE anomaly SST used
c     deg       - grid interval in degrees = 2
c               - equal to 1 if data problems      
c   
C   OUTPUT ARGUMENT LIST:
c     situ     - sum (later average) of weighted ship and buoy superobs 
c                for ndays
c     nsitu    - number of weighted ship and buoy obs
c     ierr     - equal to 0 if OK             
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension lsp(imxp,imy)
      dimension xt(imx),xtp(imxp),yt(imy)
      dimension dclimp(imxp,imy)
      dimension situ(imx,imy),nsitu(imx,imy)
      print *,'get situ: iuby =',iuby,'  iusp =',iusp,
     1 '  nby =',nby,'  nsh =',nsh
c      print *,deg
c      call zero (situ,nsitu,imx,imy)
      call sum (situ,nsitu,iuby,xt,xtp,yt,imx,imxp,imy,nby,
     1   drej,dclimp,lsp,deg,ierr)
      if (ierr.eq.1) return
      call sum (situ,nsitu,iusp,xt,xtp,yt,imx,imxp,imy,nsh,
     1   drej,dclimp,lsp,deg,ierr)
c      call ave (situ,nsitu,undef,imx,imy,iadd)
c      call maxmin (situ,imx,imy,undef,'situ         ')
c      if (ierr.eq.1) return
      return
      end
      subroutine getsat (xsat,nxsat,iu,xt,xtp,yt,
     1 imx,imxp,imy,nst,dclimp,lsp,drej,deg,ierr)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    getsat       sums in satellite data 
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: The subroutine finds the sum of satellite SST obs and number 
c       of SST obs on a 2-degree grid and stores them in xsat and nxsat.
c       It is called for each day of data added. It calls the subroutine
c       sum to add the satellite data. Before calling this routine
c       zero must be called to zero out the arrays. After all the data has
c       been summed, a call to ave is needed to change the sum to the average.
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - 
c
C   INPUT ARGUMENT LIST:
c     iu        - 13 input unit for satellite data
C     IMX       - LENGTH OF THE LONGITUDE ARRAY
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMY       - LENGTH OF THE LATITUDE ARRAY
C                 (SECOND INDEX OF ANALYSIS GRID)
c     nst       - weighted number of satellite obs, = 1
c     xt        - longitude (E) points: 1, 3, 5, .... 359
c     yt        - latitude points: -89, -87, ...., 89
c     xtp       - longitude expanded: -1, 1, 3, 5, .... 359, 361
c     lsp       - longitude expanded land sea mask
c                 land sea mask (0 ocean, 1 land) from 1-deg mask
c     dclimp    - climatology interpolated to day, expanded longitude
c     drej      - data REJECT LIMIT maximum ABSOLUTE anomaly SST used
c     deg       - grid interval in degrees = 2
c               - equal to 1 if data problems      
c   
C   OUTPUT ARGUMENT LIST:
c     xsat     - sum (later average) of satellite superobs 
c                for ndays
c     nxsat    - number of satellite obs
c     ierr     - equal to 0 if OK             
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension lsp(imxp,imy)
      dimension xt(imx),xtp(imxp),yt(imy)
      dimension dclimp(imxp,imy)
      dimension xsat(imx,imy),nxsat(imx,imy)
c      print *,iu,nwt
      print *,'getsat: iu =',iu,'  nst =',nst
c      print *,deg
c      call zero (xsat,nxsat,imx,imy)
      call sum (xsat,nxsat,iu,xt,xtp,yt,imx,imxp,imy,nst,
     1   drej,dclimp,lsp,deg,ierr)
c      if (ierr.eq.1) return
c      call ave (xsat,nxsat,undef,imx,imy,iadd)
c      call maxmin (xsat,imx,imy,undef,'sat            ')
      return
      end
      subroutine zero (fsst,nobs,imx,imy)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    zero      zeros arrays
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: zeros the arrays fsst and nobs.
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IMX       - LENGTH OF THE LONGITUDE ARRAY
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMY       - LENGTH OF THE LATITUDE ARRAY
C                 (SECOND INDEX OF ANALYSIS GRID)
c   
C   OUTPUT ARGUMENT LIST:
c     fsst     - sum of superobs for ndays
c     nobs     - number of obs
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension fsst(imx,imy),nobs(imx,imy)
      do j=1,imy
        do i=1,imx
          fsst(i,j) = 0.
	  nobs(i,j) = 0
        enddo
      enddo
c      print *,'end subroutine zero'
c      print *,im,jm
      return
      end
      subroutine ave (fsst,nobs,undef,imx,imy)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    ave      average arrays
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: Converts sum of SST to average of SST. Before calling 
c      fsst is the sum of SST obs, after fsst is the average.
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IMX       - LENGTH OF THE LONGITUDE ARRAY
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMY       - LENGTH OF THE LATITUDE ARRAY
C                 (SECOND INDEX OF ANALYSIS GRID)
c     fsst     - sum of superobs for ndays
c     nobs     - number of obs
c   
C   OUTPUT ARGUMENT LIST:
c     fsst     - average of superobs for ndays
c                missing (nobs = 0) is set to -9999.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension fsst(imx,imy),nobs(imx,imy)
      do j=1,imy
        do i=1,imx
          if (nobs(i,j).le.0) then
	    fsst(i,j) = undef
	  else
	    fsst(i,j) = fsst(i,j)/float(nobs(i,j))
	  endif
        enddo
      enddo
c      print *,'end subroutine ave'
c      print *,im,jm
      return
      end
      subroutine sum (fsst,nobs,iu,xt,xtp,yt,imx,imxp,imy,nwt,
     1   drej,dclimp,lsp,deg,ierr)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    sum       sums in satellite data 
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: The general subroutine to reads the input data and produce a 
c       sum of the data and the number of obs. It is called by both
c       getsit and getsat. Before calling this routine
c       zero must be called to zero out the arrays. After all the data has
c       been summed, a call to ave is needed to change the sum to the average.
c       The data are converted to anomalies before it is stored. The 
c       climatology is interpolated to the data using intrp1.
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
c     iu        - input unit for data
C     IMX       - LENGTH OF THE LONGITUDE ARRAY
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMXp      - LENGTH OF THE expanded LONGITUDE ARRAY imxp=ixm +2*iadd
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMY       - LENGTH OF THE LATITUDE ARRAY
C                 (SECOND INDEX OF ANALYSIS GRID)
c     nwt      -  weighted number of observation
c     xt        - longitude (E) points: 1, 3, 5, .... 359
c     yt        - latitude points: -89, -87, ...., 89
c     xtp       - longitude expanded: -1, 1, 3, 5, .... 359, 361
c     lsp       - longitude expanded land sea mask
c                 land sea mask (0 ocean, 1 land) from 1-deg mask
c     dclimp    - climatology interpolated to day, expanded longitude
c     drej      - data REJECT LIMIT maximum ABSOLUTE anomaly SST used
c     deg       - grid interval in degrees = 2
c               - equal to 1 if data problems      
c   
C   OUTPUT ARGUMENT LIST:
c     fsst     - sum of superobs for ndays
c     nobs     - number of obs
c     ierr     - equal to 0 if OK             
c
c
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - intrp1, maxmin
C
c     Important numerical arrays: common/jrec/
c     common block : jrec in subroutine sum: variables are stored obs
c                data has been written in blocks of up to 10000
c     rsum(10000) - number of obs
c     rsst(10000) - sst 
c     rlon(10000) - longitude 
c     rlat(10000) - latitude            
C
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension fsst(imx,imy),nobs(imx,imy),lsp(imxp,imy)
      dimension dclimp(imxp,imy),xt(imx),xtp(imxp),yt(imy)
      common /jrec/ rsum(10000),rsst(10000),rlon(10000),rlat(10000)             
c      character*8 type
c      ibad = 0
      idrej = 0
      ierr = 0
c      print *,yt(1),yt(imy),xtp(1),xtp(imxp),imy,imxp,deg,imx
   50 read (iu,end=100) ict
c      print *,ict
      if (ict.eq.0) go to 100
c      if (ict.gt.0) stop 999
      read (iu,end=600) (rsum(m),m = 1,ict)
      read (iu,end=600) (rsst(m),m = 1,ict)
      read (iu,end=600) (rlat(m),m = 1,ict)
      read (iu,end=600) (rlon(m),m = 1,ict)
c      if (iu.eq.15) print 101,(rsum(m),m = 1,8)
c      if (iu.eq.15) print 101,(rsst(m),m = 1,8)
c  101 format (8f10.3)
      do m = 1,ict
        if (abs(rlat(m)).lt.yt(imy)) then
	  if (rlon(m).eq.0.) rlon(m) = 0.001
	  ii = 1 + nint( (rlon(m) - xt(1))/deg )
	  jj = 1 + nint( (rlat(m) - yt(1))/deg )
	  if (ii.lt.1.or.ii.gt.imx) go to 300
          if (jj.lt.1.or.jj.gt.imy) go to 300
	  call intrp1 (dclimp,imxp,imxp,imy,xtp,yt,lsp,
     1     rlon(m),rlat(m),clm,kerr)
c         subroutine intrp1 (cn,imc,ic,jc,cx,cy,ls,fx,fy,fn,ierr)
c          print *,ii,rlon(m),jj,rlat(m),rsst(m),clm,ierr
	  if (kerr.eq.0) then
	    isum = nint(rsum(m))*nwt
            ssta = rsst(m)-clm
c	    print *,ssta,drej,isum
	    if (abs(ssta).lt.drej) then
              add = ssta*float(isum)
              fsst(ii,jj) = fsst(ii,jj) + add
              nobs(ii,jj) = nobs(ii,jj) + isum
	    else
	      idrej = idrej + 1
	    endif
	  endif
	endif
      enddo
      go to 50
c 100 print *,'end subroutine sum'
  100 print *,
     1 '  number obs rejected, [abs(ssta)> drej] = ',idrej
c      call maxmin (fsst,imx,imy,undef,'subr sum')
      return
  600 print *,'RED-STOP! - EOTBIAS-SUM:'
      print *,'premature end of input data'
      ierr = 1
      return
  300 print *,'RED-STOP! - EOTBIAS-SUM:'
      print *,'  indices out of range'
      print *,'m,rlon,ii,rlat,jj',m,rlon(m),ii,rlat(m),jj
      ierr = 1
      return
      end
      subroutine scrall(gx,dat,ntyp,nsp,max,cj,crit,
     1 nfit,iuse,undef)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    scrall determines if data supports the EOT mode
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: This code checks determines the  fraction of the EOT mode’s variance
c supported by the data sampling (in situ and satellite).  Sampling is 
c adeqate if the fraction is greater than or equal to crit. The results are 
c  stored in iuse (see below). The test is described in
c   Smith, T.M., and R.W. Reynolds, 2003: Extended Reconstruction of Global 
c   Sea Surface Temperatures Based on COADS Data (1854 1997). J. Climate, 16,
c   1495-1510. 
c   
c
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
c
C   INPUT ARGUMENT LIST:
c      gx       - smodes stored only over the ocean, nsp=11506
c      dat      - in situ and satellite super obs from situ and xsat
c      ntyp     - the number of anomalies (in situ + satellite), = 2
c                  the nytp index is 1 for in situ and 2 for satellite
c      nsp      - the number of ocean points in the (180,90) grid, = 11506 
c      cj       - cosine of latitudes for gx and other ocean-only arrays
c      crit     - critical data/total area ratio (see EOT reference)
c      max      - the number of EOT modes, = 130
c                                     
C   OUTPUT ARGUMENT LIST:
c      iuse     - modes supported by satellite and/or in situ data
c                       determined by checking data using crit 
c            iuse is 0 if NOT supported by in situ or satellite
c            iuse is 1 if supported by in situ but NOT by satellite
c            iuse is 2 if NOT supported by in situ but supported by satellite
c            iuse is 3 if supported by in situ and by satellite
c            iuse must be = 3 for that EOT mode to be used in fit
c                 Thus, only modes that support both types of data are used
c      nfit     - number of modes with iuse=3
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
c* To get the best fit using screening regression.  Two data sets are
c* fit using the same set of modes adequate for both.
c* Inputs:  gx(,).......Array of spatial modes
c           dat().......Array of data
c           nsp   ......Number of spatial points
c           max.........Maximum number of modes to use
c           cj()........relative area weights
c           crit........critical fraction var of mode supported
c*
c* Outputs: iuse()......3=use, otherwise reject.
c           nfit........Number of modes used in the final fit.
c
      real gx(nsp,max),cj(nsp),dat(nsp,ntyp)
      integer iuse(max)
c
c* Check to see if a mode is adaquately supported by sampling of both.
c  iuse will be 0 if NOT supported by in situ or satellite
c  iuse will be 1 if supported by in situ but NOT by satellite
c  iuse will be 2 if NOT supported by in situ but supported by satellite
c  iuse will be 3 if supported by in situ and by satellite
c   nt = 1 for in situ, nt = 2 for satellite
c
c   in this version iuse must be = 3 to be used in fit
c
      do 2 m=1,max
       iuse(m) = 0
       do nt=1,ntyp
        sum1=0.0
        sum2=0.0
        kn=0
        do n=1,nsp
         e2c=gx(n,m)*gx(n,m)*cj(n)
         sum1=sum1+e2c
         if (dat(n,nt).gt.undef) then
          sum2=sum2+e2c
          kn=kn+1
         endif
        end do
        fva=sum2/sum1
        if(kn.lt.3) fva=0.0
        if(fva.ge.crit) then
	  iuse(m)= iuse(m) + nt
	endif
       end do
 2    continue
c
      nfit=0
      do m=1,max
       if(iuse(m).eq.3) nfit=nfit+1
      end do
      return
      end
      subroutine scrfta(gx,dat,dat1,iuse,nsp,max,cj,nfit,ft,
     1 smode,imx,imy,wt,wf,gxf,ityp,ntyp,undef)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    scrfta   prepares to find the eot weights for the data
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: This routine uses the modes accepted by subroutine scrall 
c    and given by iuse. First it moves the modes accepted from gx
c    which contains all modes to gxf which contains only accepted modes.
c    Then it processes each type of data separately (in situ and satellite)
c    by moving data from 2-D dat to 1-D dat1. It calls lsfit twice 
c    (in situ and satellite) to find the weights of each mode in 1-D wf 
c    and puts both the in situ and satellite modes in 2-D wt. 
c    wt include all the modes in the original order
c    with weights for modes not used set to 0. In then computes the 
c    in stiu and satellite anomalies using the weights and modes.
c    
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - lsfit, ludcmp, lubksb, maxmin, mxmn2
c
C   INPUT ARGUMENT LIST:
c      gx       - smodes stored only over the ocean, nsp=11506
c      gxf      - smodes stored only over the ocean, nsp=11506
c                 the same as gx but contain only modes accepted
c      dat      - in situ and satellite super obs from situ and xsat, 2-D
c                       linear solution code required only 1-dimension
c      dat1     - either in situ and satellite super obs as needed, 1-D
c      iuse     - modes supported by satellite and/or in situ data
c                       determined by checking data using crit 
c            iuse is 0 if NOT supported by in situ or satellite
c            iuse is 1 if supported by in situ but NOT by satellite
c            iuse is 2 if NOT supported by in situ but supported by satellite
c            iuse is 3 if supported by in situ and by satellite
c            iuse must be = 3 for that EOT mode to be used in fit
c      ntyp     - the number of anomalies (in situ + satellite), = 2
c      ityp     - ityp=1,ntype: = 1 for in situ and 2 for satellite
c      nsp      - the number of ocean points in the (180,90) grid, = 11506 
c      cj       - cosine of latitudes for gx and other ocean-only arrays
c      crit     - critical data/total area ratio (see EOT reference)
c      max      - the number of EOT modes, = 130
c      nfit     - number of modes with iuse=3
C      IMX      - LENGTH OF THE LONGITUDE ARRAY
C                (FIRST INDEX OF ANALYSIS GRID)
C      IMY      - LENGTH OF THE LATITUDE ARRAY
C                (SECOND INDEX OF ANALYSIS GRID)
c      smode    - EOT modes, max = 130 the number of modes
c      undef    - indicates variable is undefined = -9999.
c                                     
c   
C   OUTPUT ARGUMENT LIST:
c      ft       - final anomaly field - legacy - recomputed in eotbias-cor4
c      wt       - EOT weights (=0 if not used), 2-D
c      wf       - EOT weights - only modes accepted, later moved to wt, 1-D
c                       linear solution code required only 1-dimension
c      nfit     - Number of modes used in the final fit.
c
c
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - lsfit, maxmin, mxmn2 
C
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
c* To get the best fit using screening regression.
c* Inputs:  gx(,).......Array of spatial modes
c           dat().......Array of data
c           nspace......Number of spatial points
c           max.........Maximum number of modes to use
c           cj()........relative area weights
c           crit........critical fraction var of mode supported
c*
c* Outputs: ft()........On output this has the best fit
c           wt()........weights (-999.9 if not used).
c           nfit........Number of modes used in the final fit.
c* To reconstruct later sum over max: gx(n,m)*(wt(m)).
c
c
c  iuse is 0 if NOT supported by in situ or satellite
c  iuse is 1 if supported by in situ but NOT by satellite
c  iuse is 2 if NOT supported by in situ but supported by satellite
c  iuse is 3 if supported by in situ and by satellite
c
c   nt = 1 for in situ, nt = 2 for satellite
c
c   in this version iuse must be = 3 to be used in fit
c
      real wt(max,ntyp),gx(nsp,max),wf(max),gxf(nsp,max),cj(nsp)
      real dat(nsp,ntyp),ft(imx,imy,ntyp)
      dimension smode(imx,imy,max),dat1(nsp)
      integer iuse(max)
      nfit=0
      do m=1,max
c       wt(m,ityp)=undef
        if (iuse(m).eq.3) then
          nfit = nfit + 1
          do n=1,nsp
            gxf(n,nfit) = gx(n,m)
          enddo
        endif
      enddo
      do n=1,nsp
        dat1(n) = dat(n,ityp)
      enddo
c* Get the fit, return the weights and full fit.
      if(nfit.gt.0) then
        call lsfit(gxf,dat1,nsp,nsp,nfit,cj,wf)
        call maxmin (wf,nfit,1,undef,'orig wt ')
        n=0
        do m=1,max
          if (iuse(m).eq.3) then
            n = n + 1
            wt(m,ityp) = wf(n)
	  else
            wt(m,ityp) = 0.
          endif
c
        enddo
      endif
      do i=1,imx
        do j=1,imy
          sum=0.0
          do mode=1,max
c            if (wt(mode,ityp).gt.undef) then
	      sum = sum + smode(i,j,mode)*wt(mode,ityp)
c	    endif
          enddo
          ft(i,j,ityp) = sum
        enddo
      enddo
      call mxmn2 (ft,imx,imy,ntyp,ityp,undef,'ft fil  ')
      return
      end
      subroutine lsfit(gx,tsinc,nspace,lspace,max,cj,wt)
c  in calling routine:      call lsfit(gxf,dat1,nsp,nsp,nfit,cj,wf)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    lsfit  prepares to set up the Linear equation A*x=b 
C   PRGMMR: R. W. REYNOLDS   2009-10-15
C
C ABSTRACT: This sets up a solution to the Linear equation A*x=b, where 
c  A is a mmax by maxx array, b and x are the maxx arrays where x is
c  the unknown which when is the EOT weights. It uses the subroutines
c  ludcmp, lubksb. See Numerical recipes, sec 2.3.
c  http://www.nrbook.com/a/bookfpdf/f2-3.pdf  
c    
C PROGRAM HISTORY LOG:
C   2009-10-15  RICHARD W. REYNOLDS
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - ludcmp, lubksb
c
C   INPUT ARGUMENT LIST:
c      gx       - smodes stored only over the ocean, nsp=11506
c                 gxf in calling routine scrfta 
c                 contains only modes accepted
c      tsinc    - either in situ and satellite super obs as needed, 1-D
c                 dat1 in calling routine scrfta 
c      nspace   - the number of ocean points in the (180,90) grid, = 11506 
c                 nsp in calling routine scrfta 
c      lspace   - the number of ocean points in the (180,90) grid, = 11506 
c                 nsp in calling routine scrfta 
c      max      - Number of modes used in the final fit.
c                 nfit in calling routine scrfta 
c      cj       - cosine of latitudes for gx and other ocean-only arrays
c                                     
C   OUTPUT ARGUMENT LIST:
c      wt       - EOT weights - only modes accepted, 1-D
c                 wf in calling routine scrfta 
c
c Other IMPORTANT scalar variables
c     maxx      - 200, if nmodes > 200, maxx must be increased
c     d         - work scalar
c
c Other Important numerical arrays: 
c      a(maxx,maxx) - Work 2-D array
c      b(maxx)      - Work 1-D array
c      indx(maxx)   - Integer 1-D work array
c
C 
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
c* To get the least-squared weights for observed data to the given
c* number of spatial EOF patterns, by solving A w = b.
c
      parameter(maxx=200)
      real tsinc(*),gx(nspace,*),wt(*),cj(*)
      double precision a(maxx,maxx),b(maxx)
      integer indx(maxx)
c* Form matrix A
      do 4 i=1,max
      do 4 j=1,max
       a(i,j) = 0.
       do k=1,lspace
        if(tsinc(k).gt.-990.) a(i,j)=a(i,j)+gx(k,i)*gx(k,j)*cj(k)
       end do
 4    continue
c* Form array x
      do i=1,max
       b(i) = 0.
       do k=1,lspace
       	if(tsinc(k).gt.-990.)b(i)=b(i)+tsinc(k)*gx(k,i)*cj(k)
       end do
      end do
c* Get the LU decomposition of A and solve the linear system AX=B.
c* The solution X is returned in B.
      call ludcmp(a,max,maxx,indx,d)
      call lubksb(a,max,maxx,indx,b)
      do i=1,max
       wt(i)=b(i)
      end do
      return
      end
c
c* This does an LU decomposition on the matrix A, size nxn.  The
c* output is returned in A.  This is Crout's method with partial
c* pivoting, described in Numerical recipes, sec 2.3.
c  http://www.nrbook.com/a/bookfpdf/f2-3.pdf
      subroutine ludcmp(a,n,np,indx,d)
      parameter (nmax=200,tiny=1.0e-20)
      dimension indx(*),vv(nmax)
      double precision a(np,np)
      d=1.
      do 12 i=1,n
       aamax=0.
       do 11 j=1,n
        if(abs(a(i,j)).gt.aamax) aamax=abs(a(i,j))
 11    continue
       if (aamax.eq.0.) pause 'singular matrix.'
       vv(i)=1./aamax
 12   continue
      do 19 j=1,n
       if (j.gt.1) then
        do 14 i=1,j-1
         sum=a(i,j)
         if(i.gt.1)then
          do 13 k=1,i-1
           sum=sum-a(i,k)*a(k,j)
 13       continue
          a(i,j)=sum
         endif
 14     continue
       endif
       aamax=0.
       do 16 i=j,n
        sum=a(i,j)
        if(j.gt.1)then
         do 15 k=1,j-1
          sum=sum-a(i,k)*a(k,j)
 15      continue
         a(i,j)=sum
        endif
        dum=vv(i)*abs(sum)
        if(dum.ge.aamax) then
         imax=i
         aamax=dum
        endif
 16    continue
       if(j.ne.imax)then
        do 17 k=1,n
         dum=a(imax,k)
         a(imax,k)=a(j,k)
         a(j,k)=dum
 17     continue
        d=-d
        vv(imax)=vv(j)
       endif
       indx(j)=imax
       if(j.ne.n)then
        if(a(j,j).eq.0.)a(j,j)=tiny
         dum=1./a(j,j)
         do 18 i=j+1,n
          a(i,j)=a(i,j)*dum
 18      continue
       endif
 19   continue
      if(a(n,n).eq.0.)a(n,n)=tiny
      return
      end
c
c* This solves the system Ax=b, where here A is the LU decomposition of
c* the matrix A that was passed into ludcmp()
      subroutine lubksb(a,n,np,indx,b)
c  See Numerical recipes, sec 2.3.
c  http://www.nrbook.com/a/bookfpdf/f2-3.pdf  
      dimension indx(*)
      double precision a(np,np),b(*)
      ii=0
      do 12 i=1,n
       ll=indx(i)
       sum=b(ll)
       b(ll)=b(i)
       if(ii.ne.0)then
        do 11 j=ii,i-1
         sum=sum-a(i,j)*b(j)
 11     continue
       else if(sum.ne.0.) then
        ii=i
       endif
       b(i)=sum
 12   continue
      do 14 i=n,1,-1
       sum=b(i)
       if(i.lt.n)then
        do 13 j=i+1,n
         sum=sum-a(i,j)*b(j)
 13     continue
       endif
       b(i)=sum/a(i,i)
 14   continue
      return
      end
      subroutine zonal (zon,zorg,norg,nzon,situ,nsitu,
     1 xsat,nxsat,imx,imy,minob,minz,njj,yt,ccos)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    zonal       Computes satellite minus in situ zonal bias 
C   PRGMMR: R. W. REYNOLDS   2009-10-19
C
C ABSTRACT: The subroutine produces zonal average and number of zonal
c           observations: both the original and smoothed.
c           The smoothing is over njj*2 + 1. Smoothed values with less than 
c           minz obs are extrapolated from last valid values to the poles
c           The zonal bias is subtracted for the satellite data before the
c           eot weights are computed and added back on to the satellite 
c           anomalies in the next code (eotbias-cor4-doc.f).
c           The final bias is not affected by the zonal correction
c           where the EOT anomalies can be computed.
c           Note the in situ data are noisy and are smoothed by 
c           subroutine smooth (situs and nsitus)
c           i.e., call zonal (zon,zorg,norg,nzon,situs,nsitus,
c                1 xsat,nxsat,imx,imy,minob,minz,njj,yt,ccos)
c
c
C PROGRAM HISTORY LOG:
C   2009-10-19  RICHARD W. REYNOLDS
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - sum
c
C   INPUT ARGUMENT LIST:
c     minz      - minimum number of smoothed zonal points used
c                 values less than this are extrapolated poeward 
c                 from the last valid values      		
c     njj       - 2*njj+1 = number of latitude points for zonal smoothing
c     minob     - minimum number of obs used in original zonal average
C     IMX       - LENGTH OF THE LONGITUDE ARRAY
C                 (FIRST INDEX OF ANALYSIS GRID)
C     IMY       - LENGTH OF THE LATITUDE ARRAY
C                 (SECOND INDEX OF ANALYSIS GRID)
c     xsat      - average satellite super obs for ndays
c                    super obs are the mean of the obs at a grid point
c     nxsat     - number of weighted satellite obs
c     situ      - smoothed average weighted ship and buoy superobs
c                 situs in main
c     nsitu     - smoothed number of weighted ship and buoy obs
c                 nsitus in main
c     yt        - latitude points: -89, -87, ...., 89
c     ccos      - cosine of latitudes for yt and other (lon,lat) arrays
c   
C   OUTPUT ARGUMENT LIST:
c      zon     - smoothed zonal bias 
c      nzon    - smoothed zonal number of points
c      norg    - unsmoothed zonal number of points
c      zorg    - unsmoothed zonal bias from smoothed in situ data
c                       and unsmoothed satellite data
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
c
c set in main: minz = 200
c set in main: njj = 10
c set in main: minob = 1
c
      dimension zon(imy),norg(imy),zorg(imy),nzon(imy)
      dimension xsat(imx,imy),nxsat(imx,imy),ccos(imy)
      dimension situ(imx,imy),nsitu(imx,imy),yt(imy)
c
      do j=1,imy
        zsum = 0.
	nzsum = 0
        do i=1,imx
	  if (nsitu(i,j).gt.minob.and.nxsat(i,j).gt.minob) then
	    nzsum = nzsum + 1
	    zsum = zsum + xsat(i,j) - situ(i,j)
	  endif
	enddo
	norg(j) = nzsum
	if (nzsum.gt.0) then
	  zorg(j) = zsum/float(nzsum)
	else
	  zorg(j) = 0.
	endif
      enddo
      do j=1,imy
        jp = j + njj
	if (jp.gt.imy) jp = imy 
        jm = j - njj
	if (jm.lt.1)   jm = 1
        zsum = 0.
	nzsum = 0
	czsum = 0.
	do jj=jm,jp
	  zsum = zsum + zorg(jj)*float(norg(jj))*ccos(jj)
	  nzsum = nzsum + norg(jj)
	  czsum = czsum + float(norg(jj))*ccos(jj)
	enddo
  220	if (nzsum.gt.minz) go to 210
	jadd = 0
	jp = jp + 1
	if (jp.le.imy) then
	  zsum = zsum + zorg(jp)*float(norg(jp))*ccos(jp)
	  nzsum = nzsum + norg(jp)
	  czsum = czsum + float(norg(jp))*ccos(jp)
	  jadd = jadd + norg(jp)
	endif  
	jm = jm - 1
	if (jm.ge.1) then
	  zsum = zsum + zorg(jm)*float(norg(jm))*ccos(jm)
	  nzsum = nzsum + norg(jm)
	  czsum = czsum + float(norg(jm))*ccos(jm)
	  jadd = jadd + norg(jm)
	endif
	if (jadd.gt.0) go to 220  
  210   if (nzsum.gt.0) zon(j) = zsum/czsum
	if (nzsum.gt.minz) then
	  nzon(j) = nzsum
	else
	  nzon(j) = 0.
	endif
      enddo
c
c  find North-most non-missing value
c  then fill more northerly value with this one  
c	
      imy2 = imy/2
      imy2p = imy2 + 1
      do j=imy,imy2p,-1
        jn = j
	if (nzon(j).gt.0) go to 100
      enddo
c      	
c  all values missing: NH zon set to 0	
c
      do j=imy2p,imy
        zon(j) = 0.
      enddo
      go to 200
  100 jnp = jn + 1
      do j=jnp,imy
        zon(j) = zon(jn)
      enddo
c
c  find South-most non-missing value
c  then fill more southerly values with this one 
c	
      do j=1,imy2
        js = j
	if (nzon(j).gt.0) go to 400
      enddo
c      	
c  all values missing: SH zon set to 0	
c
  200 do j=1,imy2
        zon(j) = 0.
      enddo
      go to 300
  400 jsm = js - 1
      do j=1,jsm
        zon(j) = zon(js)
      enddo
  300 js = 5
      print 108,( yt(j),j=js,imy,10)
  108 format (' lat =',9f8.3)
      print 109,(zon(j),j=js,imy,10)
  109 format ('bias =',9f8.3)
c
c      do j=1,imy   
c        print 101,yt(j),zorg(j),norg(j),zon(j),nzon(j)
c  101   format (f6.1,3x,'zorg =',f10.3,3x,'norg =',i5,3x,
c     1   'zon =',f10.3,3x,'nzon =',i5)
c      enddo
      return
      end
      subroutine smooth (fld,nfld,flds,nflds,ioff,joff,imx,imy)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    smooth    smooths superobs fields
C   PRGMMR: R. W. REYNOLDS   2009-10-19
C
C ABSTRACT: The general subroutine smooths the superobs fields. Note
c           that the original field fld and the smoothed flds are sums
c           of the SST observations in nfld and nflds at this point. 
c           ioff and joff indicate the range of the smoothing.
c           in 'main,' subroutine 'ave' is called to produce the average
c           For ioff the i range for smoothing is i-ioff to i+ioff by 1
c           For joff the j range for smoothing is j-joff to j+joff by 1
c           For joff=ioff=0 there is no smoothing. Note the smoothed 
c           fields are only used to find the zonal average in zonal
c
C PROGRAM HISTORY LOG:
C   2009-10-19  RICHARD W. REYNOLDS
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - sum
c
C   INPUT ARGUMENT LIST:
c     fld      - sum of superobs for ndays
c     nfld     - number of obs
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C                (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C                (SECOND INDEX OF ANALYSIS GRID)
c     ioff     - 2*ioff + 1 =number of longitude boxes smoothed 
c     joff     - 2*joff + 1 = number of latitude boxes smoothed 
c   
C   OUTPUT ARGUMENT LIST:
c     flds     - sum of smoothed superobs for ndays
c     nflds    - number of smoothed obs
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension fld(imx,imy), nfld(imx,imy)
      dimension flds(imx,imy), nflds(imx,imy)
      do i=1,imx
	ip = i + ioff
	im = i - ioff
	do j=1,imy
	  jp = j + joff
	  jm = j - joff
	  if (jp.gt.imy) jp = imy
	  if (jm.lt.1)   jm = 1
	  sum = 0.
	  nsum = 0
	  do ii=im,ip
	    is = ii
	    if (is.gt.imx) is = is - imx
	    if (is.lt.1)   is = is + imx
	    do jj=jm,jp
	      sum  = sum  +  fld(is,jj)
	      nsum = nsum + nfld(is,jj)
	    enddo
	  enddo
	  flds(i,j)  = sum
	  nflds(i,j) = nsum
	enddo
      enddo
      return
      end	  
       FUNCTION IW3JDN(IYEAR,MONTH,IDAY)
C$$$   SUBPROGRAM  DOCUMENTATION  BLOCK
C
C SUBPROGRAM: IW3JDN         COMPUTE JULIAN DAY NUMBER
C   AUTHOR: JONES,R.E.       ORG: W342       DATE: 90-06-04
C
C ABSTRACT: COMPUTES JULIAN DAY NUMBER FROM YEAR (4 DIGITS), MONTH,
C   AND DAY. IW3JDN IS VALID FOR YEARS 1583 A.D. TO 3300 A.D.
C   JULIAN DAY NUMBER CAN BE USED TO COMPUTE DAY OF WEEK, DAY OF
C   YEAR, RECORD NUMBERS IN AN ARCHIVE, REPLACE DAY OF CENTURY,
C   FIND THE NUMBER OF DAYS BETWEEN TWO DATES.
C
C PROGRAM HISTORY LOG:
C   87-01-16  R.E.JONES
C   89-02-02  R.E.JONES   CHANGE TO MICROSOFT FORTRAN 4.10
C   90-06-04  R.E.JONES   CHANGE TO SUN FORTRAN 1.3
C   91-03-29  R.E.JONES   CONVERT TO Silicon Graphics FORTRAN
C   93-03-29  R.E.JONES   ADD SAVE STATEMENT
C
C USAGE:   II = IW3JDN(IYEAR,MONTH,IDAY)
C
C   INPUT VARIABLES:
C     NAMES  INTERFACE DESCRIPTION OF VARIABLES AND TYPES
C     ------ --------- -----------------------------------------------
C     IYEAR  ARG LIST  INTEGER*4 YEAR           ( 4 DIGITS)
C     MONTH  ARG LIST  INTEGER*4 MONTH OF YEAR   (1 - 12)
C     IDAY   ARG LIST  INTEGER*4 DAY OF MONTH    (1 - 31)
C
C   OUTPUT VARIABLES:
C     NAMES  INTERFACE DESCRIPTION OF VARIABLES AND TYPES
C     ------ --------- -----------------------------------------------
C     IW3JDN FUNTION   INTEGER*4 JULIAN DAY NUMBER
C                      JAN. 1,1960 IS JULIAN DAY NUMBER 2436935
C                      JAN. 1,1987 IS JULIAN DAY NUMBER 2446797
C
C   REMARKS: JULIAN PERIOD WAS DEVISED BY JOSEPH SCALIGER IN 1582.
C     JULIAN DAY NUMBER #1 STARTED ON JAN. 1,4713 B.C. THREE MAJOR
C     CHRONOLOGICAL CYCLES BEGIN ON THE SAME DAY. A 28-YEAR SOLAR
C     CYCLE, A 19-YEAR LUNER CYCLE, A 15-YEAR INDICTION CYCLE, USED
C     IN ANCIENT ROME TO REGULATE TAXES. IT WILL TAKE 7980 YEARS
C     TO COMPLETE THE PERIOD, THE PRODUCT OF 28, 19, AND 15.
C     SCALIGER NAMED THE PERIOD, DATE, AND NUMBER AFTER HIS FATHER
C     JULIUS (NOT AFTER THE JULIAN CALENDAR). THIS SEEMS TO HAVE
C     CAUSED A LOT OF CONFUSION IN TEXT BOOKS. SCALIGER NAME IS
C     SPELLED THREE DIFFERENT WAYS. JULIAN DATE AND JULIAN DAY
C     NUMBER ARE INTERCHANGED. A JULIAN DATE IS USED BY ASTRONOMERS
C     TO COMPUTE ACCURATE TIME, IT HAS A FRACTION. WHEN TRUNCATED TO
C     AN INTEGER IT IS CALLED AN JULIAN DAY NUMBER. THIS FUNCTION
C     WAS IN A LETTER TO THE EDITOR OF THE COMMUNICATIONS OF THE ACM
C     VOLUME 11 / NUMBER 10 / OCTOBER 1968. THE JULIAN DAY NUMBER
C     CAN BE CONVERTED TO A YEAR, MONTH, DAY, DAY OF WEEK, DAY OF
C     YEAR BY CALLING SUBROUTINE W3FS26.
C
C ATTRIBUTES:
C   LANGUAGE: SiliconGraphics 3.5 FORTRAN 77
C   MACHINE:  SiliconGraphic IRIS-4D/25, 35, INDIGO
C
C$$$
C
       SAVE
C
       IW3JDN  =    IDAY - 32075
     &            + 1461 * (IYEAR + 4800 + (MONTH - 14) / 12) / 4
     &            + 367 * (MONTH - 2 - (MONTH -14) / 12 * 12) / 12
     &            - 3 * ((IYEAR + 4900 + (MONTH - 14) / 12) / 100) / 4
       RETURN
       END
       SUBROUTINE W3FS26(JLDAYN,IYEAR,MONTH,IDAY,IDAYWK,IDAYYR)
C$$$   SUBPROGRAM  DOCUMENTATION  BLOCK
C
C SUBPROGRAM: W3FS26         YEAR, MONTH, DAY FROM JULIAN DAY NUMBER
C   AUTHOR: JONES,R.E.       ORG: W342       DATE: 87-03-29
C
C ABSTRACT: COMPUTES YEAR (4 DIGITS), MONTH, DAY, DAY OF WEEK, DAY
C   OF YEAR FROM JULIAN DAY NUMBER. THIS SUBROUTINE WILL WORK
C   FROM 1583 A.D. TO 3300 A.D.
C
C PROGRAM HISTORY LOG:
C   87-03-29  R.E.JONES
C   88-07-06  R.E.JONES  CHANGE TO MICROSOFT FORTRAN 4.10
C   90-06-11  R.E.JONES  CHANGE TO SUN FORTRAN 1.3
C   91-03-29  R.E.JONES  CONVERT TO SiliconGraphics FORTRAN
C   93-03-29  R.E.JONES  ADD SAVE STATEMENT
C
C USAGE:  CALL W3FS26(JLDAYN,IYEAR,MONTH,IDAY,IDAYWK,IDAYYR)
C
C   INPUT VARIABLES:
C     NAMES  INTERFACE DESCRIPTION OF VARIABLES AND TYPES
C     ------ --------- -----------------------------------------------
C     JLDAYN ARG LIST  INTEGER*4 JULIAN DAY NUMBER
C
C   OUTPUT VARIABLES:
C     NAMES  INTERFACE DESCRIPTION OF VARIABLES AND TYPES
C     ------ --------- -----------------------------------------------
C     IYEAR  ARG LIST  INTEGER*4 YEAR  (4 DIGITS)
C     MONTH  ARG LIST  INTEGER*4 MONTH
C     IDAY   ARG LIST  INTEGER*4 DAY
C     IDAYWK ARG LIST  INTEGER*4 DAY OF WEEK (1 IS SUNDAY, 7 IS SAT)
C     IDAYYR ARG LIST  INTEGER*4 DAY OF YEAR (1 TO 366)
C
C   REMARKS: A JULIAN DAY NUMBER CAN BE COMPUTED BY USING ONE OF THE
C     FOLLOWING STATEMENT FUNCTIONS. A DAY OF WEEK CAN BE COMPUTED
C     FROM THE JULIAN DAY NUMBER. A DAY OF YEAR CAN BE COMPUTED FROM
C     A JULIAN DAY NUMBER AND YEAR.
C
C      IYEAR (4 DIGITS)
C
C      JDN(IYEAR,MONTH,IDAY) = IDAY - 32075
C    X            + 1461 * (IYEAR + 4800 + (MONTH - 14) / 12) / 4
C    Y            + 367 * (MONTH - 2 - (MONTH -14) / 12 * 12) / 12
C    Z            - 3 * ((IYEAR + 4900 + (MONTH - 14) / 12) / 100) / 4
C
C      IYR (4 DIGITS) , IDYR(1-366) DAY OF YEAR
C
C      JULIAN(IYR,IDYR) = -31739 + 1461 * (IYR + 4799) / 4
C    X                    -3 * ((IYR + 4899) / 100) / 4 + IDYR
C
C      DAY OF WEEK FROM JULIAN DAY NUMBER, 1 IS SUNDAY, 7 IS SATURDAY.
C
C      JDAYWK(JLDAYN) = MOD((JLDAYN + 1),7) + 1
C
C      DAY OF YEAR FROM JULIAN DAY NUMBER AND 4 DIGIT YEAR.
C
C      JDAYYR(JLDAYN,IYEAR) = JLDAYN -
C X  (-31739+1461*(IYEAR+4799)/4-3*((IYEAR+4899)/100)/4)
C
C      THE FIRST FUNCTION WAS IN A LETTER TO THE EDITOR COMMUNICATIONS
C      OF THE ACM  VOLUME 11 / NUMBER 10 / OCTOBER, 1968. THE 2ND
C      FUNCTION WAS DERIVED FROM THE FIRST. THIS SUBROUTINE WAS ALSO
C      INCLUDED IN THE SAME LETTER. JULIAN DAY NUMBER 1 IS
C      JAN 1,4713 B.C. A JULIAN DAY NUMBER CAN BE USED TO REPLACE A
C      DAY OF CENTURY, THIS WILL TAKE CARE OF THE DATE PROBLEM IN
C      THE YEAR 2000, OR REDUCE PROGRAM CHANGES TO ONE LINE CHANGE
C      OF 1900 TO 2000. JULIAN DAY NUMBERS CAN BE USED FOR FINDING
C      RECORD NUMBERS IN AN ARCHIVE OR DAY OF WEEK, OR DAY OF YEAR.
C
C ATTRIBUTES:
C   LANGUAGE: SiliconGraphics 3.5 FORTRAN 77
C   MACHINE:  SiliconGraphics IRIS-4D/25, 35, INDIGO
C
C$$$
C
       SAVE
C
       L      = JLDAYN + 68569
       N      = 4 * L / 146097
       L      = L - (146097 * N + 3) / 4
       I      = 4000 * (L + 1) / 1461001
       L      = L - 1461 * I / 4 + 31
       J      = 80 * L / 2447
       IDAY   = L - 2447 * J / 80
       L      = J / 11
       MONTH  = J + 2 - 12 * L
       IYEAR  = 100 * (N - 49) + I + L
       IDAYWK = MOD((JLDAYN + 1),7) + 1
       IDAYYR = JLDAYN -
     &  (-31739 +1461 * (IYEAR+4799) / 4 - 3 * ((IYEAR+4899)/100)/4)
       RETURN
       END
