c      PROGRAM OISST4-doc.f
c   
c$  MAIN PROGRAM DOCUMENTATION BLOCK
c                .      .    .                                       .
c MAIN PROGRAM:  oisst4-doc.f    COMPUTES 1/4-DEG SST OI ANALYSIS
c   PRGMMR: R.W. REYNOLDS   DATE: 2009-09-08
c
c ABSTRACT: reads super observations from buoys, ships, ice, day
c   satellite and night satellite and previous analysis (first
c   guess).  the program then computes a new analysis using optimum
c   interpolation (Oi) following the technique of lorenc (monthly
c   weather review, 1981).  The program outputs the new SST Oi (deg c)
c   analysis on a 1/4 deg grid. it also outputs the sampling +
c   random error as well as a bias error. 
c   Both random + sampling and bias error fields are in (deg-C).
c   To combine these two error fields,
c   it is necessary to take the square root of the
c   sum of the two squares of the 2 error fields.
c   The OI procedure solves a set of linear equations at each lat/long box.
c   Only nearby points are used to reduced the number of equations needed.
c   Details of the equations can be found in:
c   Reynolds, R. W., T. M. Smith, C. Liu, D. B. Chelton, K. S. Casey, 
c     and M. G. Schlax, 2007: Daily high-resolution blended analyses 
c     for sea surface temperature. J. Climate, 20, 5473-5496.
c
C   NOTE: 
C         OI    =  OPTIMUM INTERPOLATION.
C         SST   =  SEA SURFACE TEMPERATURE
C         OB(S) =  OBSERVATION(S)
c         FG    =  First Guess
C
C PROGRAM HISTORY LOG:
c   2009-09-08  Richard W. Reynolds, Upgrade to 1/4 
c
c   Input Data on unit 5 needed by program (input from script)
c
c  The 1st input record is the title
c e.g., title='Daily-OI-V2, Interim, Data (Ship, Buoy, AVHRR: NOAA17&18, NCEP-ice)' 
c
c  The 2nd input record is the date ,tm; tm is the script variable
c  tm has eight digits yyyymmdd where yyyy is 4 digit the year (iyr), 
c           mm is the 2 digit month (imo), dd is the 2 digit day (ida)
c  The Fortran code variables are iyr, imo, and ida
c        e.g., tm=20060101
c
c  The 3rd input record has 7 parameters: rmax, nmax, ifil, rej, nbias, nsat, iwarn
c     rmax is the maximum x or y distance (km) over which obs are used for each 
c       gridded OI value, 
c        e.g., rmax = 400
c     nmax is the maximum number of obs are used. Must be <201, 
c        e.g., nmax=22
c     ifil is number of times 1-2-1 smoothing is done; <1 no smoothing,
c        e.g., ifil=1 
c     rej is the REJECT LIMIT for ABSOLUTE super obs increment, 
c        ABSOLUTE SST INCREMENTS > REJ ARE REJECTED
c        e.g., rej=5.
c    nbias is the number of EOT bias estimates to be read, may be 0
c        e.g., nbias=4
c    nsat is the number independent satellite bias data sets 
c      day and night obs from the same satellite are not independent 
c        e.g., nsat=1
c    iwarn is a percentage of satellite data: if lower warning message printed 
c        e.g., iwar=20
c
c  All remaining records are input and output file names which must be in order
c  NOTE: OISST can have a large number of input file data sets
c  examples of file names follow the description
c  the script variable is given to the left of the =. 
c
c  The 1st file name is the correlation scales
c CORXY=/raid2/sstday/stats/cor4sm-stat-v2
c
c  The 2nd file name is the estimated first guess variance error
c FGVAR=/raid2/sstday/stats/path-incr-var-stat-v2
c
c  The 3rd file name is the correlation correction due to use of first guess 
c      from previous analysis instead of anomaly. Now set to 1 everywhere
c RFAC=/raid2/sstday/stats/error-cor-stat-v2
c
c  The 4th file name is the first guess
c   TMg is the first guess time which is one data earlier that TM
c FGSST=/raid2/sstday/$YEARg/eotanal/nv1718-intv2/sst4-nv1718-eot-intv2.$TMg'
c
c  The 5th file name is the land/sea mask
c  Land SEA TAGS: 0 IS LAND; 1 IS SEA
c MASK=/raid/rreynold/sstday/clim/quarter-mask-extend
c
c  The 6th file name is the monthly climatology (from weekly OI: 1971-2000)
c    (interpolated to daily from monthly
c CLIM=/raid/rreynold/sstday/clim/oiclm4.mon
c
c  The 7th file name is the output OI: this is the only output file
c OISST=/raid2/sstday/$YEAR/eotanal/nv1718-intv2/sst4-nv1718-eot-intv2.$TM
c
c  The 8th file name is the residual bias variance
c  Next nbias files follow: nbias may be 0, in the example nbias=4
c  Each of these files is from the EOT bias correction with EOT bias error
c ANALDayNV17=/raid2/sstday/$YEAR/eotcor-intv2/navy-err/dsat17/dsat17.$TM         
c ANALNteNV17=/raid2/sstday/$YEAR/eotcor-intv2/navy-err/nsat17/nsat17.$TM         
c ANALDayNV18=/raid2/sstday/$YEAR/eotcor-intv2/navy-err/dsat18/dsat18.$TM         
c ANALNteNV18=/raid2/sstday/$YEAR/eotcor-intv2/navy-err/nsat18/nsat18.$TM         
c
c  Then in file name pairs until end of data: 
c    Odd file name:  "SST noise to signal ratio (std dev)" 
c    Even file name: "SST superobs data file"
c BUOYNSR=/raid2/sstday/stats/buoy4sm-nsr-stat-v2
c BUOYDAT=/raid2/sstday/$YEAR/sobs/coad/buoy/buoy.$TM
c SHIPNSR=/raid2/sstday/stats/ship4sm-nsr-stat-v2
c SHIPDAT=/raid2/sstday/$YEAR/cobs/coad/ship/ship.$TM
c SATNSRDayNV17=/raid2/sstday/stats/day-path4sm-nsr-stat-v2
c SATDATDayNV17=/raid2/sstday/$YEAR/cobs/eotcor-intv2/navy-err/dsat17/dsat17.$TM
c SATNSRNteNV17=/raid2/sstday/stats/nte-path4sm-nsr-stat-v2
c SATDATNteNV17=/raid2/sstday/$YEAR/cobs/eotcor-intv2/navy-err/nsat17/nsat17.$TM
c SATNSRDayNV18=/raid2/sstday/stats/day-path4sm-nsr-stat-v2
c SATDATDayNV18=/raid2/sstday/$YEAR/cobs/eotcor-intv2/navy-err/dsat18/dsat18.$TM
c SATNSRNteNV18=/raid2/sstday/stats/nte-path4sm-nsr-stat-v2
c SATDATNteNV18=/raid2/sstday/$YEAR/cobs/eotcor-intv2/navy-err/nsat18/nsat18.$TM
c ICEDAT=/raid2/sstday/$YEAR/sobs/ncep/icesst/icesst.$TM
c ICENSR=/raid2/sstday/stats/cice4sm-nsr-stat-v2 
c
c   Fortran files: only units 6 and 91 are output files
c   Files with TM ($TM) in name vary with time, the others are static
c    variable    unit    name
c                  5   input data parameters and file names
c      iucor    =  1   CORXY
c      iufgv    =  8   FGVAR
c      iufac    =  9   RFAC
c      iufg     =  2   FGSST
c      iumask   =  3   MASK
c      iuclm    =  4   CLIM
c      iures    = 10   RESID
c      iubs     = 15   ANAL???????      
c      iunsr    = 11   ???NSR
c      iudat    = 12   ???DAT
c                  6   printed messages  (OUTPUT)
c      iuout    = 91   OISST             (OUTPUT) 
c
C     PARAMETERS in Main:
C      PARAMETER (IMX=1440,IMY=720,imbx=182,imby=92) in main
c      PARAMETER (JREC=10000) in subroutine getdat
c      parameter (imax=200,kmax=imax*20) in subroutine sstmap
c      OI analysis grid, 1/4 spatial degree (1440,720)
c              - IMX IS LONGITUDE DIMENSION OF analysis (0.125 TO 359.875)
C              - IMX = 1440                               (0.125E TO 359.875E)
C              - IMY IS LATITUDE DIMENSION OF analysis  (-89.875 TO 89.875)
C                IMY = 720                                (89.875S TO 89.875N)
c     Bias correction grid, 2 spatial degree (182,92)
c              - IMbX IS LONGITUDE DIMENSION OF bias fields   (-1 to 361 )
C              - IMbX = 182                                 (1W TO 361E or 1E)
C              - IMbY IS LATITUDE DIMENSION OF bias fields         (-91 to 91)
C              - IMbY = 92                                  (91S TO 91N)
c      Note: for interpolation 
c             1/4-deg grid must lie within 2-deg grid
C              - JREC IS MAXIMUM NUMBER OF TOTAL SUPER OBS per block
C              - Input data: 10,000
C              - IMAX = 200, MAXIMUM Number OF LINEAR EQUATION per box
C              - KMAX = 20*IMAX, MAXIMUM NUMBER OF DATA POINTS
C              - PER BOX. This will be reduced to imax after sorting
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - GETDAT, GETFG,  SMOOTH, GENGRD, 
C              - SSTMAP, DECOMP, ESTIM,  PUTOI,  MAXMIN
c              - xmooth, idaycl, intrp, vbias, qsort
C   Functions CALLED:
C     UNIQUE:  - integer*2 function compar(arg1,arg2), needed for qsort
c                  call qsort (array, len, isize, compar) 
c                  array is the 1-d array to be sorted
c                  len is the length of array
c                  isize is the number of bytes in 1 array element                 
c                integer function iw3jdn (iyr,imo,ida) returns julian day
c                  from year (4 digits), month, and day
C     LIBRARY:
C       lapack
C
C   EXIT STATES: There are 3 main categories
c   call exit (1) : Normal End, code color 'green'
c
c   call exit (10): Normal End with WARNING, code color 'yellow'
c                   The problem should be corrected and the code rerun.
c   The following warning 'yellow' errors may occur 
c        OI-MAIN: GRID INDEX OUT OF RANGE
c        OI-MAIN: Problem in SSTMAP, Number of INPUT OI SST DATA too low
c        OI-MAIN: Problem in SSTMAP, Number of OI results too low
c        OI-MAIN: Problem in GETDAT
c        OI-MAIN: Problem with OI data/analysis
c        OI-MAIN: OI-GETDAT: GRID INDEX OUT OF RANGE
c        OI-MAIN: OI-GETDAT: PREMATURE END OF INPUT DATA: unit =
c        OI-MAIN: OI-GETDAT: no noise to signal ratio: unit =
c
c   call exit (100): Fatal error, Premature STOP, code color 'red'
c                    The problem MUST be corrected and the code rerun.
c   The following fatal 'red' errors may occur 
c        OI-MAIN: OPEN ERROR OI OUTPUT DATA MISSING - NO OI COMPUTED
c        OI-MAIN: Missing INPUT DATA from oiparm - NO OI COMPUTED
c        OI-MAIN: NO FIRST GUESS AVAILABLE - NO OI COMPUTED
c        OI-MAIN: NO DATA AVAILABLE OR NO OBS ADDED - NO OI COMPUTED
c        OI-MAIN: NO CLIMATE SST AVAILABLE - NO OI COMPUTED
c        OI-MAIN: NO LAND SEA MASK - NO OI COMPUTED
c        OI-MAIN: initial stats files are missing - NO OI COMPUTED
c        OI-MAIN: problem with data for bias variance - NO OI COMPUTED
c        OI-MAIN: problem with subroutine SSTMAP - NO OI COMPUTED
c        OI-GETFG: NO FIRST-GUESS - NO OI COMPUTED
c
c remarks:  
c           the analysis grid is 1/4-deg 
c  Unless specifically defined in this program and subroutines,
c      Scalars and arrays starting with letters A-H, and O-Z are real*4 
c      Scalars and arrays starting with letters I-N are integer*4 
c
c common block : sclrsqr
c     scx2     -  square of longitude guess scale
c     scy2     -  square of latitude guess scale
c              - scx2,scy2 - the units are km**2
c              - dtk = 111.19 (converts deg to km)
c common block :  comwork -- work array
c     a        - decomposed "a" after  l2cds
c     b        - "b" array
c
c
c     - VARIABLES FOLLOW:
c
c  Input scalar variables from input line 3 (see above)
c     rmax     - maximum x or y distance (km) over which obs are used 
c                for each gridded OI value 
c     nmax     - maximum number of obs are used. Must be <201, 
c     ifil     - number of times 1-2-1 smoothing is done; <1 no smoothing,
c     rej      - REJECT LIMIT for ABSOLUTE super obs increment, 
c                ABSOLUTE SST INCREMENTS > REJ ARE REJECTED
c     nbias    - number of EOT bias estimates to be read, may be 0
c     nsat     - number independent satellite bias data sets 
c                day and night obs from the same satellite are not independent 
c     iwarn    - percentage of satellite data: if lower warning message printed 
c
c Other IMPORTANT scalar variables
c     iyr      - year of analysis (4 digits)
c     imo      - month of analysis (2 digits)
c     ida      - day of analysis (2 digits)
c     ICT      - NUMBER OF OBSERVATIONS IN ARRAYS
c                xtdat and xnsr
c     iday     - Julian day of iyr,imo,ida from iw3jdn 
c     ndays    - number of days between analyses = 1
c     dxy      - grid interval in degrees = 0.25
c     undef    - indicates variable is undefined = -9999.
c     bigsst   - biggest sst allowed = 40.
c     smsst    - smallest SST allowed = -2.
c
c  Important character variables
c     fname*80    - File names with full path, may not be larger than 80
c     title*80    - Analysis summary indicating days used and input data
c     type(30)*8  - Input data label. No more than 30 type of data may be used
c      
c     Important numerical arrays: 
c        imxb and imyb are bias grid, imx and imy are analysis grid
c     varbs(imbx,imby)  -  bias residual variance error - bias grid
c     vsum(imbx,imby)   -  final accumulated bias variance -bias grid
c     xtb(imbx)         -  longitudes - bias grid
c     ytb(imby)         -  latitudes - bias grid
c     bsvar(imx,imy)    -  final accumulated bias variance + residual
c                          if nbias <1, bsvar = residual
c     resid(imx,imy)    -  residual bias ranges from 0.1 to 0.3
c                          0.3 in Arctic and south of 60S with linear tapering
c     epsmin(imx,imy)   -  obs random error (deg C) is eps
c                          for multiple obs in the same box the min eps is used
c     rsvar(imx,imy)    -  The OI random + sampling error variance: OUTPUT
c                          rsvar = e*fgvar*rfac 
c     rfac(imx,imy)     -  correlation correction due to use of first guess 
c                          instead of anomaly. Now set to 1 
c     fgvar(imx,imy)    -  first guess variance - random + sampling before OI is run. 
c                          is run. Reduced by data where present to rsvar
c     lsm(IMX,IMY)      -  modified ls mask. Set to land north of 85N 
c                          OI not processed there, ice SST missinq, 
c                          analysis filled later there later
c     iflag(IMX,IMY)    -  counts how many observations are used in each analysis box
c     sstc(12,IMX,IMY)  -  monthly climatology from weekly OI interpolated to 1/4 deg
c                                base period 1971-2000
c     dclim(imx,imy)    -  climate of analysis day
c     gclim(imx,imy)    -  climate of first guess day, 
c                          later gclim = dclim(t) - gclim(t-1)
c                          climate change: adjusts first guess    
c     corx(imx,imy)     -  zonal decorrelation scales (km)
c     cory(imx,imy)     -  meridional decorrelation scales (km)
c     acor(imx,imy)     -  Guess anomaly is damped toward 0, constant = 0.90
c     ccos(imy)         -  cosine of latitude
c     E(IMX,IMY)        -  Normalized OI random + sampling error variance (range: 0-1)
c     ANAL(IMX,IMY)     -  Analysis increment Will be added to first guess to give total 
c     LS(IMX,IMY)       -  Land/sea mask 0 for land; 1 for ocean, Lakes are treated
c                          as land except for the Great Lakes
c                          OI only computed for ocean
c     ZNS(IMX,IMY)      -  The starting number of obs used in each box for the OI
c     ZNE(IMX,IMY)      -  The final number of obs used in each box for the OI
c                          if the solution is unstable the number of obs is reduced.
c                          Reduction by choosing obs with the largest diagonals
c     alpha(IMX,IMY)    -  sum of the oi weights - if too small may be problem.
c     xtdat(imx,imy)    -  combined SST data - value returned from getdat is 
c                          sum of [sst increment/(noise-to-signal-ratios)**2)]
c                          when all data obtained in main: xtdat is divided by bh
c     xnsr(imx,imy)     -  sum of [1/(noise-to-signal-ratios)**2)] in getdat
c                          in the scaling of xtdat xnsr is set to bh and adjusts xtdat
c                          then xnsr is reused and set to epsmin
c                          originally the equation in Appendix A of Reynolds et al 2007
c                          was used. However, that lowered the diagonal of the OI
c                          matrix which lead to unstable solutions.
c     ansr(imx,imy)     -  Noise to signal standard deviation, differs by input type
c     xt(imx)           -  LONGITUDES ON ANALYSIS GRID
c     yt(imy)           -  LATITUDES ON ANALYSIS GRID
c     gsst(imx,imy)     -  first guess, analysis from previous day 
c                          adjusted by daily climate change
c                          Thus without data anomaly persists not total field
c                          Anomaly is then reduced by acor
c     wk(imx,imy)       -  temporary work array
c
c ATTRIBUTES:
c   LANGUAGE: FORTRAN 77 
c             
c   MACHINE:  Linux: warmcore/rainband
c
c$
c

      PARAMETER (IMX=1440,IMY=720,imbx=182,imby=92) 
      character*20 name
      character*80 fname
      character*80 title
      character*8 type(30)
c      character*9 label(30),blank
c      data blank /'--------'/
      dimension varbs(imbx,imby),vsum(imbx,imby),xtb(imbx),ytb(imby)
      dimension bsvar(imx,imy),epsmin(imx,imy)
      dimension rsvar(imx,imy),rfac(imx,imy),fgvar(imx,imy)
      DIMENSION lsm(IMX,IMY)
      DIMENSION iflag(IMX,IMY)
      dimension sstc(12,IMX,IMY),dclim(imx,imy),gclim(imx,imy)      
      dimension corx(imx,imy),cory(imx,imy),acor(imx,imy)
      DIMENSION ccos(imy),E(IMX,IMY),ANAL(IMX,IMY),LS(IMX,IMY)
      DIMENSION resid(imx,imy)
c      DIMENSION ZNS(IMX,IMY),ZNE(IMX,IMY),anali(IMX,IMY)
      DIMENSION ZNS(IMX,IMY),ZNE(IMX,IMY)
      DIMENSION alpha(IMX,IMY)
      DIMENSION xtdat(imx,imy),xnsr(imx,imy),ansr(imx,imy)
      DIMENSION xt(imx),yt(imy),gsst(imx,imy),wk(imx,imy)
      data bigsst,smsst /40.,-2./
c
      print *,'START: oisst4.f'
      print 921,bigsst,smsst
  921 format ('big-sst =',f8.1,3x,'sm-sst =',f8.1)
      call timer (itime1)
c 
c23456
c
      read (5,70,end=1010) title
      print *,'title for oisst4-col.f'
      print 70,title
c
      read (5,4,end=1010) iyr,imo,ida
    4 format (i4,2i2)
      print 901,iyr,imo,ida 
  901 format ('iyr =',i5,3x,'imo =',i3,3x,'ida =',i3)
      read (5,*,end=1010) rmax,nmax,ifil,rej,nbias,nsat,iwarn
      print 902,rmax,nmax,ifil,rej
  902 format ('rmax =',f10.3,3x,'nmax =',i5,3x,
     1 'ifil =',i3,3x,'rej =',f10.3)
      if (nsat.lt.1) then
	print *,'nsat lt 1: neff is set to nbias'
        neff = nbias
      else
        neff = nbias*nsat
      endif
      print 903,nbias,nsat,neff,iwarn
  903 format ('nbias =',i3,3x,'nsat =',i3,3x,'neff =',i3,3x,
     1 'iwarn (%) =',i5)
      iucor = 1
      read (5,70,end=1010) fname
   70 format (a80)
      print *,'unit ',iucor,' is the oi sst correlation data file'
      print *,'unit ',iucor,' file name: '
      print *,fname
      open (iucor,file=fname,form='unformatted',status='old',
     1 err=1018)
c      
      iufgv = 8
      read (5,70,end=1010) fname
      print *,'unit ',iufgv,' is the oi variance of the increment'
      print *,'unit ',iufgv,' file name: '
      print *,fname
      open (iufgv,file=fname,form='unformatted',status='old',
     1 err=1018)
c      
      iufac = 9
      read (5,70,end=1010) fname
      print *,'unit ',iufac,' is the lag1 correlation factor'
      print *,'unit ',iufac,' file name: '
      print *,fname
      open (iufac,file=fname,form='unformatted',status='old',
     1 err=1018)
c      
      iufg = 2
      read (5,70,end=1010) fname
      print *,'unit ',iufg,' is the oi sst first guess'
      print *,'unit ',iufg,' file name: '
      print *,fname
      open (iufg,file=fname,form='unformatted',status='old',
     1 err=1011)
c      
      iumask = 3
      read (5,70,end=1010) fname
      print *,'unit ',iumask,' is the oi sst land sea mask'
      print *,'unit ',iumask,' file name: '
      print *,fname
      open (iumask,file=fname,form='unformatted',status='old',
     1 err=1016)
c      
      iuclm = 4
      read (5,70,end=1010) fname
      print *,'unit ',iuclm,' is the oi.v2 sst monthly climate'
      print *,'unit ',iuclm,' file name: '
      print *,fname
      open (iuclm,file=fname,form='unformatted',status='old',
     1 err=1015)
c      
      iuout = 91
      read (5,70,end=1010) fname
      print *,'unit ',iuout,' is the oi sst output SST'
      print *,'unit ',iuout,' file name: '
      print *,fname
      open (iuout,file=fname,form='unformatted',status='unknown',
     1 err=900)
c
      iures = 10
      read (5,70,end=1010) fname
      print *,'unit ',iures,' is the residual bias variance'
      print *,'unit ',iures,' file name: '
      print *,fname
      open (iures,file=fname,form='unformatted',status='old',
     1 err=1015)
c
      undef = -9999.
      READ (iumask) wk  
      call maxmin (wk,imx,imy,undef,' 1/4 deg land sea    ')
      call maxmin (wk,imx,imy,1.,   ' 1/4 deg land pts    ')
      call maxmin (wk,imx,imy,0.,   ' 1/4 deg ocean pts   ')
      lsct = 0
      lgrid = imx*imy
      do i=1,imx
        do j=1,imy
          ls(i,j)=nint(wk(i,j))
	  lsm(i,j) = ls(i,j)
	  lsct = lsct + ls(i,j)
        enddo
      enddo
      rland = 100.*float(lsct)/float(lgrid)
      print 118,lsct,rland
  118 format ('ocean points =',i8,3x,'percent ocean =',f8.2)
c
c
      dxy = 0.25
      dxy2 = 0.5*dxy
c      dxy2p = dxy2*1.01
      xt1 = dxy2
      yt1 = -90. + dxy2
c
      CALL GENGRD (IMX,IMY,XT,YT,CCOS,xt1,yt1,dxy)
      call maxmin (xt,imx,1,undef,   '  1/4 deg lon        ')
      call maxmin (yt,imy,1,undef,   '  1/4 deg lat        ')
      call maxmin (ccos,imy,1,undef, '  1/4 deg cos        ')
c
C   FILL IN Rows NORTH OF 85N and as land
c   analysis will be set to filled in there
c   the last row of analysis will be averaged to provide a pole
c   value at 90N. The remaining values will be filled by interpolation 
c   along longitude lines
c
      igrid = 0
      altmax = 85.1
      jjmax = nint(4.*(altmax - yt1)) + 1
      if (jjmax.lt.1) jjmax = 1
      if (jjmax.gt.imy) jjmax = imy
      dlat =  altmax - yt(jjmax)
      if (abs(dlat).gt.dxy2) then
	igrid = igrid + 1
	print *,'YELLOW-WARNING! - OI-MAIN: ',
     1  '  GRID INDEX OUT OF RANGE'
        print 102,jjmax,yt(jjmax),altmax,dlat
      endif
      do i=1,imx
        do j=jjmax,imy
          lsm(i,j) = 0
        enddo
      enddo
      jjmaxm = jjmax - 1
      print *,'ocean values set to land between ',
     1 yt(jjmax),'  and  ', yt(imy)
c
      read (iucor,end=1018) name
      read (iucor,end=1018) corx
      print *,'got   ',name
      call maxmin (corx,imx,imy,undef,'x-correlation        ')
      read (iucor,end=1018) name
      read (iucor,end=1018) cory
      print *,'got   ',name
      call maxmin (cory,imx,imy,undef,'Y-correlation        ')
      read (iucor,end=1018) name
      read (iucor,end=1018) acor
      print *,'got   ',name
      call maxmin (acor,imx,imy,undef,'daily lag-1 autocor  ')
c     
      read (iufgv,end=1018) name
      read (iufgv,end=1018) fgvar
      print *,'got   ',name
      call maxmin (fgvar,imx,imy,undef,'variance increment  ')
c     
      read (iufac,end=1018) name
      read (iufac,end=1018) rfac
      print *,'got   ',name
      call maxmin (rfac,imx,imy,undef,'rfac: lag1 1 autocorr')
c     
      read (iures,end=1018) name
      read (iures,end=1018) resid
      print *,'got   ',name
      call maxmin (resid,imx,imy,undef,'resid bias variance ')
c     
c
c  get monthly climatology 
c
      do kk=1,12
        read (iuclm) kyr,kmo,wk
        if (kmo.eq.1.or.kmo.eq.12) then
	  print *,'base period =',kyr,'   month =',kmo
          call maxmin (wk,imx,imy,undef,'clm-4                ')
	endif
        do j=1,imy
          do i=1,imx
            sstc(kk,i,j) = wk(i,j)
          enddo
        enddo
      enddo
c
c    interpolate climatology to day
c
      kmo = imo
      kda = ida
      print *,'interpolated climatology:   month = ',
     1  kmo,'  day = ',kda
      call idaycl (sstc,kmo,kda,dclim,ierr,imx,imy)
c      do i=1,imx
c        do j=1,imy
c           if (ls(i,j).ne.0.and.dclim(i,j).lt.-50.) then
c	     print *,'dclim',xt(i),yt(j),ls(i,j),dclim(i,j)
c             if (ls(1,1).lt.10) stop 99
c	   endif
c	 enddo
c      enddo
      call maxmin (dclim,imx,imy,undef,'daily climate        ')
      if (ierr.ne.0) go to 1015
c
      iday = iw3jdn (iyr,imo,ida)
      ndays = 1
      CALL GETFG (IUfg,IMX,IMY,IERR,iday,NDAYS,
     1 gsst,acor,dclim,gclim,sstc,undef,ls,bigsst,smsst)
c      do i=1,imx
c        do j=1,imy
c           if (ls(i,j).ne.0.and.gsst(i,j).lt.-50.) then
c	     print *,'gsst',xt(i),yt(j),ls(i,j),gsst(i,j)
c	   endif
c	 enddo
c      enddo
      if (ierr.ne.0) go to 1011      
      IF (IFIL.gt.0) then
        do ijk=1,ifil
	  CALL SMOOTH (GSST,WK,LS,IMX,IMX,IMY)
	enddo
      endif
      call maxmin (gsst,imx,imy,undef,'gsst after smooth     ')
c
c      iis = 1121
c      iie = 1137
c      jjs = 533
c      jje = 538
c      print *,iis,iie,jjs,jje
c
c      print 808,(yt(jj),jj=jjs,jje)
c  808 format (/'first guess  ',8f8.3)
c      do ii=iis,iie
c        print 809,xt(ii),(gsst(ii,jj),jj=jjs,jje)
c  809   format ('lon',f8.3,2x,8f8.3)
c      enddo    
c
      do i=1,imx
        do j=1,imy
          xtdat(i,j) = 0.
          xnsr(i,j) = 0.
          iflag(i,j) = 0
	  epsmin(i,j) = 9999.
        enddo
      enddo
c
c    compute variance of bias (bsvar): nbias files read
c    if nbias = 0 the bias is set to the residual value
c      resid = 0.01
c      print 904,resid
c  904 format ('residual bias =',f12.5)
      iubs = 15
c
      if (nbias.lt.1) then
        do i=1,imx
	  do j=1,imy
	    bsvar(i,j) = resid(1,1)
c           maximum residual value is over land 
c          resid(1,1) is (0.125E, 89.875S)
          enddo
        enddo
        print *,'NO MODES USED: only residual bias computed'
	call maxmin (bsvar,imx,imy,undef,'tot bias: no correct')
      else
        do nloop=1,nbias
          close (iubs)
          read (5,70,end=1020) fname
          print *,'unit ',iubs,' is the noise to signal ratio:',
     1     '  nloop = ',nloop
          print *,'unit ',iubs,' file name: '
          print *,fname
          open (iubs,file=fname,form='unformatted',status='old',
     1    err=1020)
	  call vbias (iubs,varbs,vsum,imbx,imby,bsvar,imx,imy,resid,
     1     iyr,imo,ida,undef,nloop,nbias,neff,xt,yt,xtb,ytb,
     2     ierr)
          if (ierr.ne.0) go to 1020
	enddo
      endif
c23456
c
c      initially xtdat is the combined SST for the grid * xbh
c      wk = the sum of the 1./ansr**2
c      intially xnsr = number of obs

      igeter = 0
      ICT = 0
      ICTS = ICT
c
C - READING IN AND STACKING THE input DATA
c
      ndat = 0
      igtdat = 0
      iunsr = 11
      iudat = 12
 2010 close (iunsr)
      close (iudat)
c      if (ndat.gt.0) stop 1234
      read (5,70,end=2020) fname
      print *,'unit ',iunsr,' is the noise to signal ratio'
      print *,'unit ',iunsr,' file name: '
      print *,fname
      open (iunsr,file=fname,form='unformatted',status='old',
     1 err=1012)
      read (5,70,end=2020) fname
      ndat = ndat + 1
      print *,'unit ',iudat,' is the data file'
      print *,'number of data files =',ndat
      print *,'unit ',iudat,' file name: '
      print *,fname
      open (iudat,file=fname,form='unformatted',status='old',
     1 err=1012)
c
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c     7                                                               72
c     |                                                                |
c     if data dates are off by more than ndays from analysis date,
c       a warning is issued and those data are ignored
c
      call getdat (iunsr,iudat,ict,rej,iday,ndays,dxy,undef,
     1  xtdat,xnsr,ansr,xt,yt,gsst,iflag,imx,imy,type(ndat),ndat,ls,
     2  igeter,igtdat,iyr,imo,ida,epsmin)
c     SUBROUTINE getdat (iunsr,iudat,ict,rej,iday,ndays,dxy,undef,
c    1  xtdat,xnsr,ansr,xt,yt,gsst,iflag,imx,imy,type,ndat,ls,
c    2  igeter,igtdat,iyr,imo,ida,epsmin)
      JCT = ICT - ICTS
      ICTS = ICT
      PRINT 931,JCT
  931 FORMAT (i10,'  obs added')
c      IF (jct.eq.0) GO TO 1012
c
      go to 2010
 2020 print *,'end of data  ict = ',ict      
      IF (ICT.eq.0) GO TO 1012
c
c     now scale obs
c
      do i=1,imx
        do j=1,imy
	  wk(i,j) = iflag(i,j)
c	  if (iflag(i,j).eq.0) then
c	    xtdat(i,j) = undef
c	    xnsr(i,j)  = undef
c	  endif
	enddo
      enddo
      CALL maxmin (wk,imx,imy,0.,'iflag                ')
c      CALL maxmin (xtdat,imx,imy,undef,'increment obs          ')
c      CALL maxmin (xnsr,imx,imy,undef,'signal-noise ratio      ')
      do i=1,imx
        do j=1,imy
	  if (iflag(i,j).eq.0) then
	    xtdat(i,j)  = undef
	    xnsr(i,j)   = undef
	    epsmin(i,j) = undef
	  else
	    bh = xnsr(i,j)
c
c  note is xnsr is reused: here it is "H" or bh
c    next it is the combined noise to signal ratio
c
c    The combined noise to signal ratio was changed from normlized OA
c       to minimum individual noise to signal ratio: May 11, 2007
c    This was done to raise the combined noise to signal ratio
c       to improve the stabliity of the OI solution
c
	    xtdat(i,j) = xtdat(i,j)/bh
c	    xnsr(i,j) =  sqrt(1./bh)
	    xnsr(i,j) = epsmin(i,j)
c	    if (i.eq.720.and.j.eq.220) then
c	      print 109,iflag(i,j),bh,
c     1          xtdat(i,j),xnsr(i,j)
c  109        format (i2,3x,8f8.3)
c	    endif
	  endif
	enddo
      enddo
      CALL maxmin (xtdat,imx,imy,undef,'increment obs           ')
      lct = 0
      iprob = 0
      do i=1,imx
        do j=1,imy
	  if (xtdat(i,j).ne.undef) then
	    lct = lct + 1
	  endif
	enddo
      enddo
      rper = 100.*float(lct)/float(lsct)
      print 211,rper,iwarn
  211 format ('OI input values (%) =',f10.4,2x,'iwarn (%) =',i5)
      if (rper.lt.float(iwarn)) then
	iprob = iprob + 1
        print *,'YELLOW-WARNING! - OI-MAIN: Problem in SSTMAP ',
     1 '  Number of INPUT OI SST DATA too low'
      endif
c      print 807,(yt(jj),jj=jjs,jje)
c  807 format (/'data incr    ',8f8.3)
c      do ii=iis,iie
c        print 809,xt(ii),(xtdat(ii,jj),jj=jjs,jje)
c      enddo    
      CALL maxmin (xnsr,imx,imy,undef,'signal-noise ratio      ')
c
      call timer (itime2)
      tsec = float(itime2-itime1)*.01
      print *,tsec,' seconds to get data and first guess'
c
c      if (tsec.gt.0.) stop 9999
      CALL SSTMAP (xtdat,xnsr,anal,e,imx,imy,lsm,ierr,
     1 xt,yt,ccos,corx,cory,nmax,rmax,dxy,undef,zns,zne,alpha)
      if (ierr.gt.0) go to 1022
c
c     no solution found over land or latitudes north of 85N
c     values north of 85N are later filled by interpolation
c
      call timer (itime4)
c      tsec = float(itime4-itime3)*.01
      tsec = float(itime4-itime2)*.01
      print *,tsec,' seconds to compute analysis'
      nalp0 = 0
      nalp1 = 0
      nalp11 = 0
      nalp12 = 0
c
      DO J = 1,IMY
        DO I = 1,IMX
	  IF (LS(I,J).eq.0) THEN
            bsvar(I,J) = undef
	  endif
	enddo
      enddo
c
      DO J = 1,IMY
        DO I = 1,IMX
	  IF (LSm(I,J).eq.0) THEN
            ANAL(I,J) = undef
            rsvar(I,J) = undef
c            bsvar(I,J) = undef
	    alpha(i,j) = undef
	  else
	    rsvar(i,j) = e(i,j)*fgvar(i,j)*rfac(i,j)
	  endif
c	  anali(i,j) = anal(i,j)
	  if (alpha(i,j).eq.0.) then
            nalp0 = nalp0 + 1
	  endif
	  if (alpha(i,j).gt.1.) then
            nalp1 = nalp1 + 1
	    if (alpha(i,j).gt.1.1) then
              nalp11 = nalp11 + 1
	      if (alpha(i,j).gt.1.2) then
                nalp12 = nalp12 + 1
	        print 209,i,xt(i),j,yt(j),alpha(i,j),zne(i,j)
  209           format ('i =',i5,2x,'lon =',f8.3,2x,
     1          'j =',i4,2x,'lat =',f8.3,2x,'alpha =',f7.3,2x
     2          'n =',f5.0)
              endif
            endif
          endif
	enddo
      enddo
      CALL MAXMIN (rsvar,IMX,IMY,undef,'random+sample error ')
      CALL MAXMIN (bsvar,IMX,IMY,undef,'bias variance error ')
      CALL MAXMIN (Alpha,IMX,IMY,undef,'OI sum of weights   ')
      print *,'number alpha = 0:',nalp0
      print *,'number alpha > 1:',nalp1
      print *,'number alpha > 1.1:',nalp11
      print *,'number alpha > 1.2:',nalp12
      CALL MAXMIN (ANAL,IMX,IMY,undef,'OI analysis incr     ')
      call maxmin (e   ,imx,imy,undef,'error correction     ')
C
C   CONVERT ANALYSIS INCREMENT IN ANAL TO MEAN FIELD BY ADDING GUESS
C   FIELD, GSST.
C
      lct = 0
      do i=1,imx
        do j=1,imy
	  if (zne(i,j).gt.0.5) then
	    lct = lct + 1
	  endif
	enddo
      enddo
      rper = 100.*float(lct)/float(lsct)
      print 229,rper,iwarn
  229 format ('OI gridded values (%) =',f10.4,2x,'iwarn (%) =',i5)
      if (rper.lt.float(iwarn)) then
	iprob = iprob + 1
        print *,'YELLOW-WARNING! - OI-MAIN: Problem in SSTMAP ',
     1 '  Number of OI results too low'
      endif
c
      DO J = 1,IMY
        DO I = 1,IMX
	  IF (LSm(I,J).ne.0) THEN
            ANAL(I,J) = GSST(I,J) + ANAL(I,J)
c check sst not below -1.8: if so set to -1.8
            IF (ANAL(I,J).LT.-1.8) ANAL(I,J) = -1.8
          else
            ANAL(I,J) = undef
	  endif
        enddo
      enddo

c check sst not below 0.0 in Great Lakes 
c
c     between 93W and 75W and 40N to 50N
c
      glnmax = -75. + 360.
      glnmin = -93. + 360
      gltmax = 50.
      gltmin = 40.
      is = nint(4.*(glnmin - xt(1))) + 1
      if (is.lt.1) is = 1
      if (is.gt.imx) is = imx
      dlon =  glnmin - xt(is)
      if (abs(dlon).gt.dxy2) then
	igrid = igrid + 1
	print *,'YELLOW-WARNING! - OI-MAIN: ',
     1 '  GRID INDEX OUT OF RANGE'
        print 101,is,xt(is),glnmin,dlat
 101    format ('lon: ',i5,3f10.3)
      endif
      ie = nint(4.*(glnmax - xt(1))) + 1
      if (ie.lt.1) ie = 1
      if (ie.gt.imx) ie = imx
      dlon =  glnmax - xt(ie)
      if (abs(dlon).gt.dxy2) then
	igrid = igrid + 1
	print *,'YELLOW-WARNING! - OI-MAIN: ',
     1  '  GRID INDEX OUT OF RANGE'
        print 101,ie,xt(ie),glnmin,dlat
      endif
      js = nint(4.*(gltmin - yt(1))) + 1
      if (js.lt.1) js = 1
      if (js.gt.imy) js = imy
      dlat =  gltmin - yt(js)
      if (abs(dlat).gt.dxy2) then
	igrid = igrid + 1
	print *,'YELLOW-WARNING! - OI-MAIN: ',
     1 '  GRID INDEX OUT OF RANGE'
        print 102,js,yt(js),gltmin,dlat
 102    format ('lat: ',i5,3f10.3)
      endif
      je = nint(4.*(gltmax - yt(1))) + 1
      if (je.lt.1) je = 1
      if (je.gt.imy) je = imy
      dlat =  gltmax - yt(je)
      if (abs(dlat).gt.dxy2) then
	igrid = igrid + 1
	print *,'YELLOW-WARNING! - OI-MAIN: ',
     1 '  GRID INDEX OUT OF RANGE'
        print 102,je,yt(je),gltmax,dlat
      endif
      print *
      print 302,yt(js),yt(je),xt(is),xt(ie)
  302 format ('great lake temps values set to a minimum of 0C '/
     1 'between lats ',2f8.3,3x,
     2 'and between lons  ',2f8.3)
      DO J = js,je
        DO I = is,ie
          IF (LS(I,J).ne.0) THEN
            IF (ANAL(I,J).LT.0.) ANAL(I,J) = 0.
          endif
        enddo
      enddo
c
c   FILL IN Ocean SSTS North of 85N 
c
c
c  first step: multiple zonal 1-2-1 smoothing north of 80N with stronger 
c  smoothing further north
c
      altmin = 80.1
      jjmin = nint(4.*(altmin - yt(1))) + 1
      icc = 0
      do jmin=jjmin,jjmaxm
        icc = icc + 1
c	print *,icc,'  zonal smooth between',yt(jmin),yt(jjmaxm)
	CALL zSMOTH (anal ,WK,LSm,IMX,IMX,IMY,jmin,jjmaxm)
	CALL zSMOTH (rsvar,WK,LSm,IMX,IMX,IMY,jmin,jjmaxm)
	CALL zSMOTH (anal ,WK,LSm,IMX,IMX,IMY,jmin,jjmaxm)
	CALL zSMOTH (rsvar,WK,LSm,IMX,IMX,IMY,jmin,jjmaxm)
      enddo
c
c   second step; next compute pole values. These are defined as the zonal average
c    of the last ring of analyses j=jjmaxm (84.875)
c
      nave = 0
      tave = 0.
      eave = 0.
      j = jjmaxm
      do i=1,imx
        if (ls(i,j).eq.1) then
	  tave = tave + anal(i,j)
	  eave = eave + rsvar(i,j)
	  nave = nave + 1
	 endif
      enddo
      if (nave.gt.0) then
        tave = tave/float(nave)
	 eave = eave/float(nave)
	 if (tave.lt.-1.8) tave = -1.8
      else
        tave = -1.8
	eave = 1.
      endif
      print 301,tave,eave,nave,yt(jjmaxm)
  301 format ('pole values (90N): sst ave =',f8.3,3x,'err ave =',
     1 f8.3/'num pts =',i5,3x,'lat =',f8.3)
c
c  Step 3: linear interpolated from 85N north to pole 
c           quad weights strengthen influence of pole value
c           the sum of the weights is always 1
c  e.g., at 87.125 the quad pole weight is 0.685 
c                      while the linear is 0.439
c
       ft = (90. - yt(jjmaxm))**2
       f1 = (90. - yt(jjmaxm))**2/ft
       f2 = 1. - f1
       print 303,f1,f2,yt(jjmaxm)
  303    format ('quad fac*yt(j) =',f8.3,3x,
     1  'quad fac*90N ='f8.3,3x,'lat =',f8.3)
       do j=jjmax,imy
         f1 = (90. - yt(j))**2/ft
	  f2 = 1. - f1
c         print 303,f1,f2,yt(j)
         do i=1,imx
          if (ls(i,j).eq.1) then
	    anal(i,j)  =  anal(i,jjmaxm)*f1 + tave*f2
	    rsvar(i,j) = rsvar(i,jjmaxm)*f1 + eave*f2
 	  endif
        enddo
      enddo
c
      CALL MAXMIN (ANAL,IMX,IMY,undef, 'Total SST Analysis   ')
      CALL MAXMIN (rsvar,IMX,IMY,undef,'final random+sample  ')
c
c - WRITE OUT OI ANALYSIS TO FT03F001
c
c      CALL maxmin (xtdat,imx,imy,undef,'increment obs           ')
c      CALL maxmin (xnsr,imx,imy,undef,'signal-noise ratio      ')
c
c  final check on analysis max and min
c
      do i=1,imx
        do j=1,imy
          if (ls(i,j).eq.1) then
	    if (anal(i,j).lt.smsst.or.anal(i,j).gt.bigsst) then
	      print *,'RED - STOP -  OI-MAIN: ',
     1        'final analysis out of range: anal =',anal(i,j)
	      call exit (100)
	    endif 
	  endif 
        enddo
      enddo
      call putoi (iuout,anal,rsvar,bsvar,imx,imy,iyr,imo,ida,
     1 title)
c
      call timer (itime5)
      tsec = float(itime5-itime1)*.01
      print *,tsec,' seconds for complete oi code'
      if (igeter.gt.0) then
        PRINT *,'YELLOW-WARNING! - OI-MAIN: ',
     1 ' Problem in GETDAT'
        call exit (10)
      endif
      if (igrid.gt.0) then
        PRINT *,'YELLOW-WARNING! - OI-MAIN: ',
     1 '  Grid index out of range'
        call exit (10)
      endif
      if (igtdat.gt.0) then
        PRINT *,'YELLOW-WARNING! - OI-MAIN: ',
     1 '  Grid index out of range in GETDAT'
        call exit (10)
      endif
      if (iprob.gt.0) then
        PRINT *,'YELLOW-WARNING! - OI-MAIN: ',
     1   ' Problem with OI data/analysis'
        call exit (10)
      endif
      print *,'GREEN-SUCCESSFUL! - OI-MAIN: OI code completed'
      call exit (1)
c
  900 PRINT *,'RED-STOP! - OI-MAIN: OPEN ERROR ',
     1 '  OI OUTPUT DATA MISSING - NO OI COMPUTED'
      call exit (100)
 1010 PRINT *,'RED-STOP! - OI-MAIN: Missing INPUT DATA from oiparm ',
     1 '  NO OI COMPUTED'
      call exit (100)
 1011 PRINT *,'RED-STOP! - OI-MAIN: ',
     1 '  NO FIRST GUESS AVAILABLE - NO OI COMPUTED'
      call exit (100)
 1012 PRINT *,'RED-STOP! - OI-MAIN: ',
     1 '  NO DATA AVAILABLE OR NO OBS ADDED - NO OI COMPUTED'
      call exit (100)
 1015 PRINT *,' RED-STOP! - OI-MAIN:',
     1 '  NO CLIMATE SST AVAILABLE: month, day ',kmo,kda,
     2 '  NO OI COMPUTED'
      call exit (100)
 1016 PRINT *,' RED-STOP! - OI-MAIN: ',
     1 '  *** NO LAND SEA MASK - NO OI COMPUTED'
      call exit (100)
 1018 PRINT *,'RED-STOP! - OI-MAIN: ',
     1 '  initial stats files are missing - NO OI COMPUTED'
      call exit (100)
 1020 PRINT *,'RED-STOP! - OI-MAIN: ',
     1 '  problem with data for bias variance -NO OI COMPUTED'
      call exit (100)
 1022 PRINT *,'RED-STOP! - OI-MAIN: ',
     1  '  problem with SSTMAP - NO OI COMPUTED'
      call exit (100)
      END
      SUBROUTINE getdat (iunsr,iudat,ict,rej,iday,ndays,dxy,undef,
     1  xtdat,xnsr,ansr,xt,yt,gsst,iflag,imx,imy,type,ndat,ls,
     2  igeter,igtdat,iyr,imo,ida,epsmin)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C               .      .    .                                       .
C SUBPROGRAM:    GETDAT      GETS SUPER OBS FOR OI ANALYSIS
C   PRGMMR: R. W. REYNOLDS   2009-09-10
C ABSTRACT: READ SUPER OBSERVATIONS IN BLOCKS OF UP TO
C   10000. THE SUPER OBS ARRAYS ARE REAL.   
C   THIS SUBROUTINE IS CALLED FOR EACH DATA TYPE (BUOY, SHIP,
C   DAY SATELLITE, NIGHT SATELLITE, AND ICE DATA).  THE NEW DATA
C   ARE added to the data array
C
C PROGRAM HISTORY LOG:
C   2009-09-10  RICHARD W. REYNOLDS
C
c      PARAMETER (JREC=10000) 
C
C   INPUT ARGUMENT LIST:
c     iudat    - input unit number for data
c     iunsr    - input unit number for stand dev of noise to signal ratio
c     rej      - the reject limit for absolute super obs increment 
c     iday     - Julian day of iyr,imo,ida from iw3jdn 
c     ndays    - number of days between analyses = 1
c     dxy      - grid interval in degrees = 0.25
c     undef    - indicates variable is undefined = -9999.
c     xt       - longitudes on analysis grid
c     yt       - latitudes on analysis grid
c     gsst     - first guess, analysis from previous day 
c     imx      - longitude dimension of analysis (0.125 to 359.875) = 1440
c     imy      - latitude dimension of analysis  (-89.875 to 89.875) = 720
c     ls       - Land/sea mask 0 for land; 1 for ocean
c     iyr      - year of analysis (4 digits)
c     imo      - month of analysis
c     ida      - day of analysis
c
c   Output ARGUMENT LIST:
c     ict      - number of total observations added arrays
c              - this variable is increased with added data
c     xtdat    - combined SST data - value returned from getdat is 
c                sum of [sst increment/(noise-to-signal-ratios)**2)]                              
c     xnsr     - sum of [1/(noise-to-signal-ratios)**2)] in getdat
c     ansr     - Noise to signal standard deviation, differs by input type
c     iflag    - counts number of observations used in each analysis box
c     igeter   - 0 OK, gt 0: PREMATURE END OF INPUT DATA
c     igtdat   - 0 OK, gt 0: GRID INDEX OUT OF RANGE
c     type     - Data label, character*8
c     ndat     - integer for number of calls to getdat; not used
c     epsmin   - for multiple obs in the same box the min error
c
c     OTHER Local VARIABLES
c     jrec     - maximum number of obs in super ob block =10000
c     rSST     - data ARRAY OF SST SUPER OBS 
C     rnso     - data array of number of super obs
C     rLAT     - data ARRAY OF LATITUDE SUPER OBS 
C     rLON     - data ARRAY OF LONGITUDE SUPER OBS 
C   
C   OUTPUT FILES: PRINTOUT
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - MAXMIN
C   Functions CALLED:
c     UNIQUE:  - integer function iw3jdn (iyr,imo,ida) returns julian day
c                FROM YEAR (4 DIGITS), MONTH, AND DAY
C
C REMARKS:
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      PARAMETER (JREC=10000) 
      CHARACTER*20 name
      CHARACTER*8 TYPE
      dimension rsst(jrec),rlat(jrec),rlon(jrec),rnso(jrec)
      dimension xtdat(imx,imy),xnsr(imx,imy),ansr(imx,imy)
      dimension xt(imx),yt(imy),gsst(imx,imy),iflag(imx,imy)
      dimension ls(IMX,IMY),epsmin(imx,imy)
c
C - READING IN THE array of expected stand dev of noise to signal ratios
c
      dxy2 = dxy*2
      READ (iunsr,END=6100) name
      print *,name
      READ (iunsr,END=6100) ansr
      call maxmin (ansr,imx,imy,undef,name)
c      npow = 2**(ndat-1)
C
C - READING IN THE DATA
c
      READ (iudat,END=6000) jyr,jmo,jda,type
c      Print *,jyr,jmo,jda,npow,type
      jday =  iw3jdn (jyr,jmo,jda)
      if (iabs(iday-jday).gt.ndays) then
        PRINT *,' WARNING DATA DATES DIFFER FROM ANALYSIS DATES'
	print *,'   BY MORE THAN NDAYS = ',ndays 
	print *,'  DATA ARE NOT USED'
        print*,' analysis date:',iyr,imo,ida
        print*,'     data date:',jyr,jmo,jda
	go to 6000
      ENDIF
      jct = 0
      iok = 0
      iland = 0
      ibsst = 0
   50 READ (IUdat,END=80) NOBS
c      print *,type,nobs
      IF (NOBS.EQ.0) GO TO 80
      READ (iudat,END=6000) (rNSO(M),M = 1,NOBS)
      READ (iudat,END=6000) (rSST(M),M = 1,NOBS)
      READ (iudat,END=6000) (RLAT(M),M = 1,NOBS)
      READ (iudat,END=6000) (RLON(M),M = 1,NOBS)
c      CALL maxmin (rnso,nobs,1,undef,'data nobs        ')
c      CALL maxmin (rsst,nobs,1,undef,'data sst         ')
c      CALL maxmin (rlat,nobs,1,undef,'data lat         ')
c      CALL maxmin (rlon,nobs,1,undef,'data lon         ')
C
      DO 10 I = 1,NOBS
c
c     compute closest grid point to get data noise to signal
c     no need to linear interpolate because values are smoothed
c      
        jct = jct + 1
	ii = nint(4.*(rlon(i)- xt(1))) + 1
        if (ii.lt.1)ii = 1
        if (ii.gt.imx) ii = imx
        dlon = rlon(i)-xt(ii)
	if (abs(dlon).gt.dxy2) then
	  igtdat = igtdat + 1
	  print *,'YELLOW-WARNING! - OI-GETDAT: ',
     1      '  GRID INDEX OUT OF RANGE'
          print 101,i,ii,xt(ii),rlon(i),dlon
 101      format ('lon: ',2i5,3f10.3)
        endif
        jj = nint(4.*(rlat(i) - yt(1))) + 1
        if (jj.lt.1) jj = 1
        if (jj.gt.imy) jj = imy
    	dlat = rlat(i)-yt(jj)
        if (abs(dlat).gt.dxy2) then
	  igtdat = igtdat + 1
	  print *,'YELLOW-WARNING! - OI-GETDAT: ',
     1 '  GRID INDEX OUT OF RANGE'
          print 102,i,jj,yt(jj),rlat(i),dlat
 102      format ('lat: ',i5,3f10.3)
        endif
	if (ls(ii,jj).eq.1) then
	  dsst = rsst(i) - gsst(ii,jj)
	  adf = abs(dsst)
	  bnsr = ansr(ii,jj)
          if (adf.le.rej) then
c        if (adf.gt.rej) then
c          amult = 1. + adf - rej
c          if (amult.gt.rej) go to 10
c          bnsr = bnsr*amult
c        endif
	    ict = ict + 1
	    iok = iok + 1
	    bnsr2 = bnsr**2
	    r = dsst/bnsr2
	    xtdat(ii,jj) = xtdat(ii,jj) + r
	    xnsr(ii,jj) = xnsr(ii,jj) + 1./bnsr2
	    epsmin(ii,jj) = amin1(epsmin(ii,jj),bnsr)
	    iflag(ii,jj) = iflag(ii,jj) + 1
c	    iflag(ii,jj) = iflag(ii,jj) + npow
c	    if (ii.eq.720.and.jj.eq.220) then
c	      print 109,iflag(ii,jj),dsst,rsst(i),gsst(ii,jj),
c     1          ansr(ii,jj),xtdat(ii,jj),xnsr(ii,jj)
c            endif
	  else
c	    print 108,dsst,gsst(ii,jj),
c     1        rsst(i),rlon(i),xt(ii),rlat(i),yt(jj)
c  108       format ('sst bad  ',5x,8f8.3)
	      ibsst = ibsst + 1
	  endif
	else
c	  print 109,ls(ii,jj),gsst(ii,jj),
c     1      rsst(i),rlon(i),xt(ii),rlat(i),yt(jj)
c  109     format ('land  ',i2,3x,8f8.3)
  	  iland = iland + 1
	endif
   10 continue
      go to 50
C
   80 PRINT 301,jyr,jmo,jda,type,jct,iok,iland,ibsst
  301 FORMAT ('GETDAT: YEAR =',I5,3X,'MONTH =',I3,3X,
     1 'DAY =',I3,2x,a8/
     2 'obs READ =',I8,2x,'obs kept =',i8,2x,'land obs =',i5,2x,
     3 'bad SST =',i5)
      RETURN
 6000 PRINT 7000,iudat
      igeter = igeter + 1
 7000 FORMAT ('YELLOW-WARNING! - OI-GETDAT: ',
     1 ' PREMATURE END OF INPUT DATA: unit =',i3)
      RETURN
 6100 PRINT 7100,iunsr
      igeter = igeter + 1
 7100 FORMAT ('YELLOW-WARNING! - OI-GETDAT: ',
     1 ' no noise to signal ratio: unit =',i3)
      RETURN
      END
      SUBROUTINE GETFG (IUN,IMX,IMY,IERR,iday,NDAYS,
     1 gsst,acor,dclim,gclim,sstc,undef,ls,bigsst,smsst)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    GETFG       GETS FIRST GUESS FIELD ON ANALYSIS GRID
C   PRGMMR: R. W. REYNOLDS   2009-0910
C
C ABSTRACT: READS THE GUESS FIELD FROM UNIT IUN. This is the analysis from
c           the previous day. The code corrects the guess by the 1-day
c           change in climatology which preserves the anomaly and then
c           reduces the anomaly by acor. The output gsst is the total
c           field (anomaly plus climatology)
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C   INPUT ARGUMENT LIST:
C     IUN      - input unit number for first guess
c     imx      - longitude dimension of analysis (0.125 to 359.875) = 1440
c     imy      - latitude dimension of analysis  (-89.875 to 89.875) = 720
c     iday     - Julian day of iyr,imo,ida from iw3jdn 
c     ndays    - number of days between analyses = 1
c     acor     - Lag autocorrelation
c                Guess anomaly is damped toward 0, constant = 0.90
c     sstc     - monthly climatology from weekly OI interpolated to 1/4 deg
c                                base period 1971-2000
c     undef    - indicates variable is undefined = -9999.
c     ls       - Land/sea mask 0 for land; 1 for ocean
c     bigsst   - biggest sst allowed = 40.
c     smsst    - smallest SST allowed = -2.
c     iday     - julian day wrt 4713 BC
c     ndays    - number of analys days (=1)
c     acor     - lag 1 autocorrelation reduction for first guess
c     undef    - value for missing data (=-9999.0)
C
C   OUTPUT ARGUMENT LIST:
C     IERR     - ERROR PARAMETER: 0 OK, 1  NO FIRST GUESS - NO OI COMPUTED 
c     gsst     - first guess, analysis from previous day 
c                adjusted by daily climate change
c                Thus without data anomaly persists not total field
c                Anomaly is then reduced by acor
c     dclim    - climate of analysis day
c     gclim    - climate of first guess day, 
c                later gclim = dclim(t) - gclim(t-1); adjusts gsst
C
C   INPUT FILES:  DETERMINE BY IUN; SEE MAIN PROGRAM
C
C   OUTPUT FILES: PRINTOUT
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - MAXMIN, idaycl
C   Functions CALLED:
c     UNIQUE:  - integer function iw3jdn (iyr,imo,ida) returns julian day
c                FROM YEAR (4 DIGITS), MONTH, AND DAY
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      DIMENSION GSST(IMX,IMY),acor(IMX,IMY),ls(IMX,IMY)
      dimension sstc(12,IMX,IMY),dclim(imx,imy),gclim(imx,imy)      
      IERR = 0
C - READING IN THE FIRST-GUESS
      READ (IUN,END=6000) jyr,jmo,jda,gsst
      PRINT 10,jyr,jmo,jda
   10 FORMAT (' GUESS DATE:  YEAR =',I5,3X,'MONTH =',I3,
     1 3X,'DAY =',I3/'Guess adjusted to day using climatology; '
     2  'then anomaly reduced by acor')
      jday =  iw3jdn (jyr,jmo,jda)
      if (iday-jday.ne.ndays) then
        print *,'FIRST GUESS DATE ERROR: Program ends'
	go to 6000
      endif
c
c     get climatology for guess
c
c
c    interpolate climatology to day
c
      kmo = jmo
      kda = jda
      print *,'interpolated climatology:   month = ',
     1  kmo,'  day = ',kda
      call idaycl (sstc,kmo,kda,gclim,ierr,imx,imy)
c      do i=1,imx
c        do j=1,imy
c           if (ls(i,j).ne.0.and.dclim(i,j).lt.-50.) then
c	     print *,'dclim',xt(i),yt(j),ls(i,j),dclim(i,j)
c             if (ls(1,1).lt.10) stop 99
c	   endif
c	 enddo
c      enddo
      call maxmin (gclim,imx,imy,undef,'first guess: climate ')
      do i=1,imx
        do j=1,imy
          if (ls(i,j).eq.1) then
	    gclim(i,j) = dclim(i,j) - gclim(i,j)
	  else
	    gclim(i,j) = undef   
	    gsst(i,j) = undef   
          endif
        enddo
      enddo
      call maxmin (gclim,imx,imy,undef,'1 day climate change')
      if (ierr.ne.0) go to 6000
      CALL MAXMIN (GSST,IMX,IMY,undef,'first guess: before  ')
      do i=1,imx
        do j=1,imy
          if (ls(i,j).eq.1) then
	    if (gsst(i,j).lt.smsst.or.gsst(i,j).gt.bigsst) then
	      print *,'RED-STOP! - OI-GETFG: ',
     1        'guess out of range: gsst =',gsst(i,j)
	      ierr = 1
              return
	    endif 
	    gsst(i,j) = gsst(i,j) + gclim(i,j)
	    if (gsst(i,j).lt.-1.8) gsst(i,j) = -1.8 
	    gsst(i,j) = acor(i,j)*( gsst(i,j)  - dclim(i,j) ) 
     1                  + dclim(i,j)
          endif
        enddo
      enddo
      CALL MAXMIN (GSST,IMX,IMY,undef,'first guess: after     ')
      RETURN
 6000 PRINT 7000
 7000 FORMAT('RED-STOP! - OI-GETFG: NO FIRST-GUESS')
      IERR = 1
      RETURN
      END
      SUBROUTINE SMOOTH (A,B,LS,IDIM,M,N)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    SMOOTH      PROVIDES 1-2-1 SMOOTHER
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-10
C
C ABSTRACT: SMOOTHS INPUT ARRAY A WITH 1-2-1 SMOOTHER
C   IN THE NORTH/SOUTH AND EAST/WEST DIRECTION AND RETURNS RESULT IN
C   ARRAY A.
C   SMOOTHER ONLY IS DONE USING OCEAN POINTS (LS=1)
C
C   INPUT ARGUMENT LIST:
C     A        - TWO INDEX ARRAY TO BE SMOOTHED
C     LS       - LAND SEA TABLE (OCEAN IS 1, LAND IS 0)
C     IDIM     - ACTUAL FIRST INDEX OF A,B AND LS
C              - AS DIMENSIONED IN MAIN PRGM
C     M        - FIRST INDEX OF A, B AND LS
C     N        - SECOND INDEX OF A, B, AND LS
C
C   OUTPUT ARGUMENT LIST:
C     A        - SMOOTHED RESULT
C     B        - WORK ARRAY
C
C   OUTPUT FILES: PRINTOUT
C
C REMARKS:
C   SMOOTHER IS CALLED TO SMOOTH GUESS FIELD IF IFIL=1 IN MAIN PRGM
C   B IS WORK ARRAY
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      DIMENSION A(IDIM,N),B(IDIM,N),LS(IDIM,N)
      PRINT 5
    5 FORMAT (' CALL SUBROUTINE SMOOTH')
      DO I=1,M
        DO J=1,N
          B(I,J) = A(I,J)
        enddo
      enddo
      NM = N - 1
      MM = M - 1
      DO I=2,MM
        IP = I + 1
        IM = I - 1
        DO J=2,NM
          JP = J + 1
          JM = J - 1
          IF (LS(I,J).ne.0) then
            WT = LS(IP,J) + LS(IM,J) + LS(I,JP) + LS(I,JM) + 4
            CIP = B(IP,J) * FLOAT( LS(IP,J) )
            CIM = B(IM,J) * FLOAT( LS(IM,J) )
            CJP = B(I,JP) * FLOAT( LS(I,JP) )
            CJM = B(I,JM) * FLOAT( LS(I,JM) )
            A(I,J) = (4.*B(I,J) + CIP + CIM + CJP + CJM)/WT
          endif
	enddo
      enddo
      I = 1
      IP = 2
      IM = IDIM 
      DO J=2,NM
        JP = J + 1
        JM = J - 1
        IF (LS(I,J).ne.0) then
          WT = LS(IP,J) + LS(IM,J) + LS(I,JP) + LS(I,JM) + 4
          CIP = B(IP,J) * FLOAT( LS(IP,J) )
          CIM = B(IM,J) * FLOAT( LS(IM,J) )
          CJP = B(I,JP) * FLOAT( LS(I,JP) )
          CJM = B(I,JM) * FLOAT( LS(I,JM) )
          A(I,J) = (4.*B(I,J) + CIP + CIM + CJP + CJM)/WT
        endif
      enddo
      I = IDIM
      IP = 1
      IM = IDIM - 1 
      DO J=2,NM
        JP = J + 1
        JM = J - 1
        IF (LS(I,J).ne.0) then
          WT = LS(IP,J) + LS(IM,J) + LS(I,JP) + LS(I,JM) + 4
          CIP = B(IP,J) * FLOAT( LS(IP,J) )
          CIM = B(IM,J) * FLOAT( LS(IM,J) )
          CJP = B(I,JP) * FLOAT( LS(I,JP) )
          CJM = B(I,JM) * FLOAT( LS(I,JM) )
          A(I,J) = (4.*B(I,J) + CIP + CIM + CJP + CJM)/WT
        endif
      enddo
c      print *,'end subr smooth'
      RETURN
      END
      SUBROUTINE zSMOTH (A,B,LS,IDIM,M,N,jmin,jmax)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    SMOOTH      PROVIDES 1-2-1 SMOOTHER
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-10
C
C ABSTRACT: SMOOTHS INPUT ARRAY A WITH 1-2-1 SMOOTHER
C   IN THE NORTH/SOUTH AND EAST/WEST DIRECTION AND RETURNS RESULT IN
C   ARRAY A.
C   SMOOTHER ONLY IS DONE USING OCEAN POINTS (LS=1) and is limited
c   to a zonal band between jmin and jmax.
C
C   INPUT ARGUMENT LIST:
C     A        - TWO INDEX ARRAY TO BE SMOOTHED
C     LS       - LAND SEA TABLE (OCEAN IS 1, LAND IS 0)
C     IDIM     - ACTUAL FIRST INDEX OF A,B AND LS
C              - AS DIMENSIONED IN MAIN PRGM
C     M        - FIRST INDEX OF A, B AND LS
C     N        - SECOND INDEX OF A, B, AND LS
c     jmin     - lower index of zonal band
c     jmax     - upper index of zonal band
C
C   OUTPUT ARGUMENT LIST:
C     A        - SMOOTHED RESULT
C     B        - WORK ARRAY
C
C   OUTPUT FILES: PRINTOUT
C
C REMARKS:
C   SMOOTHER IS CALLED TO SMOOTH GUESS FIELD IF IFIL=1 IN MAIN PRGM
C   B IS WORK ARRAY
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      DIMENSION A(IDIM,N),B(IDIM,N),LS(IDIM,N)
c      PRINT 5
c    5 FORMAT (' CALL SUBROUTINE SMOOTH')
      DO I=1,M
        DO J=1,N
          B(I,J) = A(I,J)
        enddo
      enddo
      js = jmin
      if (js.lt.2) js = 2
      je = jmax
      if (je.ge.n) je = n - 1
      MM = M - 1
      DO I=2,MM
        IP = I + 1
        IM = I - 1
        DO J=js,je
          JP = J + 1
          JM = J - 1
          IF (LS(I,J).ne.0) then
            WT = LS(IP,J) + LS(IM,J) + LS(I,JP) + LS(I,JM) + 4
            CIP = B(IP,J) * FLOAT( LS(IP,J) )
            CIM = B(IM,J) * FLOAT( LS(IM,J) )
            CJP = B(I,JP) * FLOAT( LS(I,JP) )
            CJM = B(I,JM) * FLOAT( LS(I,JM) )
            A(I,J) = (4.*B(I,J) + CIP + CIM + CJP + CJM)/WT
          endif
	enddo
      enddo
      I = 1
      IP = 2
      IM = IDIM 
        DO J=js,je
        JP = J + 1
        JM = J - 1
        IF (LS(I,J).ne.0) then
            WT = LS(IP,J) + LS(IM,J) + LS(I,JP) + LS(I,JM) + 4
            CIP = B(IP,J) * FLOAT( LS(IP,J) )
            CIM = B(IM,J) * FLOAT( LS(IM,J) )
            CJP = B(I,JP) * FLOAT( LS(I,JP) )
            CJM = B(I,JM) * FLOAT( LS(I,JM) )
            A(I,J) = (4.*B(I,J) + CIP + CIM + CJP + CJM)/WT
        endif
      enddo
      I = IDIM
      IP = 1
      IM = IDIM - 1 
      DO J=js,je
        JP = J + 1
        JM = J - 1
        IF (LS(I,J).ne.0) then
            WT = LS(IP,J) + LS(IM,J) + LS(I,JP) + LS(I,JM) + 4
            CIP = B(IP,J) * FLOAT( LS(IP,J) )
            CIM = B(IM,J) * FLOAT( LS(IM,J) )
            CJP = B(I,JP) * FLOAT( LS(I,JP) )
            CJM = B(I,JM) * FLOAT( LS(I,JM) )
            A(I,J) = (4.*B(I,J) + CIP + CIM + CJP + CJM)/WT
        endif
      enddo
c      print *,'end subr smooth'
      RETURN
      END
      SUBROUTINE GENGRD (IMX,IMY,XT,YT,CCOS,xt1,yt1,dxy)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    GENGRD      GENERATES LAT AND LONG OF OI GRID
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-10
C
C ABSTRACT: GENERATES THE LATITUDE AND LONGITUDES
C   FOR THE GRID CENTER OF THE OI ANALYSIS GRID.  IT ALSO GENERATES
C   THE COSINE OF THE LATITUDE POSITION
c   The cosine absolute magnitude of the latitude is limited to 80 degrees
c   This prevents the longitudinal distances in km from becoming too small
C
C
C   INPUT ARGUMENT LIST:
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
C     XT1      - First x (lon) grid point
C     yT1      - First y (lat) grid point
c     dxy      - x and y grid interval
C
C   OUTPUT ARGUMENT LIST:
C     XT       - LONGITUDES ON ANALYSIS GRID
C     YT       - LATITUDES ON ANALYSIS GRID
C     ccos     - cosine of LATITUDES ON ANALYSIS GRID
C
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      DIMENSION XT(IMX),YT(IMY),ccos(IMY)
C - DO THE LONGITUDES
      rad = 3.14159/180.
      ymax = 80.
      ymin = -ymax
      DO I = 1,IMX
        XT(I) = dxy*FLOAT(I-1) + xt1
      enddo
C - DO THE LATITUDES
      DO J = 1,IMY
        YT(J) = dxy*FLOAT(J-1) + yt1
	 yy = yt(j)
	 if (yy.gt.ymax) yy = ymax
	 if (yy.lt.ymin) yy = ymin
        ccos(J) = cos(rad*yy)
      enddo
      RETURN
      END
      subroutine sstmap (xtdat,xnsr,anal,e,imx,imy,ls,ierr,
     1 xt,yt,ccos,corx,cory,nmax,rmax,dxy,undef,zns,zne,alpha)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    SSTMAP      DETERMINES OI SOLUTION for each box
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-10
C
C ABSTRACT: This subroutine actually computes the OI analysis and error.
c   The solution is found at each 1/4 degree grid point or box which is 
c   labeled ocean (ls=1). It then searches the data arrays for all data within
c   fixed distance (rmax) from each ocean grid box. The maximum number of 
c   data points that can be selected is kmax (4000). However, this must be
c   reduced to imax (200). The solution reduces to solving a linear system of 
c   equations at each grid point. the equation
c   can be written as "A"*"X"="B" where "A" is a two dim.
c   array of coefficients determined by the data and the
c   guess at the locations of the data. "B" is a one
c   dimensional array of coefficients determined by the guess
c   at the locations of the data and the analysis grid.
c   the array "X" is a one dimensional array of the required
c   weights found by the solution of the linear equation.
c   the weights times the data increments yields the OI SSTs.
c   another linear relationship gives the associated errors.
c   The size of the matrices is 'N' where N is the number of data points.
c   The computer time for each solution is the proportional
c   to the square of N. Thus, it is important to keep N small enough to save 
c   time and large enough to give an accurate solution. 
c   For each grid point, the code sorts the obs and puts them into
c   the arrays  s1 ,x1 ,y1 ,d1. Here s1 is the error standard deviation,
c   x1 is the longitude location of the data, y1 is the latitude location of the data
c   d1 is the sst increment. ll is the number of obs in these arrays.
c   Note x1 is determined from the variable FLON which shortest distance,
c   which avoids the artificial jump of 360 degrees when crossing Greenwich.
c   To reduce the number of data points to nmax, only the closest grid points
c   could be used. However, this does not take into account the error the data.
c   A better way is to sort by the magnitude of the diagonal elements in a.
c   This takes into account both distance and error. This diagonals are stored 
c   in the array gwt and sorted into the array twt. The desired number of elements
c   (say 24) allows the smallest value of twt to be selected. All other obs
c   are discarded. s2, x2, y2 and d2 are work arrays of the corresponding variables.
c   mmax is set equal to nmax
C   subroutine decomp forms the array "A" and attempts
c   to decompose it.  estim calculates "B" and finds "X"
c   be using the results from the decomposition.
c   finds all computational boxes for the oi
c   solution.  The decomposing of A is tested. If the returned variable 'COND'
c   is larger than cnmx (100) the decomposition is unstable. The mmax must be reduced.
c   the array nsub has a suggest reduction number. Thus, if mmax = 24,
c   nsub(24) = is 10 and mmax is reduced to 14. A new sort and ranking is repeated
c   and the procedure is continued until there is a solution. This must always 
c   be successful because when N = 1. The arrays become scalars and 
c   X(1) = B(1)/A(1,1). If N > 1 Esdim is called and the analysis and error 
c   is computed. In the case for N = 1, the analysis and error is computed 
c   directly from X(1) = B(1)/A(1,1).
c   The OI solution is returned in ANAL (analysis
c   increment sst) and E (normalized analysis error).
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C      parameter (imax=200,kmax=imax*20) 
c   
C   INPUT ARGUMENT LIST:
c     xtdat    -  combined SST data increment-
c     xnsr     -  set to epsmin; min eps of input data within box
c     IMX      -  LONGITUDE DIMENSION OF analysis (0.125 TO 359.875)
C              -  IMX = 1440                      (0.125E TO 359.875E)
C     IMY      -  LATITUDE DIMENSION OF analysis  (-89.875 TO 89.875)
C                 IMY = 720                          (89.875S TO 89.875N)
C     Xnsr     - ARRAY OF SUPER OBS noise to signal error (std dev)
C     LS       - ARRAY OF LAND SEA TAGS
C              - (LS=1 OVER WATER AND LS=0 OVER LAND)
c              - Note in main this is called with lsm so no computations
c              - are done north of 85N
c     ierr     - 0 OK, 1 if problem in sstmap
C     XT       - ARRAY LONGITUDE LOCATIONS OF ANALYSIS GRID
C     YT       - ARRAY OF LATITUDE LOCATIONS
c     ccos     -  cosine of latitudes
c     corx     -  zonal decorrelation scales (km)
c     cory     -  meridional decorrelation scales (km)
c     rmax     - maximum x or y distance (km) over which obs are used 
c                for each gridded OI value 
c     nmax     - maximum number of obs are used. Must be <201, 
c     dxy      - grid interval in degrees = 0.25
c     undef    - indicates variable is undefined = -9999.
c
C     PARAMETERS:
C              - IMAX = 200, MAXIMUM SIZE OF LINEAR EQUATION
C              - KMAX = 20*IMAX, MAXIMUM NUMBER OF DATA POINTS
C                used in a box. Data points will be reduced to imax or less 
C
C   OUTPUT ARGUMENT LIST:
c     E        -  Normalized OI random + sampling error variance (range: 0-1)
c     ANAL     -  Analysis increment
c     ZNS      -  The starting number of obs used in each box for the OI
c     ZNE      -  The final number of obs used in each box for the OI
c                 if the solution is unstable the number of obs is reduced.
c                 Reduction by choosing obs with the largest diagonals
c     alpha    -  sum of the OI weights - if too small may be problem.
C
c Other IMPORTANT scalar variables
c     mmax     -  local value of nnmx, mmax may be reduced if decomp fails
c     ll       -  number of defined elements in s1,x1,y1,d1
c     lll      -  number of defined elements in s2,x2,y2,d2
c     cnmx     -  minimum condition code for successful solution using decomp/estim
c     ICT      - NUMBER OF OBSERVATIONS IN ARRAYS
c                xtdat and xnsr
c     iday     - Julian day of iyr,imo,ida from iw3jdn 
c     ndays    - number of days between analyses = 1
c     dxy      - grid interval in degrees = 0.25
c     undef    - indicates variable is undefined = -9999.
c     bigsst   - biggest sst allowed = 40.
c     smsst    - smallest SST allowed = -2.
c
c  Important character variables
c     fname*80    - File names with full path, may not be larger than 80
c     title*80    - Analysis summary indicating days used and input data
c     type(30)*8  - Input data label. No more than 30 type of data may be used
c      
c     Important numerical arrays: 
c     s1       -  the error standard deviation
c     x1       -  longitude location of the data
c     y1       -  latitude location of the data
c     d1       -  sst increment of the data
c     s2       -  working error standard deviation
c     x2       -  working longitude location of the data
c     y2       -  working latitude location of the data
c     d2       -  working sst increment of the data
c     nsub     -  array which specifies reduction in mmax by current value of mmax
c     gwt      - diagonal elements of A
c     twt      - sorted diagonal elements of A
c
C   OUTPUT FILES: PRINTOUT
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - DECOMP, ESTIM, MAXMIN, QSORT
C
C REMARKS:
C            NOTE: IF THE SIZE OF ARRAY TO BE DECOMPOSED, IMAX,
C           IS CHANGED, NMX WHICH IS = TO IMAX MUST ALSO BE
C           CHANGED IN DECOMP AND ESTIM.
c     data error is assumed to be uncorrelated                                    .
c
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      COMMON /SCLSQR/ SCX2,SCY2
      parameter (imax=200,kmax=imax*20)
c     compar required by sort program
      external compar
      integer*2 compar
c
c
      DIMENSION alpha(IMX,IMY)
      dimension anal(imx,imy),e(imx,imy),xt(imx),yt(imy),ls(imx,imy)
      dimension s1(kmax) ,x1(kmax) ,y1(kmax) ,d1(kmax)
      dimension s2(kmax) ,x2(kmax) ,y2(kmax) ,d2(kmax)
      dimension xtdat(imx,imy),xnsr(imx,imy),niter(10)
      dimension corx(imx,imy),cory(imx,imy),ccos(imy)
      DIMENSION ZNS(IMX,IMY),ZNE(IMX,IMY)
c     dimension gwt(kmax)
c  added
      dimension gwt(kmax),rcor(kmax)
      dimension twt(kmax),nsub(imax)
      data nsub /1,1,1,1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,178*10/
c
c      data niter /10*0/
c      data numdec /0/
c      data cmin,cmx1,cmx2/9999.,0.,0./,dmin/25./,cnmx/100/
c      
c     gwt is the weight of each oi value if matrix were diagonal
c     gwt will be used to rank observation 
c     larger gwt values will be kept
c
      cnmx = 100.
      nsqn1 = 0
      nsqn0 = 0
c      
      rdmax2 = rmax**2
      dtk = 111.19
      mmax = nmax
      if (mmax.gt.imax) mmax = imax
      do l=1,10
        niter(l) = 0
      enddo
      numdec = 0
c     set anal and e to result if no data
      do jj = 1,imy
        do ii = 1,imx
          anal(ii,jj) = 0.
          e(ii,jj) = 1.
          zns(ii,jj) = 0.
          zne(ii,jj) = 0.
	  alpha(ii,jj) = 0.
        enddo
      enddo
c     
      jadd = nint(rmax/dtk)
      jaddmx = jadd*11
c
c     this keeps (11) boxes square (rmax x rmax) until |lat| > 84.8 degrees
c
c      rglat = dxy*float(jadd)
c      print *,jadd,rglat,dy
c     call maxmin (corx,imx,imx,imy,emin,emax,undef)
c      print *,' min.,max. of corx:',emin,emax
c      call maxmin (cory,imx,imx,imy,emin,emax,undef)
c      print *,' min., max., of cory:',emin,emax
      do 1000 j=1,imy
c      print *,'sstmap  col =',j
      iadd = nint( rmax/(dtk*ccos(j)) )
      if (iadd.gt.jaddmx) iadd = jaddmx
c      rglon = dxy*float(iadd)
c      print *,j,iadd,rglon
      js = j - jadd
      je = j + jadd
      if (js.lt.1) js = 1
      if (je.gt.imy) je = imy
      do 1100 i=1,imx
      if (ls(i,j).eq.0) go to 1100
      is = i - iadd
      ie = i + iadd
      scx2 = corx(i,j)**2
      scy2 = cory(i,j)**2
c      if (j.eq.13) print *,i,j,xt(i),yt(j),scx2,scy2
c
c     begin selecting data needed for each grid point
c
      ll=0
      mmax = nmax
      if (mmax.gt.imax) mmax = imax
      do 71 ii=is,ie
      ir = ii
      if (ir.lt.1) then
        ir = ir + imx
        if (ir.lt.1.or.ir.gt.imx) then
	  print 301,ir,i,iadd
  301     format ('RED-STOP! - OI-SSTMAP: index error - ir =',i5,3x,
     1     'i =',i5,3x,'iadd =',i5)
          ierr = 1
	  return
        endif
      endif
      if (ir.gt.imx) then
        ir = ir - imx
        if (ir.lt.1.or.ir.gt.imx) then
	  print 302,ir,i,iadd
  302     format ('RED-STOP! - OI-SSTMAP: index error - ir =',i5,3x,
     1     'i =',i5,3x,'iadd =',i5)
          ierr = 1
	  return
        endif
      endif
      flon = float(ii-i)*dxy + xt(i)
      do 72 jj=js,je
      if (xtdat(ir,jj).ne.undef) then
        ll=ll+1
        if (ll.gt.kmax) go to 73
c        x1(ll) = xt(ir)
        x1(ll) = flon
        y1(ll) = yt(jj)
        d1(ll) = xtdat(ir,jj)
        s1(ll) = xnsr(ir,jj)
      endif
   72 continue
   71 continue
   73 if (ll.gt.kmax) then
        print 59,kmax,ll,i,j
   59   format (' *** ll will be reduced to kmax:  kmax =',i5,3x,
     1 'll =',i5,3x,'i =',i3,3x,'j =',i3)
        ll = kmax
      endif
c
c      now compute potential weights
c      note combined obs (dz=undef) get negative potential weight
c      also obs outside of rmax radius from grid point also
c         get negative potential weight
c      all obs with negative potential weight will be discarded
c
c      
      lll = 0
      do ijk=1,ll
        if (d1(ijk).ne.undef) then
          xd = (x1(ijk)-xt(i))*ccos(j)*dtk
          yd = (y1(ijk)-yt(j))*dtk
          xd2 = xd**2
          yd2 = yd**2
          dist = xd2 + yd2
          if (dist.le.rdmax2) then
            dg = xd2/scx2 + yd2/scy2
            if (dg.lt.16.) then
              lll = lll + 1
c  added
	      rcor(lll) = exp(-dg)
c  added
	      gwt(lll) = exp(-dg)/(1+s1(ijk)**2)
	      d2(lll) = d1(ijk)
	      s2(lll) = s1(ijk)
	      x2(lll) = x1(ijk)
	      y2(lll) = y1(ijk)
            endif
          endif
         endif
      enddo
      lmax = lll
c      if (lmax.eq.1) print 219,xt(i),yt(j),lmax
c  219   format ('xt =',f6.1,3x,'yt =',f6.1,3x,'lmax =',i5)    
c
c    now first toss out obs with lower diagnonal or negative weights 
c   now reduce obs so that number is not greater than mmax
c   this selection is done using sorted gwt values     
c   note negative gtw values have already been flagged to be be tossed
c      
c   if matrix does not converge, mmax is reduced by 10 each loop
c   this must work because eventually there will be no obs.
c
c    the first step is to sort gwt
      do ll=1,lmax
        twt(ll) = gwt(ll)
      enddo
      call qsort (twt,lmax,4,compar)
      iter = 0
      zns(i,j) = min(mmax,lmax)
 180  ltmp = min(mmax,lmax)
      lcut = lmax - ltmp
      if (lcut.le.0) then
        gcut = 0.
      else
        gcut = twt(lcut)
      endif
c      if (gcut.lt.0.) gcut = 0.
      lll=0
      do ll=1,lmax
        if (gwt(ll).gt.gcut) then
          lll = lll + 1
	  d1(lll) = d2(ll)
	  s1(lll) = s2(ll)
	  x1(lll) = x2(ll)
	  y1(lll) = y2(ll)
c     w1 only needed if n = 1
c	  w1 = gwt(ll)
c  added start
	  if (lll.eq.1) then
c     w1 and r1 only needed if n = 1
	    w1 = gwt(ll)
	    r1 = rcor(ll)
	  endif
c  added end
	endif
      enddo
      n = lll
c      if (n.eq.1) then
      if (n.gt.mmax) then
c      if (iter.gt.0) then
        print 192,kmax,mmax,lmax,lcut,n
 192    format ('kmax =',i5,3x,'mmax =',i5,3x,'lmax =',i5,3x,
     1   'lcut =',i5,3x,'n =',i5)
c        print 119,xt(i),yt(j),lmax,ismall,iter,n
c  119   format ('xt =',f6.1,3x,'yt =',f6.1,3x,'lmax =',i5,
c     1 3x,'ismall =',i5,3x,'iter =',i5,3x,'n =',i5)    
        print 201,gcut
  201   format ('gcut =',f10.3)
  	print 202,(gwt(ll),ll=1,lmax)
  202   format ('gwt =',8f8.3)	
        print 203,(twt(ll),ll=1,lmax)
  203   format ('twt =',8f8.3)	
	print 204,(d1(ll),ll=1,lmax)
  204   format (' d1 =',8f8.3)	
c	print 205,(s1(ll),ll=1,lmax)
c  205   format (' s1 =',8f8.3)	
c	print 206,(x1(ll),ll=1,lmax)
c  206   format (' x1 =',8f8.3)	
c 	print 207,(y1(ll),ll=1,lmax)
c  207   format (' y1 =',8f8.3)
      endif
      if (n.lt.2) go to 750
      call decomp (x1,y1,s1,n,cond,ccos(j))
c      subroutine decomp (x,y,s,n,cond,ccos)
      if (cond.gt.cnmx) then
        mmax = ltmp - nsub(ltmp)
        iter = iter + 1
	 if (mmax.lt.0) then
	   print *,'RED-STOP! - OI-SSTMAP: mmax lt 0'
	   ierr = 1
	   return
	 endif
        go to 180
      endif
      numdec = numdec + 1
      if (iter.gt.0) then
        if (iter.gt.10) iter=10
        niter(iter) = niter(iter) + 1
      endif
c
      xe = xt(i)
      ye = yt(j)
      call estim (x1,y1,d1,n,xe,ye,anal(i,j),e(i,j),ccos(j),alp)
c      subroutine estim (x,y,t,n,xe,ye,te,err,ccos)
c     print *,' anal,err,n from estim:',anal(ii,jj),
c    * e(ii,jj),n
      zne(i,j) = n
      alpha(i,j) = alp
      go to 1100
  750 if (n.eq.1) then
c    solution for 1 ob where w1 is weight
        nsqn1 = nsqn1 + 1
c        e(i,j) = 1. - w1
c  added
        e(i,j) = 1. - w1*r1
	alp = w1
        anal(i,j) = w1*d1(n)
c	print 911,xt(i),yt(j),anal(i,j),e(i,j)
c 911    format ('   n=1:  xt =',f6.1,3x,'yt =',f6.1,3x
c     1 'sst inc =',f8.4,3x,'  error =',f8.4)
      else
        nsqn0 = nsqn0 + 1
	alp = undef
      endif
      zne(i,j) = n
      alpha(i,j) = alp
 1100 continue
 1000 continue
      print 101,numdec,nsqn1,nsqn0
  101 format (' number of successful calls to decomp  =',i8/
     1 7x,'number with only 1 observation  =',i8/
     2 7x,'number with      0 observations =',i8)
      do 103 i=1,10
      if (niter(i).lt.1) go to 103
      print 102,niter(i),i
  102 format (1x,i8,'  regions required',i5,
     1 '  iterative call(s) to decomp for thinning')
  103 continue
c      call maxmin (anal,imx,imy,undef,'analysis sst  field   ')
c      call maxmin (e   ,imx,imy,undef,'error correction field')
      call maxmin (zns,imx,imy,0., 'start n               ')
      call maxmin (zne ,imx,imy,0.,'end n                 ')
      return
      end
      integer*2 function compar(arg1,arg2)
C$   SUBPROGRAM  DOCUMENTATION  BLOCK
C
C SUBPROGRAM
c      integer*2 function
C   PRGMMR: R. W. REYNOLDS   2009-09-15
C ABSTRACT:
c     Orders the variables arg1 and arg2.
c     Called by subroutine qsort (array, len, isize, compar)
c     Compar is the name of a user supplied integer*2 function that will
c     determine the sorting order. This function will be called with 2
c     arguments that will be elements of array. The function must return -
c       negative if arg 1 is considered to precede arg 2
c       zero if arg 1 is equivalent to arg 2
c       positive if arg 1 is considered to follow arg 2
c      Before calling qsort it is necessary to include the following Fortran statements
c      external compar
c      integer*2 compar
c
c  In the sample call to qsort:
c  Array is the one dimensional array containing the elements to be sorted.
c   len is the number of elements in the array.
c   isize is the size of an element,
c  typically -
c       4 for integer and real
c       8 for double precision or complex
c       16 for double complex
c       (length of character object) for character arrays
c  On return from qsort, the elements of array will be sorted.
c
c  PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS     C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      real*4 arg1,arg2
      if(arg1.lt.arg2) then
        compar=-1
      else if(arg1.eq.arg2) then
        compar=0
      else
        compar=1
      endif
      return
      end
      subroutine decomp (x,y,s,n,cond,ccos)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    DECOMP      FORMS ARRAY "A" IN LINEAR EQUATION
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-10
C
C ABSTRACT: THIS ROUTINE AND ESTIM SOLVES THE LINEAR SYSTEM OF
C   EQUATIONS WHICH CAN BE WRITTEN AS "A"*"X"="B" WHERE
C   "A" IS A TWO DIMENSIONAL ARRAY OF COEFFICIENTS.
C   "B" AND "X" ARE ONE DIMENSIONAL ARRAYS OF COEFFICIENTS.
C   DECOMP FORMS "A" IN THE ARRAY A.  DECOMP CALLS THE
C   lapack ROUTINE  spoco TO COMPUTE THE CHOLESKY FACTORIZATION
C   OR DECOMPOSITION OF "A".
C   THIS RESULT IS RETURNED IN "A" OVERWRITING IT.
C   ESTIM COMPUTES "B" AND
C   USES THE DECOMPOSED ARRAY "A" TO COMPUTE THE UNKNOWN "X".
C   SEE SUBROUTINE SSTMAP FOR MORE DETAILS.
C
C PROGRAM HISTORY LOG:
C   2009-09-114  RICHARD W. REYNOLDS
C
C   INPUT        ARGUMENT LIST:
C     X        - ARRAY OF LONGITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE XZ IN SUBROUTINE SSTMAP
C     Y        - ARRAY OF LATITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE YZ IN SUBROUTINE SSTMAP
C     S        - ARRAY OF UNCORR. ERROR NOISE TO SIGNAL RATIO
C     N        - NUMBER OF OBSERVATIONS
C     ccos     - cosine of latitude at grid point to be analyzed
C     PARAMETERS:
C     NMX      - SIZE OF "A" ARRAY; MUST BE THE SAME AS
C              - IMAX IN SUBROUTINE SSTMAX AND NMX IN ESTIM
C
C   OUTPUT       ARGUMENT LIST:
C     COND     - ESTIMATE OF THE CONDITION CODE OF "A"
C
C COMMON BLOCK : SCLRSQR
C     SCX2     -  SQUARE OF LONGITUDE SCALE
C     SCY2     -  SQUARE OF LATITUDE SCALE
C              - SCX2,SCY2 - THE UNITS ARE KM**2
C              - DTK = 111.19 (CONVERTS DEG TO KM)
C COMMON BLOCK : COMWORK -- A WORK ARRAY
C     A        - "A" ARRAY BEFORE CALLING  spoco
C              - DECOMPOSED ARRAY "A" AFTER CALLING  spoco
C     B        - WORK ARRAY
C
C REMARKS:
C           NOTE: COMWORK IS WORK COMMON: ALSO NEEDED FOR ESTIM
C
C     LIBRARY:
C       lapack
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      PARAMETER (NMX=200)
C
C
      COMMON /SCLSQR/ SCX2,SCY2
      DIMENSION X(N),Y(N),S(N)
c      INTEGER   JTYPE(*)
c      real   a(nmx,nmx),dg,expd,expg,rc,b(nmx)
      real   a(nmx,nmx),dg,expg,rc,b(nmx)
      common /comwrk/ a,b
      data dtk/111.19/
c
      if (n.gt.nmx) then
c        print *,' warning, n.gt.nmx, data truncated in decomp'
        n = nmx
      endif
c     print *,' in decomp iskc=',skc
c
      do 10 i = 1,n
        do 20 j = 1,i
          xd = (x(i)-x(j))*ccos*dtk
          yd = (y(i)-y(j))*dtk
          dg = xd*xd/scx2+yd*yd/scy2
          if (dg.gt.16.  ) then
            expg = 0.
          else
            expg =  exp(-dg)
          endif
c
          if (i.eq.j) then
c            a(i,j) = expg + sc(i)*sc(j) + su(i)*su(j)
             a(i,j) = expg +  s(i)*s(j)
          else
c            expd = 0.
c            if (jtype(i).eq.jtype(j)) expd = expg
c            a(i,j) = expg + sc(i)*sc(j)*expd
            a(i,j) = expg 
            a(j,i) = a(i,j)
          endif
c           if (i.eq.j) print *,' diagonal,i,j,a:',i,j,a(j,i)
c           if (i.ne.j) print *,' off diagonal,i,j,a:',i,j,a(j,i)
  20    continue
  10  continue
c     print * ,' n = ',n
c     print * ,' a'
c     print * ,a
c
c     call l2cds (n,a,nmx,a,nmx,rc,b)  old imsl routine
      call spoco (a,nmx,n,rc,b,info)
      if (info.ne.0) rc = 1.0e-10
c
 
      cond = 1./rc
*      print * ,' condition numb. for a = ',cond,'     info =',info
      return
      end
      subroutine estim (x,y,t,n,xe,ye,te,err,ccos,alp)
c$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    ESTIM       FORMS "B" AND FINDS "X" IN LINEAR EQN
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-14
C
C ABSTRACT: THIS ROUTINE AND DECOMP SOLVES THE LINEAR SYSTEM OF
C   EQUATIONS WHICH CAN BE WRITTEN AS "A"*"X"="B" WHERE
C   "A" IS A TWO DIMENSIONAL ARRAY OF COEFFICIENTS.
C   "B" AND "X" ARE ONE DIMENSIONAL ARRAY OF COEFFICIENTS.
C   ESTIM FORMS "B" ARRAY.  ESTIM CALLS THE
C   IMSL ROUTINE  LFSDS TO COMPUTE UNKNOWN "X" FROM "B" AND
C   THE DECOMPOSED "A". ESTIM THEN COMPUTES
C   TE (THE OI SST ANALYSIS INCREMENT) AND ERR (THE OI ERROR)
C   ESTIMATE AT GRID POINT XE AND YE.
C   NOTE: THE ARRAY NAME COEF IS USED FOR THE UNKNOWN "X".
C
C PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS
C
C   INPUT        ARGUMENT LIST:
C     X        - ARRAY OF LONGITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE XZ IN SUBROUTINE SSTMAP
C     Y        - ARRAY OF LATITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE YZ IN SUBROUTINE SSTMAP
C     T        - ARRAY OF DATA INCREMENTS AT LOCATION (X,Y)
C              - VARIABLE DZ IN SUBROUTINE SSTMAP
C     N        - NUMBER OF OBSERVATIONS
C     XE       - LONGITUDE OF GRID POINT ON ANALYSIS GRID FOR
C              - OI SOLUTION (MUST LIE WITHIN BOX DEFINED IN SSTMAP
C              - AND USED IN DECOMP
C     YE       - LATITUDE OF GRID POINT ON ANALYSIS GRID FOR
C              - OI SOLUTION (MUST LIE WITHIN BOX DEFINED IN SSTMAP
C              - AND USED IN DECOMP
C     ccos     - cosine of latitude at grid point to be analyzed
C
C     PARAMETERS:
C     NMX      - SIZE OF "A" ARRAY; MUST BE THE SAME AS
C              - IMAX IN SUBROUTINE SSTMAX AND NMX IN DECOMP
C
C   OUTPUT       ARGUMENT LIST:
C     TE      -  OI SST ANALYSIS INCREMENT (DEG C)
C             -  AT LOCATION (XE,YE)
C     ERR     -  OI SST ANALYSIS ERROR (RANGE 0 TO 1)
C             -  AT LOCATION (XE,YE)
c     alp     -  sum of the weights 
C
C COMMON BLOCK : SCLRSQR
C     SCX2     -  SQUARE OF LONGITUDE GUESS SCALE
C     SCY2     -  SQUARE OF LATITUDE GUESS SCALE
C              - SCX2,SCY2 - THE UNITS ARE KM**2
C              - DTK = 111.19 (CONVERTS DEG TO KM)
C COMMON BLOCK :  COMWORK -- WORK ARRAY
C     A        - DECOMPOSED "A" AFTER  L2CDS
C     B        - "B" ARRAY
C
C REMARKS:
C           NOTE:  COEF IS THE UNKNOWN "X" ARRAY, the OI SST weights
C           RETURNED BY CALLING sposl
c           coef is a local work array to this subroutine only
C           NOTE: COMWORK IS WORK COMMON: ALSO NEEDED FOR DECOMP
C
C     LIBRARY:
C       lapack:   sposl 
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      parameter (nmx=200)
      real   a(nmx,nmx),b(nmx),coef(nmx),d,sm1,cts
      common /comwrk/ a,b
      common /sclsqr/ scx2,scy2
      dimension x(n),y(n),t(n)
      data dtk/111.19/
c
      if (n.gt.nmx) then
        print *,' dimension is incorrectly set, reset parameters'
        n = nmx
      endif
c
c     print *,' n =',n
c  form right hand side vector of the linear system.
      do 30 i=1,n
        xd=(xe-x(i))*ccos*dtk
        yd=(ye-y(i))*dtk
        d = (xd*xd/scx2+yd*yd/scy2)
        if (d.gt.16.  ) then
          b(i) = 0.
          coef(i) = b(i)
          go to 30
        endif
        b(i) =  exp(-d)
        coef(i) = b(i)
 30   continue
c     print *,' xe ',xe
c     print *,' ye ',ye
c     print *,' x(i) '
c     write (6,601) (x(i),i=1,n)
c     print *,' y(i) '
c     write (6,601) (y(i),i=1,n)
c     print *,' b(i)'
c     write (6,601) (b(i),i=1,n)
c 601 format(9(1pe8.1))
c
c  solve for linear systems with pivoting.
c  sm1 = tr(r.h.s.)*inv(cov.matrix)*(r.h.s) = tr(r.h.s)*coeff.
c
c     print *,' coef(i) '
c     write (6,601) (coef(i),i=1,n)
c
c     call  lfsds (n,a,nmx,b,coef)  original imsl routine
      call sposl (a,nmx,n,coef)
c      call maxmin (coef,n,1,-9999.,'wts      ')
      sm1=0.
      alp=0.
      do 40 i=1,n
        sm1=sm1+coef(i)*b(i)
        alp = alp + coef(i)
 40   continue
c
c  compute analyzed corrections and  estimate analysis error:
c     err = 1.- tr(r.h.s)*inv(cov.matrix)*(r.h.s)
c
c     print *,' sm1 =',sm1
c     print *,' anomaly data:'
c     write (6,601) (t(i),i=1,n)
      cts = 0.
      do 60 i=1,n
        d = t(i)
        cts = cts + coef(i)*d
 60   continue
      err = 1.   - sm1
      te  = cts
      return
      end
      subroutine putoi (iun,anal,rsvar,bsvar,imx,imy,iyr,imo,ida,
     1 title)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    PUTOI       WRITES OUT DATES, OI ANALYSIS AND ERROR
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-14
C
C ABSTRACT: WRITES OUT THE DATES, OI ANALYSIS AND OI ERROR (both bias and
c        random + sampling) and title 
C
C PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS
C
C
C   INPUT ARGUMENT LIST:
C     IUN      - FORTRAN OUTPUT UNIT NUMBER
C     ANAL     - ARRAY OF SST OI ANALYSIS MEAN FIELD (DEG C)
C     E        - ARRAY OF OI ERROR VALUES (RANGE 0 TO 1)
C     IMX      - FIRST DIMENSION OF ANAL AND E ARRAYS
C     IMY      - SECOND DIMENSION OF ANAL AND E ARRAYS
C     IYR      - YEAR (4 DIGITS) OF ANALYSIS
C     IMo      - MONTH OF ANALYSIS
C     IDa      - DAY OF ANALYSIS
c     rsvar    - The OI random + sampling error variance
c                rsvar = e*fgvar*rfac 
c     bsvar   -  final accumulated bias variance + residual
c                computed by satellite bias correction routines
c                eotbias-wt4-col.f    & eotbias-cor4-col.f       
c     title   - a 80 character title to identify the OI analysis
c e.g., title='Daily-OI-V2, Interim, Data (Ship, Buoy, AVHRR: NOAA17&18, NCEP-ice)' 
c
C
C   OUTPUT FILE:  DETERMINE BY IUN; SEE MAIN PROGRAM
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      character*80 title
      dimension anal(imx,imy)
      
c      dimension anal(imx,imy),e(imx,imy)
c      dimension anali(imx,imy)
c      DIMENSION ZNS(IMX,IMY),ZNE(IMX,IMY)
      dimension rsvar(imx,imy),bsvar(imx,imy)
c      DIMENSION xtdat(imx,imy),xnsr(imx,imy)
      print 7,iyr,imo,ida
    7 format (' iyr = ',i5,3x,'imo = ',i3,3x,'ida = ',i3,
     2 3x,'oi dates output')
      write (iun) iyr,imo,ida,anal
      write (iun) iyr,imo,ida,rsvar
      write (iun) iyr,imo,ida,bsvar
      write (iun) title
c      
c      write (iun) iyr,imo,ida,e
c      write (iun) iyr,imo,ida,zns
c      write (iun) iyr,imo,ida,zne
c      write (iun) iyr,imo,ida,xtdat
c      write (iun) iyr,imo,ida,xnsr
c      write (iun) iyr,imo,ida,anali
      return
      end
      subroutine maxmin (fld,im,jm,undef,name)
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    maxmin       WRITES OUT statistics for the array fld
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-14
C
C ABSTRACT: WRITES OUT THE average, minimum, maximum and number of
c           values of array fld. Array values equal to undef are ignored
C
C PROGRAM HISTORY LOG:
C   2009-09-14  RICHARD W. REYNOLDS
C
C
C   INPUT ARGUMENT LIST:
C     fld      - ARRAY to be analyzed
C     IM       - FIRST DIMENSION OF fld array
C     jm       - SECOND DIMENSION OF fld array
c     undef    - indicates variable is undefined = -9999.
c     name     - a 20 character label of the fld array
C
C   OUTPUT FILES: PRINTOUT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C REMARKS:
C   The number of values in fld is printed out as 'inum ='
C$
c
c   compute maximum, minium and average of fld and print
c
      dimension fld(im,jm)
      real*8 ave
      character*20 name
      fldmin = 1.e30
      fldmax = -fldmin
      inum = 0
      ave = 0.
      do i=1,im
        do j=1,jm
          if (fld(i,j).ne.undef) then
            inum = inum + 1
            ave = ave + fld(i,j)
            if (fldmin.gt.fld(i,j))  fldmin = fld(i,j)
            if (fldmax.lt.fld(i,j))  fldmax = fld(i,j) 
          endif
        enddo
      enddo
      if (inum.gt.0) then
        r = 1./float(inum)
        ave = ave*r
      else
        fldmin = 0.
        fldmax = 0.
      endif
      print 88,ave,fldmin,fldmax,inum,name
   88 format ('ave =',f9.3,1x,'min =',f9.3,1x,
     1 'max =',f9.3,1x,'inum =',i8,1x,a20)
      return 
      end
      subroutine idaycl (sstc,imo,ida,dclim,ierr,imx,imy)
c$    subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    dayclm1d      finds climatology of given day and month   
c   prgmmr: d. c. stokes     date: 1996-8-9             
c                                                                       
c abstract: temporally interpolates the climatology field of the day    
c           (defined by input month and day of the month) from the      
c           12 monthly climatology fields.  all fields are defined on   
c           the 1/4-degree OI analysis grid                            
c                                                                       
c program history log:                                                  
c   1996-08-09  diane c. stokes
c   2009-10-14  R. W. Reynolds
c                                                                       
c   input argument list:                                                
c    sstc      - field of the 12 months of climatology                  
c              - dimension (12,1440,720)                                  
c              - the first index is the month,                          
c              - from january (= 1) to december (=12)                   
c              - the second index is longitude
c              - the third index is latitude          
c     imo      - month of desired interpolated climatology              
c     ida      - day of desired interpolated climatology                
c     IMX      - LONGITUDE DIMENSION OF analysis (0.125 TO 359.875)
C                IMX = 1440                      (0.125E TO 359.875E)
C     IMY      - LATITUDE DIMENSION OF analysis  (-89.875 TO 89.875)
C                IMY = 720                       (89.875S TO 89.875N)
c                                                                       
c   output argument list:                                               
c     dclim    - field of the temporally interpolated                   
c              - climatology for imo and ida                            
c              - dimension (1440,720)                                     
c     ierr     -  equal to 0 if interpolated field found                
c              -  equal to 1 if no interpolated field because of        
c              -  bad imo or ida                
c                                                                       
c   output files:                                                       
c     unit 6 - printout                                               
c                                                                       
c remarks:  Note if the month is 2 and the day is 29, the day is 
c           changed to 28. This will change the day in the main program.
c           Thus before calling the subroutine, dummy month and
c           day files are created:
c           kmo = imo;  kda = ida; call idaycl (sstc,kmo,kda,...
c                                                                    
c attributes:                                                           
c   language: fortran 77                                                
c   machine:  Linux: warmcore/rainband
c                                                                       
c$                                                                    
      dimension ndays(12)
      dimension sstc(12,imx,imy)
      dimension dclim(imx,imy)
c
      data ndays/31,28,31,30,31,30,31,31,30,31,30,31/
c  change february 29 to february 28: effect of leap year ignored       
      ierr = 0
c
      if (imo.eq.2.and.ida.eq.29) ida = 28
c                                                                       
c  check for bad date
      if (imo.lt.1.or.imo.gt.12.or.ida.lt.1.or.ida.gt.ndays(imo)) then
      print 10, imo,ida
   10 format('interpolated climate for month = ',i3,3x,'day =',i3)
      print 222
  222 format ('RED-STOP! - OI-DCLIM: interpolation error in dclim')
      ierr = 1
      return
      endif
      halfm1 = real(ndays(imo))/2.
      if (ida.le.int(halfm1)) then
        if (imo.eq.1) then
          jmo = 12
        else
          jmo = imo-1
        endif
        d = halfm1 - real(ida) + .5
      else
        if (imo.eq.12) then
          jmo = 1
        else
          jmo = imo+1
        endif
        d = real(ida) - halfm1 - .5
      endif
      halfm2 = real(ndays(jmo))/2.
      span = halfm1+halfm2
      do j=1,imy
        do i=1,imx
          t1 = sstc(imo,i,j)
          t2 = sstc(jmo,i,j)
          dclim(i,j) = ((t2-t1)/span)*d + t1
        enddo
      enddo
      return
      end
      subroutine intrp (cn,imc,ic,jc,fn,imf,if,jf,
     1 cx,cy,fx,fy,ierr)          
c$    subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    intrp    interpolates between grids   
c   prgmmr: ron pacanowski: gfdl/noaa  date: 1987 ??        
c                                                                       
c abstract: bilinear interpolation from grid cn to grid fn where the         
c     gridspace cn completely overlies the gridspace fn.                        
c     The distributions cx and fx (cy and fy) are arbitrary except that 
c     they must be monotonic increasing and consistent with each other. 
c                                                                       
c program history log:                                                  
c   1987-????  ron pacanowski
c   2009-10-14  R. W. Reynolds
c                                                                       
c   input argument list:                                                
c     cn -   input field 
c     imc -  1st dimension of cn in calling routine                       
c     ic  -  ending i index on cn                                          
c     jc  -  ending j index on cn                                          
c     imf -  1st dimension of fn in calling routine                       
c     if  -  ending i index on fn                                          
c     jf  -  ending j index on fn                                          
c     cx  -  distribution of grid points in i direction on cn              
c     cy  - distribution of grid points in j direction on cn              
c     fx  - distribution of grid points in i direction on fn              
c     fy  - distribution of grid points in j direction on fn              
c                                                                         
c   output argument list:                                               
c     fn       - output field interpolated from cn                  
c     ierr     -  equal to 0 if interpolated field found                
c              -  equal to 1 if no interpolated field because
c                 cx(1) NOT less than or equal to fx(1)             
c                 cy(1) NOT less than or equal to fy(1)             
c                 cx(ic) NOT greater than or equal to fx(ic)        
c                 cy(ic) NOT greater than or equal to fy(ic)        
c                 Distributions cx and fx (cy and fy) NOT monotonic increasing 
c                                                                       
c   output files:                                                       
c     unit 6 - printout                                               
c                                                                       
c remarks:  cx, cy, fx and fy arrays printed if ierr = 1
c                                                                    
c attributes:                                                           
c   language: fortran 77                                                
c   machine:  Linux: warmcore/rainband                                 
c                                                                       
c$                                                                    
      dimension cn(imc,jc),cx(ic),cy(jc),fn(imf,jf),fx(if),fy(jf)       
c
      ierr = 0
      if(cx(1).le.fx(1).and.cx(ic).ge.fx(if).and.cy(1).le.fy(1).and.    
     1 cy(jc).ge.fy(jf)) go to 10                                       
      write(6,9999)                                                     
      go to 20                                                          
   10 do 12 i=2,ic                                                      
      if(cx(i).ge.cx(i-1)) go to 12                                     
      write (6,9998)                                                    
      go to 20                                                          
   12 continue                                                          
      do 14 j=2,jc                                                      
      if(cy(j).ge.cy(j-1)) go to 14                                     
      write (6,9998)                                                    
      go to 20                                                          
   14 continue                                                          
      do 16 i=2,if                                                      
      if(fx(i).ge.fx(i-1)) go to 16                                     
      write (6,9998)                                                    
      go to 20                                                          
   16 continue                                                          
      do 18 j=2,jf                                                      
      if(fy(j).ge.fy(j-1)) go to 18                                     
      write(6,9998)                                                     
      go to 20                                                          
   18 continue                                                          
      go to 30                                                          
   20 write (6,9994)                                                    
      write (6,9993) (cx(i),i=1,ic)                                     
      write (6,9993) (cy(j),j=1,jc)                                     
      write (6,9993) (fx(i),i=1,if)                                     
      write (6,9993) (fy(j),j=1,jf)                                     
      ierr = 1
      return                                                              
   30 continue                                                          
      do 100 j=1,jf                                                     
      do 110 l=1,jc                                                     
      if(cy(l).lt.fy(j)) go to 110                                      
      jn=l-1                                                            
      js=l                                                              
      if(jn.eq.0) js=2                                                  
      if(jn.eq.0) jn=1                                                  
      go to 111                                                         
  110 continue                                                          
  111 continue                                                          
      do 100 i=1,if                                                     
      do 120 l=1,ic                                                     
      if(cx(l).lt.fx(i)) go to 120                                      
      ie=l                                                              
      iw=l-1                                                            
      if(iw.eq.0) ie=2                                                  
      if(iw.eq.0) iw=1                                                  
      go to 121                                                         
  120 continue                                                          
  121 continue                                                          
      a=fx(i)-cx(iw)                                                    
      b=cx(ie)-fx(i)                                                    
      tx=cx(ie)-cx(iw)                                                  
      c=fy(j)-cy(jn)                                                    
      d=cy(js)-fy(j)                                                    
      ty=cy(js)-cy(jn)                                                  
  100 fn(i,j)=(c*(a*cn(ie,js)+b*cn(iw,js))+d*(a*cn(ie,jn)+b*cn(iw,jn))) 
     1 /(tx*ty)                                                         
      return                                                            
 9999 format(1x,'grid interpolated to exceeds grid int from')           
 9998 format(1x,'cx,cy,fx, or fy not monotonic')                        
 9994 format(1x,'cx,cy,fx,fy follow')                                   
 9993 format(1x,10e13.6)                                                
      end                                                               
      subroutine vbias (iub,varbs,vsum,imbx,imby,bsvar,imx,imy,resid,
     1 iyr,imo,ida,undef,nloop,nbias,neff,xt,yt,xtb,ytb,ierr)
c
C$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    vbias     Computes Bias Variance
C   PRGMMR: R. W. REYNOLDS   DATE: 2009-09-10
C
C ABSTRACT: Bias errors are computed in the bias correction codes: 
c   eotbias-wt4-col.f   and eotbias-cor4-col.f. This subroutine reads the 
c   bias variances from nbias fields, sums them up, and normalizes the final bias
c   variance. It then interpolates the final bias from the 2-degree bias
c   correction grid to the 1/4 OI grid, adds a residual error and computes
c   the final bias estimate. The code is called in a loop from 1 to nbias.
c   The final bias estimate is computed in the last loop.
C
C   INPUT ARGUMENT LIST:
C     iub      - FORTRAN input unit number
c     varbs   -  bias residual variance error - bias grid
c     IMbX    -  LONGITUDE DIMENSION OF bias fields   (-1 to 361 )
C             -  IMbX = 182                           (1W TO 361E or 1E)
C     IMbY    -  LATITUDE DIMENSION OF bias fields    (-91 to 91)
C             -  IMbY = 92                            (91S TO 91N)
c     IMX     -  LONGITUDE DIMENSION OF analysis (0.125 TO 359.875)
C             -  IMX = 1440                      (0.125E TO 359.875E)
C     IMY     -  LATITUDE DIMENSION OF analysis  (-89.875 TO 89.875)
C                IMY = 720                       (89.875S TO 89.875N)
c     resid   -  residual bias ranges from 0.1 to 0.3 - oi grid
c                0.3 in Arctic and south of 60S with linear tapering
c     iyr      - year of analysis (4 digits)
c     imo      - month of analysis (2 digits)
c     ida      - day of analysis (2 digits)
c     undef    - indicates variable is undefined = -9999.
c     nloop    - number of times subroutine is called
c     nbias    - number of EOT bias estimates to be processed
c                when nloop = nbias final output is generated - oi grid
c     neff     - number of effective bias fields: vsum/neff
c                if (nsat.lt.1) neff = nsat
c                otherwise nbias*nsat where nsat is 
c                the number independent satellite bias data sets 
c     xt       - LONGITUDES ON ANALYSIS GRID
c     yt       - LATITUDES ON ANALYSIS GRID
c
C   OUTPUT ARGUMENT LIST:
c     xtb      - longitudes - bias grid
c                only computed when nloop=nbias
c     ytb      - latitudes - bias grid
c                only computed when nloop=nbias
c     vsum    -  initially the sum of the individual bias variances
c                when nloop=nbias, vsum = vsum/neff
c                and vsum is the final accumulated bias variance - bias grid
c     bsvar   -  final accumulated bias variance + residual - bias grid
c                only computed when nloop=nbias
C     IERR     - ERROR PARAMETER: 0 OK, 1 Error
c                ierr=1 if bias dates do not match OI dates, if the
c                bias fields end prematurely or there is an interpolation error
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - MAXMIN, intrp
C
C REMARKS:
C   If nbias < 1, bsvar is set equal to the residual in main and this
c   subroutine is not called
c   the bias grid is extended by 1 value in all 4 directions for the 
c   intrp program. 
c   The extra longitude values are repeated since the positions
c   at 1W values = 359E values and 1E values = 361E values. 
c   The latitude values are simply repeated by the nearest real value:
c   91N values set = 89N values; 91S values set = 89S values 
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  Linux: warmcore/rainband
C
C$
      dimension varbs(imbx,imby),vsum(imbx,imby),xtb(imbx),ytb(imby)
      dimension resid(imx,imy),bsvar(imx,imy),xt(imx),yt(imy)
       ix = imbx - 1
       iy = imby - 1
c      write (iub) iyr,imo,ida,bias
c      write (iub) iyr,imo,ida,situ
c      write (iub) iyr,imo,ida,xsat
c      write (iub) iyr,imo,ida,varbs
c
      ierr = 0
c      print *,nloop,nbias,neff,ierr
c      print *,imbx,imby,ix,iy
      read  (iub,end=100) 
      read  (iub,end=100) 
      read  (iub,end=100) 
      read  (iub,end=100) jyr,jmo,jda,((varbs(i,j),i=2,ix),j=2,iy)
      print 7,iyr,imo,ida
    7 format ('bias dates: ',i5,2i3)
      do j=2,iy
        varbs(1,j) = varbs(ix,j)
	 varbs(imbx,j) = varbs(2,j)
      enddo
      do i=1,imbx
        varbs(i,1) = varbs(i,2)
	 varbs(i,imby) = varbs(i,iy)
      enddo
      call maxmin (varbs,imbx,imby,undef,'variance of bias    ')
c
      isum = iyr*10000 + imo*100 + ida
      jsum = jyr*10000 + jmo*100 + jda
c      print *,isum,jsum
      if (jsum.ne.isum) go to 150
      if (nloop.eq.1) then
        do i=1,imbx
	   do j=1,imby
	     vsum(i,j) = varbs(i,j)
          enddo
        enddo
      else
        do i=1,imbx
	   do j=1,imby
	     vsum(i,j) = vsum(i,j) + varbs(i,j)
          enddo
        enddo
      endif
c
c      print *,nloop,nbias,neff,ierr
      if (nloop.ne.nbias) return
c
      rr = float(neff)
      do i=1,imbx
	 do j=1,imby
	   vsum(i,j) = vsum(i,j)/rr 
        enddo
      enddo
      call maxmin (vsum,imbx,imby,undef,'total variance bias ')
      xtb1 = -1.
      do i=1,imbx
        xtb(i) = 2.*float(i-1) + xtb1
      enddo
      ytb1 = -91.
      do j=1,imby
        ytb(j) = 2.*float(j-1) + ytb1
      enddo
      call maxmin (xtb,imbx,1,undef,'bias xtb grid       ')
      call maxmin (ytb,imby,1,undef,'bias ytb grid       ')
      call intrp (vsum,imbx,imbx,imby,bsvar,imx,imx,imy,
     1 xtb,ytb,xt,yt,ierr)
      if (ierr.ne.0) go to 200          
      do i=1,imx
	 do j=1,imy
	   bsvar(i,j) = bsvar(i,j) + resid(i,j)
        enddo
      enddo
      call maxmin (bsvar,imx,imy,undef,'tot interp var bias ')
      return
  100 ierr = 1
      print *,'RED-STOP! - OI-VBIAS: ',
     1 ' premature end of bias data in vbias'
      return
  150 ierr = 1
      print *,'RED - STOP - OI-VBIAS: ',
     1 'bias dates do not match in vbias',
     2 iyr,imo,ida,jyr,jmo,jda
      return
  200 ierr = 1
      print *,'RED-STOP! - OI-INTERP: - error in intrp'
      return
      end
