c     program shipbias4-doc.f
c
c$  MAIN PROGRAM DOCUMENTATION BLOCK
c                .      .    .                                       .
c MAIN PROGRAM: shipbias4-doc.f  Corrects ship data
c   PRGMMR: R.W. REYNOLDS   DATE: 2009-10-20
c
c ABSTRACT: Comaprisons of global average ship to buoy data for 1989-2006 
c           shows that ship data are warmenr than buoy data by 0.14C. 
c           This program reads in the ship data and subtracts 0.14C from
c           the data and then writes the ship data back out.
C
C PROGRAM HISTORY LOG:
c   2009-10-20  Richard W. Reynolds
c
c   Input Data on unit 5 needed by program (input from script)
c
c   The 1st shipbias4 input record has 1 parameters: globc
c globc is the average ship bias with respect to buoy: e.g.,  globc=0.14	        
c
c  The remaining records are input and output file names which must be in order
c  The first file name is the input uncorrected ship data
c  e.g., SHIP=/raid2/sstday/$YEAR/sobs/coad/ship/ship.$TM
c  The second and last file name is the output bias corrected ship data
c  e.g., SHIPC=/raid2/sstday/$YEAR/cobs/coad/ship/ship.$TM
c
c  Fortran files: only units 6 and 52 are output files
c   Files with TM ($TM) in name vary with time, the others are static
c           unit    name
c              5       input data parameters and file names
c              6       print out
c
c             12       uncorrected ship data, SHIP
c             52       bias corrected satellite data, SHIPc - output
c
C     PARAMETERS in Main: None
C   SUBPROGRAMS CALLED: None
c
C   EXIT STATES: There are 3 main categories
c   call exit (1) : Normal End, code color 'green'
c
c   call exit (10): Normal End with WARNING, code color 'yellow'
c                   The problem should be corrected and the code rerun.
c   The following warning 'yellow' errors may occur 
c       shipbias4-main: NO ship data to correct
c
c   call exit (100): Fatal error, Premature STOP, code color 'red'
c                    The problem MUST be corrected and the code rerun.
c   The following fatal 'red' errors may occur 
c       shipbias4-main: premature end of ship input data
c       shipbias4-main: Could not open existing ship input file
c       shipbias4-main: error in ship input file
c       shipbias4-main: error in ship OUTPUT file
c     
c remarks:  
c
c common block : jrec in main: variables are stored obs
c                data has been written in blocks of up to 10000
c     rsum(10000) - number of obs
c     rsst(10000) - sst 
c     rlon(10000) - longitude 
c     rlat(10000) - latitude            
c                                                                   
c     - VARIABLES FOLLOW:
c
c  Input scalar variables from input lines (see above)
c
c  globc  - average ship bias with respect to buoy: e.g.,  globc=0.14
c
c Other IMPORTANT scalar variables
c     iyr      - year of analysis (4 digits)
c     imo      - month of analysis (2 digits)
c     ida      - day of analysis (2 digits)
c     ndays    - number of days used in computing anomalies
c
c  Important character variables
c     fdate*8     - date computed by script 8 digits
c     fship*80    - file name for uncorrected ship data
c     fshipc*80   - file name for bias corrected ship data -output
c                 - File names with full path, may not be larger than 80
c
c     Important numerical arrays: None
c
c ATTRIBUTES:
c   LANGUAGE: FORTRAN 77 
c             
c   MACHINE:  Linux: warmcore/rainband
c
c$
c
      common /jrec/ rsum(10000),rsst(10000),rlon(10000),rlat(10000)             
      character*8 type
c
      character*80 fship,fshipc
c
c      unit 12 is ship data
c      unit 52 is corrected ship data
c
      logical lexist
c
c     input data: order is critical
c
c23456
c
      read *,globc 
      print 118,globc
  118 format ('Subtract ',f8.3,3x,'from all ship data')
c
      read 1,fship
    1 format (a80)
      print *,'unit 12  '
      print *, 'open  ',fship
      inquire(file=fship,exist=lexist)
c       print *,lexist
      if(.not.lexist) then
        go to 1010
      endif  
c
      open (12,err=1030,file=fship,form='unformatted')
c
      read 1,fshipc
      print *,'unit 52  '
      print *, 'open  ',fshipc
      open (52,err=1050,file=fshipc,form='unformatted')
c
      read (12,end=1000) iyr,imo,ida,type       
c
      print 260,iyr,imo,ida,type         
  260 format ('date = ',i5,2i3,3x,'type = ',a8)   
      write (52) iyr,imo,ida,type           
      amin = +1.e30                                                     
      amax = -amin                                                      
      cmin = +1.e30                                                     
      cmax = -amin                                                      
      ncor = 0
      ntot = 0                                                      
   50 read (12,end=550) ict                                       
      write (52) ict
      if (ict.eq.0) go to 550                                               
c     print 40,type,ict                                                 
c  40 format (/'read',a10,3x,'numb super obs =',i10)                    
      read (12,end=1000) (rsum(ii),ii=1,ict)                            
      read (12,end=1000) (rsst(ii),ii=1,ict)                            
      read (12,end=1000) (rlat(ii),ii=1,ict)                            
      read (12,end=1000) (rlon(ii),ii=1,ict)                            
      do 30 l=1,ict                                                     
        ncor = ncor + 1
	ntot = ntot + nint(rsum(l))
        if (amax.lt.rsst(l)) amax = rsst(l)                                    
        if (amin.gt.rsst(l)) amin = rsst(l)                                     
	rsst(l) = rsst(l) - globc
        if (cmax.lt.rsst(l)) cmax = rsst(l)                                    
        if (cmin.gt.rsst(l)) cmin = rsst(l)                                     
   30 continue                                                          
      write (52) (rsum(ii),ii=1,ict)                                    
      write (52) (rsst(ii),ii=1,ict)                                    
      write (52) (rlat(ii),ii=1,ict)                                    
      write (52) (rlon(ii),ii=1,ict)                                    
      go to 50                                                          
  550 continue
      if (ncor.gt.0) then
        print 735,amax,amin,ntot                                               
  735   format ('ship initial:  max =',f8.2,3x,'min =',f8.2,3x,
     1 'number =',i7)     
        print 736,cmax,cmin                                              
  736   format ('ship final  :  max =',f8.2,3x,'min =',f8.2)     
        print *,'GREEN - OK - shipbias4: ship bias computed'
        call exit (1)
      else
       print *,'YELLOW Warning shipbias4: NO ship data to correct'
       call exit (10)       
      endif
 1000 print *,'RED - STOP - shipbias4: premature end of ship data',
     1  ' NO ship bias computed'
      call exit (100)         
c
 1010 PRINT *,'RED - STOP - shipbias4: Could not open existing ',
     1 'ship input file'
      call exit (100)
c
 1030 PRINT *,'RED - STOP - shipbias4: error in ship input file'
      call exit (100)
c
 1050 PRINT *,'RED - STOP - shipbias4: error in ship OUTPUT file ',
     1 'input file'
      call exit (100)
c
      END
